@@ -12,6 +12,7 @@
 
 #include "ActionHandler.hh"
 
+#include "Debug.hh"
 #include "PWinObj.hh"
 #include "PDecor.hh"
 #include "PMenu.hh"
@@ -33,18 +34,11 @@
 #include "WORefMenu.hh"
 #include "ActionMenu.hh"
 #include "FrameListMenu.hh"
-#include "DecorMenu.hh"
 
 #include <memory>
 
-#ifdef DEBUG
-#include <iostream>
-using std::cerr;
-using std::endl;
-#endif // DEBUG
 using std::auto_ptr;
 using std::string;
-using std::list;
 using std::find;
 using std::map;
 
@@ -80,7 +74,7 @@ ActionHandler::handleAction(const ActionPerformed &ap)
     bool matched = false;
 
     // go through the list of actions and execute them
-    list<Action>::const_iterator it = ap.ae.action_list.begin();
+    vector<Action>::const_iterator it = ap.ae.action_list.begin();
     for (; it != ap.ae.action_list.end(); ++it, matched = false) {
         // Determine what type if any of the window object that is focused
         // and check if it is still alive.
@@ -149,6 +143,9 @@ ActionHandler::handleAction(const ActionPerformed &ap)
             case ACTION_KILL:
                 client->kill();
                 break;
+            case ACTION_SET_GEOMETRY:
+                frame->setGeometry(it->getParamS(), it->getParamI(0));
+                break;
             case ACTION_RAISE:
                 if (it->getParamI(0)) {
                     WindowManager::instance()->familyRaiseLower(client, true);
@@ -176,6 +173,9 @@ ActionHandler::handleAction(const ActionPerformed &ap)
                     wo->giveInputFocus();
                 }
                 break;
+            case ACTION_MOVE_TO_HEAD:
+                frame->moveToHead(it->getParamI(0));
+                break;
             case ACTION_MOVE_TO_EDGE:
                 frame->moveToEdge(OrientationType(it->getParamI(0)));
                 break;
@@ -330,7 +330,7 @@ ActionHandler::handleAction(const ActionPerformed &ap)
                                ap.type, client ? client : wo);
                 break;
             case ACTION_HIDE_ALL_MENUS:
-                MenuHandler::instance()->hideAllMenus();
+                MenuHandler::hideAllMenus();
                 break;
             case ACTION_RELOAD:
                 WindowManager::instance()->reload();
@@ -346,23 +346,19 @@ ActionHandler::handleAction(const ActionPerformed &ap)
                 WindowManager::instance()->shutdown();
                 break;
             case ACTION_SHOW_CMD_DIALOG:
-                if (WindowManager::instance()->getCmdDialog()->isMapped()) {
-                    WindowManager::instance()->getCmdDialog()->unmapWindow();
-                } else {
-                    WindowManager::instance()->getCmdDialog()->mapCentered(it->getParamS(), true,
-                                                     frame ? frame : wo);
-                }
+                actionShowInputDialog(WindowManager::instance()->getCmdDialog(), it->getParamS(), frame, wo);
                 break;
             case ACTION_SHOW_SEARCH_DIALOG:
-                if (WindowManager::instance()->getSearchDialog()->isMapped()) {
-                    WindowManager::instance()->getSearchDialog()->unmapWindow();
-                } else {
-                    WindowManager::instance()->getSearchDialog()->mapCentered(it->getParamS(), true, frame ? frame : wo);
-                } 
+                actionShowInputDialog(WindowManager::instance()->getSearchDialog(), it->getParamS(), frame, wo);
                 break;
             case ACTION_HIDE_WORKSPACE_INDICATOR:
                 WindowManager::instance()->getWorkspaceIndicator()->unmapWindow();
                 break;
+            case ACTION_DEBUG:
+#ifdef DEBUG
+                Debug::doAction(it->getParamS());
+#endif
+                break;
             default:
                 matched = false;
                 break;
@@ -511,7 +507,7 @@ ActionHandler::checkAEThreshold(int x, int y, int x_t, int y_t, uint t)
 
 //! @brief Searches the actions list for an matching event
 ActionEvent*
-ActionHandler::findMouseAction(uint button, uint state, MouseEventType type, std::list<ActionEvent> *actions)
+ActionHandler::findMouseAction(uint button, uint state, MouseEventType type, vector<ActionEvent> *actions)
 {
     if (! actions) {
         return 0;
@@ -520,7 +516,7 @@ ActionHandler::findMouseAction(uint button, uint state, MouseEventType type, std
     X11::stripStateModifiers(&state);
     X11::stripButtonModifiers(&state);
 
-    list<ActionEvent>::iterator it(actions->begin());
+    vector<ActionEvent>::iterator it(actions->begin());
     for (; it != actions->end(); ++it) {
         if ((it->type == unsigned(type))
             && ((it->mod == MOD_ANY) || (it->mod == state))
@@ -672,7 +668,7 @@ ActionHandler::actionFocusToggle(uint button, uint raise, int off,
     if (PWinObj::isFocusedPWinObj(PWinObj::WO_CLIENT)) {
         fo_wo = PWinObj::getFocusedPWinObj()->getParent();
 
-        list<PMenu::Item*>::iterator it(menu->m_begin());
+        vector<PMenu::Item*>::const_iterator it(menu->m_begin());
         for (; it != menu->m_end(); ++it) {
             if ((*it)->getWORef() == fo_wo) {
                 menu->selectItem(it);
@@ -846,7 +842,7 @@ void
 ActionHandler::actionShowMenu(const std::string &name, bool stick,
                               uint e_type, PWinObj *wo_ref)
 {
-    PMenu *menu = MenuHandler::instance()->getMenu(name);
+    PMenu *menu = MenuHandler::getMenu(name);
     if (! menu) {
         return;
     }
@@ -880,6 +876,25 @@ ActionHandler::actionShowMenu(const std::string &name, bool stick,
     }
 }
 
+void
+ActionHandler::actionShowInputDialog(InputDialog *dialog,
+                                     const std::string &initial, Frame *frame, PWinObj *wo)
+{
+    if (dialog->isMapped()) {
+        dialog->unmapWindow();
+    } else {
+        Geometry gm;
+        if (frame) {
+            frame->getGeometry(gm);
+        } else {
+            uint head = X11::getCurrHead();
+            X11::getHeadInfo(head, gm);
+        }
+
+        dialog->mapCentered(initial, gm, frame ? frame : wo);
+    }
+}
+
 //! @brief Creates a menu containing a list of Frames currently visible
 //! @param show_iconified Flag to show/hide iconified windows
 //! @param mru Whether MRU order should be used or not.
@@ -893,8 +908,8 @@ ActionHandler::createNextPrevMenu(bool show_iconified, bool mru)
                   mru?L"MRU Windows":L"Windows",
                   "" /* Empty name*/);
 
-    list<Frame*>::iterator itr = mru?WindowManager::instance()->mru_begin():Frame::frame_begin();
-    list<Frame*>::iterator end = mru?WindowManager::instance()->mru_end():Frame::frame_end();
+    vector<Frame*>::const_iterator itr = mru?WindowManager::instance()->mru_begin():Frame::frame_begin();
+    vector<Frame*>::const_iterator end = mru?WindowManager::instance()->mru_end():Frame::frame_end();
     for (; itr != end; ++itr) {
         fr = *itr;
         if (createMenuInclude(fr, show_iconified)) {
@@ -937,8 +952,8 @@ ActionHandler::findClientFromTitle(const std::wstring &or_title)
     RegexString o_rs;
 
     if (o_rs.parse_match(or_title, true)) {
-        list<Client*>::iterator it (Client::client_begin());
-        for (; it != Client::client_end (); ++it) {
+        vector<Client*>::const_iterator it(Client::client_begin());
+        for (; it != Client::client_end(); ++it) {
             if (o_rs == (*it)->getTitle()->getVisible()) {
                 return (*it);
             }
@@ -955,11 +970,7 @@ ActionHandler::gotoClient(Client *client)
 {
     Frame *frame = dynamic_cast<Frame*>(client->getParent());
     if (! frame) {
-#ifdef DEBUG
-        cerr << __FILE__ << "@" << __LINE__ << ": "
-             << "ActionHandler(" << this << ")::gotoClient(" << client << ")"
-             << endl << " *** parent is not a Frame!" << endl;
-#endif // DEBUG
+        WARN("parent is not a Frame");
         return;
     }
 
@@ -967,6 +978,7 @@ ActionHandler::gotoClient(Client *client)
     if (! frame->isSticky()
         && (frame->getWorkspace() != Workspaces::getActive())) {
         Workspaces::setWorkspace(frame->getWorkspace(), false);
+        WindowManager::instance()->addToMRUFront(frame);
     }
 
     if (! frame->isMapped()) {
@@ -980,7 +992,7 @@ ActionHandler::gotoClient(Client *client)
 }
 
 /**
- * Initilalize XEvent for sending KeyPress/KeyRelease events.
+ * Initialize XEvent for sending KeyPress/KeyRelease events.
  *
  * @param ev Reference to event.
  * @param wo PWinObj key event is aimed for.
