@@ -11,7 +11,6 @@
 #endif // HAVE_CONFIG_H
 
 #include <algorithm>
-#include <vector>
 #include <iostream>
 
 #include "AutoProperties.hh"
@@ -21,10 +20,8 @@
 using std::cerr;
 using std::endl;
 using std::find;
-using std::list;
 using std::map;
 using std::string;
-using std::vector;
 using std::strtol;
 
 AutoProperties *AutoProperties::_instance = 0;
@@ -109,7 +106,7 @@ bool
 AutoProperties::load(void)
 {
     string cfg_file(Config::instance()->getAutoPropsFile());
-    if (! Util::requireReload(_cfg_state, cfg_file)) {
+    if (! _cfg_files.requireReload(cfg_file)) {
         return false;
     }
 
@@ -129,9 +126,9 @@ AutoProperties::load(void)
     loadRequire(a_cfg, cfg_file);
 
     if (a_cfg.is_dynamic_content()) {
-        _cfg_state.clear();
+        _cfg_files.clear();
     } else {
-        _cfg_state = a_cfg.get_file_list();
+        _cfg_files = a_cfg.getCfgFiles();
     }
 
     // reset values
@@ -139,7 +136,7 @@ AutoProperties::load(void)
 
     vector<string> tokens;
     vector<string>::iterator token_it;
-    list<uint> workspaces;
+    vector<uint> workspaces;
 
     CfgParser::iterator it(a_cfg.get_entry_root()->begin());
     for (; it != a_cfg.get_entry_root()->end(); ++it) {
@@ -155,16 +152,11 @@ AutoProperties::load(void)
             parseDockAppProperty(*it);
         } else if (*(*it) == "WORKSPACE") { // Workspace section
             CfgParser::Entry *workspace = (*it)->get_section();
-            CfgParser::Entry *value = workspace->find_entry("WORKSPACE");
-            if (! value) {
-                continue; // Need workspace numbers.
-            }
-            
             tokens.clear();
-            if (Util::splitString(value->get_value(), tokens, " \t")) {
+            if (Util::splitString(workspace->get_value(), tokens, " \t")) {
                 workspaces.clear();
                 for (token_it = tokens.begin(); token_it != tokens.end(); ++token_it)
-                    workspaces.push_back(strtol(token_it->c_str(), 0, 10) - 1);
+                    workspaces.push_back(strtol(token_it->c_str(), 0, 10));
 
                 // Get all properties on for these workspaces.
                 CfgParser::iterator workspace_it(workspace->begin());
@@ -192,11 +184,11 @@ AutoProperties::loadRequire(CfgParser &a_cfg, std::string &file)
     // Look for requires section, 
     section = a_cfg.get_entry_root()->find_section("REQUIRE");
     if (section) {
-        list<CfgParserKey*> key_list;
+        vector<CfgParserKey*> keys;
 
-        key_list.push_back(new CfgParserKeyBool("TEMPLATES", _extended, false));
-        section->parse_key_values(key_list.begin(), key_list.end());
-        for_each(key_list.begin(), key_list.end(), Util::Free<CfgParserKey*>());
+        keys.push_back(new CfgParserKeyBool("TEMPLATES", _extended, false));
+        section->parse_key_values(keys.begin(), keys.end());
+        for_each(keys.begin(), keys.end(), Util::Free<CfgParserKey*>());
 
         // Re-load configuration with templates enabled.
         if (_extended) {
@@ -249,7 +241,7 @@ AutoProperties::unload(void)
 //! @brief Finds a property from the prop_list
 Property*
 AutoProperties::findProperty(const ClassHint* class_hint,
-                             std::vector<Property*>* prop_list, int ws, ApplyOn type)
+                             vector<Property*>* prop_list, uint ws, ApplyOn type)
 {
     // Allready remove apply on start
     if (! _apply_on_start && (type == APPLY_ON_START))
@@ -257,8 +249,6 @@ AutoProperties::findProperty(const ClassHint* class_hint,
 
     vector<Property*>::const_iterator it(prop_list->begin());
     vector<Property*>::const_iterator end(prop_list->end());
-    list<uint>::iterator w_it;
-
     // start searching for a suitable property
     for (; it != end; ++it) {
         // see if the type matches, if we have one
@@ -266,17 +256,7 @@ AutoProperties::findProperty(const ClassHint* class_hint,
             continue;
 
         if (matchAutoClass(*class_hint, *it)) {
-
-            // make sure it applies on the correct workspace
-            if (! (*it)->getWsList().empty()) {
-                w_it = find((*it)->getWsList().begin(), (*it)->getWsList().end(),
-                            unsigned(ws));
-                if (w_it != (*it)->getWsList().end()) {
-                  return *it;
-                }
-            } else {
-                return *it;
-            }
+            return (*it)->applyOnWs(ws)?*it:0;
         }
     }
 
@@ -301,17 +281,16 @@ AutoProperties::parseRegexpOrWarning(RegexString &regex, const std::string regex
 //! @brief Parses a property match rule
 //! @param str String to parse.
 //! @param prop Property to place result in.
-//! @param extended Extended syntax including role and title in the name, defaults to true.
 //! @return true on success, else false.
 bool
-AutoProperties::parsePropertyMatch(const std::string &str, Property *prop, bool extended)
+AutoProperties::parsePropertyMatch(const std::string &str, Property *prop)
 {
     bool status = false;
 
     // Format of property matches are regexp,regexp . Split up in class
     // and role regexps.
     vector<string> tokens;
-    Util::splitString(str, tokens, ",", extended ? 5 : 2, true);
+    Util::splitString(str, tokens, ",", _extended ? 5 : 2, true);
 
     if (tokens.size() >= 2) {
         // Make sure one of the two regexps compiles
@@ -320,7 +299,7 @@ AutoProperties::parsePropertyMatch(const std::string &str, Property *prop, bool
     }
 
     // Parse extended part of regexp, role, title and apply on
-    if (status && extended) {
+    if (status && _extended) {
         if (status && tokens.size() > 2) {
             status = parseRegexpOrWarning(prop->getRole(), tokens[2], "role");
         }
@@ -342,17 +321,17 @@ AutoProperties::parseProperty(CfgParser::Entry *section, Property *prop)
     CfgParser::Entry *value;
 
     // Get extra matching info.
-    value = section->find_entry ("TITLE");
+    value = section->find_entry("TITLE");
     if (value) {
         parseRegexpOrWarning(prop->getTitle(), value->get_value(), "title");
     }
-    value = section->find_entry ("ROLE");
+    value = section->find_entry("ROLE");
     if (value) {
         parseRegexpOrWarning(prop->getRole(), value->get_value(), "role");
     }
 
     // Parse apply on mask.
-    value = section->find_entry ("APPLYON");
+    value = section->find_entry("APPLYON");
     if (value) {
         parsePropertyApplyOn(value->get_value(), prop);
     }
@@ -367,7 +346,7 @@ void
 AutoProperties::parsePropertyApplyOn(const std::string &apply_on, Property *prop)
 {
     vector<string> tokens;
-    if ((Util::splitString(apply_on, tokens, " \t", 5))) {
+    if (Util::splitString(apply_on, tokens, " \t", 5)) {
         vector<string>::iterator it(tokens.begin());
         for (; it != tokens.end(); ++it) {
             prop->applyAdd(static_cast<unsigned int>(ParseUtil::getValue<ApplyOn>(*it, _apply_on_map)));
@@ -377,7 +356,7 @@ AutoProperties::parsePropertyApplyOn(const std::string &apply_on, Property *prop
 
 //! @brief Parses AutopProperty
 void
-AutoProperties::parseAutoProperty(CfgParser::Entry *section, std::list<uint>* ws)
+AutoProperties::parseAutoProperty(CfgParser::Entry *section, vector<uint> *ws)
 {
     // Get sub section
     section = section->get_section();
@@ -386,12 +365,11 @@ AutoProperties::parseAutoProperty(CfgParser::Entry *section, std::list<uint>* ws
     }
 
     AutoProperty* property = new AutoProperty();
-    parsePropertyMatch(section->get_value(), property, _extended);
+    parsePropertyMatch(section->get_value(), property);
 
     if (parseProperty(section, property)) {
         parseAutoPropertyValue(section, property, ws);
         _prop_list.push_back(property);
-
     } else {
         delete property;
     }
@@ -457,7 +435,7 @@ AutoProperties::parseTitleProperty(CfgParser::Entry *section)
         }
 
         title_property = new TitleProperty();
-        parsePropertyMatch(title_section->get_value(), title_property, _extended);
+        parsePropertyMatch(title_section->get_value(), title_property);
         if (parseProperty(title_section, title_property)) {
             CfgParser::Entry *value = title_section->find_entry("RULE");
             if (value && title_property->getTitleRule().parse_ed_s(Util::to_wide_str(value->get_value()))) {
@@ -491,7 +469,7 @@ AutoProperties::parseDecorProperty(CfgParser::Entry *section)
         }
 
         decor_property = new DecorProperty();
-        parsePropertyMatch(decor_section->get_value (), decor_property, _extended);
+        parsePropertyMatch(decor_section->get_value (), decor_property);
         if (parseProperty(decor_section, decor_property)) {
             CfgParser::Entry *value = decor_section->find_entry("DECOR");
             if (value) {
@@ -530,7 +508,7 @@ AutoProperties::parseDockAppProperty(CfgParser::Entry *section)
         }
 
         dock_property = new DockAppProperty();
-        parsePropertyMatch(dock_section->get_value(), dock_property, _extended);
+        parsePropertyMatch(dock_section->get_value(), dock_property);
         if (parseProperty(dock_section, dock_property)) {
             CfgParser::Entry *value = dock_section->find_entry("POSITION");
             if (value) {
@@ -702,11 +680,11 @@ AutoProperties::setDefaultTypeProperties(void)
 //! @param prop Property to store result in.
 //! @param ws List of workspaces to apply property on.
 void
-AutoProperties::parseAutoPropertyValue(CfgParser::Entry *section, AutoProperty *prop, std::list<uint> *ws)
+AutoProperties::parseAutoPropertyValue(CfgParser::Entry *section, AutoProperty *prop, vector<uint> *ws)
 {
     // Copy workspaces, if any
     if (ws) {
-        prop->getWsList().assign(ws->begin(), ws->end());
+        prop->setWorkspaces(*ws);
     }
 
     // See if we have a group section
