@@ -15,21 +15,51 @@
 #include <cassert>
 #include <cstring>
 
+#include <sys/types.h>
+#include <sys/stat.h>
+#include <unistd.h>
+#include <time.h>
+
 enum {
     PARSE_BUF_SIZE = 1024
 };
 
 using std::cerr;
 using std::endl;
-using std::list;
+using std::vector;
 using std::map;
 using std::set;
 using std::string;
-using std::auto_ptr;
 
 const string CfgParser::_root_source_name = string("");
 const char *CP_PARSE_BLANKS = " \t\n";
 
+bool
+TimeFiles::requireReload(const std::string &file)
+{
+    // Check for the file, signal reload if not previously loaded.
+    vector<std::string>::const_iterator it(find(files.begin(), files.end(), file));
+    if (it == files.end()) {
+        return true;
+    }
+
+    struct stat stat_buf;
+    // Check state of all files, if one is updated reload.
+    for (it = files.begin(); it != files.end(); ++it) {
+        if (stat((*it).c_str(), &stat_buf))
+            stat_buf.st_mtime = 0;
+
+        if (stat_buf.st_mtime > mtime) {
+            if (time(0) > stat_buf.st_mtime) {
+                mtime = stat_buf.st_mtime;
+            }
+            return true;
+        }
+    }
+
+    return false;
+}
+
 //! @brief CfgParser::Entry constructor.
 CfgParser::Entry::Entry(const std::string &source_name, int line,
                         const std::string &name, const std::string &value,
@@ -48,7 +78,7 @@ CfgParser::Entry::Entry(const CfgParser::Entry &entry)
       _name(entry._name), _value(entry._value),
       _line(entry._line), _source_name(entry._source_name)
 {
-    list<CfgParser::Entry*>::const_iterator it(entry._entries.begin());
+    vector<CfgParser::Entry*>::const_iterator it(entry._entries.begin());
     for (; it != entry._entries.end(); ++it) {
         _entries.push_back(new Entry(*(*it)));
     }
@@ -139,7 +169,7 @@ CfgParser::Entry*
 CfgParser::Entry::find_entry(const std::string &name, bool include_sections, const char *value)
 {
     CfgParser::Entry *value_check;
-    list<CfgParser::Entry*>::iterator it(_entries.begin());
+    vector<CfgParser::Entry*>::iterator it(_entries.begin());
     for (; it != _entries.end(); ++it) {
         value_check = include_sections ? (*it)->get_section() : (*it);
 
@@ -158,7 +188,7 @@ CfgParser::Entry::find_entry(const std::string &name, bool include_sections, con
 CfgParser::Entry*
 CfgParser::Entry::find_section(const std::string &name, const char *value)
 {
-    list<CfgParser::Entry*>::iterator it(_entries.begin());
+    vector<CfgParser::Entry*>::iterator it(_entries.begin());
     for (; it != _entries.end(); ++it) {
         if ((*it)->get_section() && *(*it) == name.c_str()
             && (! value || (*it)->get_section()->get_value() == value)) {
@@ -172,11 +202,11 @@ CfgParser::Entry::find_section(const std::string &name, const char *value)
 
 //! @brief Sets and validates data specified by key list.
 void
-CfgParser::Entry::parse_key_values(std::list<CfgParserKey*>::iterator begin,
-                                   std::list<CfgParserKey*>::iterator end)
+CfgParser::Entry::parse_key_values(std::vector<CfgParserKey*>::const_iterator begin,
+                                   std::vector<CfgParserKey*>::const_iterator end)
 {
     CfgParser::Entry *value;
-    list<CfgParserKey*>::iterator it;
+    vector<CfgParserKey*>::const_iterator it;
 
     for (it = begin; it != end; ++it) {
         value = find_entry((*it)->get_name());
@@ -290,10 +320,10 @@ CfgParser::clear(bool realloc)
     _overwrite = false;
 
     // Clear lists
-    _source_list.clear();
-    _source_name_list.clear();
+    _sources.clear();
+    _source_names.clear();
     _source_name_set.clear();
-    _section_list.clear();
+    _sections.clear();
     _var_map.clear();
 
     // Remove sections
@@ -323,13 +353,13 @@ CfgParser::parse(const std::string &src, CfgParserSource::Type type, bool overwr
 
     // Open initial source.
     parse_source_new(src, type);
-    if (_source_list.size() == 0) {
+    if (_sources.size() == 0) {
         return false;
     }
 
     int c, next;
-    while (_source_list.size()) {
-        _source = _source_list.back();
+    while (_sources.size()) {
+        _source = _sources.back();
         if (_source->is_dynamic()) {
             _is_dynamic_content = true;
         }
@@ -358,21 +388,21 @@ CfgParser::parse(const std::string &src, CfgParserSource::Type type, bool overwr
                 value.clear();
                 break;
             case '}':
-                if (_section_list.size() > 0) {
+                if (_sections.size() > 0) {
                     if (buf.size() && parse_name(buf)) {
                         parse_entry_finish(buf, value);
                         buf.clear();
                         value.clear();
                     }
-                    _section = _section_list.back();
-                    _section_list.pop_back();
+                    _section = _sections.back();
+                    _sections.pop_back();
                 } else {
                     cerr << "Extra } character found, ignoring." << endl;
                 }
                 break;
             case '=':
                 value.clear();
-                parse_value(_source, value);
+                parse_value(value);
                 break;
             case '#':
                 parse_comment_line(_source);
@@ -401,8 +431,8 @@ CfgParser::parse(const std::string &src, CfgParserSource::Type type, bool overwr
             cerr << ex << endl;
         }
         delete _source;
-        _source_list.pop_back();
-        _source_name_list.pop_back();
+        _sources.pop_back();
+        _source_names.pop_back();
     }
 
     if (buf.size()) {
@@ -426,19 +456,24 @@ CfgParser::parse_source_new(const std::string &name_orig, CfgParserSource::Type
         // Open and set as active, delete if fails.
         try {
             source->open();
+            time_t time;
             // Add source to file list if file
             if (type == CfgParserSource::SOURCE_FILE) {
-                _file_list[name] = Util::getMtime(name);
+                time = Util::getMtime(name);
+                _cfg_files.files.push_back(name);
+                if (_cfg_files.mtime < time) {
+                    _cfg_files.mtime = time;
+                }
             }
 
             _source = source;
-            _source_list.push_back(_source);
+            _sources.push_back(_source);
             done = 1;
 
         } catch (string &ex) {
             delete source;
             // Previously added in source_new
-            _source_name_list.pop_back();
+            _source_names.pop_back();
 
 
             // Display error message on second try
@@ -449,8 +484,8 @@ CfgParser::parse_source_new(const std::string &name_orig, CfgParserSource::Type
             // If the open fails and we are trying to open a file, try
             // to open the file from the current files directory.
             if (! done && (type == CfgParserSource::SOURCE_FILE)) {
-                if (_source_name_list.size() && (name[0] != '/')) {
-                    name = Util::getDir(_source_name_list.back());
+                if (_source_names.size() && name[0] != '/') {
+                    name = Util::getDir(_source_names.back());
                     name += "/" + name_orig;
                 }
             }
@@ -488,26 +523,26 @@ CfgParser::parse_name(std::string &buf)
     return true;
 }
 
-//! @brief Parses from = to end of " pair.
+//! @brief Parses _source after = to end of " pair.
 void
-CfgParser::parse_value(CfgParserSource *source, std::string &value)
+CfgParser::parse_value(std::string &value)
 {
     // We expect to get a " after the =, however we ignore anything else.
     int c;
-    while ((c = source->getc()) != EOF && c != '"')
+    while ((c = _source->getc()) != EOF && c != '"')
          ;
 
-    // Check if we got to a " or found EOF first.
+    // Check if we got EOF before getting a quotation mark.
     if (c == EOF) {
         cerr << "Reached EOF before opening \" in value." << endl;
         return;
     }
 
     // Parse until next ", and escape characters after \.
-    while ((c = source->getc()) != EOF && c != '"') {
+    while ((c = _source->getc()) != EOF && c != '"') {
         // Escape character after \, if newline drop it.
         if (c == '\\') {
-            c = source->getc();
+            c = _source->getc();
             if (c == '\n' || c == EOF) {
                 continue;
             }
@@ -612,7 +647,7 @@ CfgParser::parse_section_finish(std::string &buf, std::string &value)
     }
 
     // Set current Entry to newly created Section.
-    _section_list.push_back(_section);
+    _sections.push_back(_section);
     _section = section;
 }
 
@@ -670,7 +705,7 @@ CfgParser::source_new(const std::string &name, CfgParserSource::Type type)
     CfgParserSource *source = 0;
 
     // Create CfgParserSource.
-    _source_name_list.push_back(name);
+    _source_names.push_back(name);
     _source_name_set.insert(name);
     switch (type) {
     case CfgParserSource::SOURCE_FILE:
