@@ -1,6 +1,6 @@
 //
 // Client.cc for pekwm
-// Copyright © 2002-2009 Claes Nästén <me{@}pekdon{.}net>
+// Copyright © 2002-2013 Claes Nästén <me@pekdon.net>
 //
 // client.cc for aewm++
 // Copyright (C) 2000 Frank Hale <frankhale@yahoo.com>
@@ -16,6 +16,7 @@
 
 #include <cstdio>
 #include <iostream>
+#include <algorithm>
 
 extern "C" {
 #include <X11/Xlib.h>
@@ -27,6 +28,7 @@ extern "C" {
 }
 
 #include "Compat.hh" // setenv, unsetenv
+#include "Debug.hh"
 #include "PWinObj.hh"
 #include "PDecor.hh" // PDecor::TitleItem
 #include "Client.hh"
@@ -48,24 +50,23 @@ extern "C" {
 using std::cerr;
 using std::endl;
 using std::find;
-using std::list;
 using std::string;
 using std::vector;
 using std::wstring;
 
-list<Client*> Client::_client_list = list<Client*>();
-vector<uint> Client::_clientid_list = vector<uint>();
+vector<Client*> Client::_clients;
+vector<uint> Client::_clientids;
 
 Client::Client(Window new_client, bool is_new)
     : PWinObj(),
       _id(0), _size(0),
-      _transient(None), _strut(0), _icon(0),
+      _transient_for(0), _strut(0), _icon(0),
       _pid(0), _is_remote(false), _class_hint(0),
       _window_type(WINDOW_TYPE_NORMAL),
       _alive(false), _marked(false),
       _send_focus_message(false), _send_close_message(false),
       _wm_hints_input(true), _cfg_request_lock(false),
-      _shaped(false), _extended_net_name(false),
+      _extended_net_name(false),
       _demands_attention(false)
 {
     // PWinObj attributes, required by validate etc.
@@ -84,11 +85,31 @@ Client::Client(Window new_client, bool is_new)
     _title.setId(_id);
     _title.infoAdd(PDecor::TitleItem::INFO_ID);
 
-    if (X11::hasExtensionShape()) {
 #ifdef HAVE_SHAPE
+    if (X11::hasExtensionShape()) {
         XShapeSelectInput(X11::getDpy(), _window, ShapeNotifyMask);
-#endif // HAVE_SHAPE
+
+        int isShaped;
+        X11::shapeQuery(_window, &isShaped);
+        _shape_bounding = isShaped;
+
+        int num=0; int foo;
+        XRectangle *rect = XShapeGetRectangles(X11::getDpy(), _window, ShapeInput, &num, &foo);
+        if (rect) {
+            if (num == 1) {
+                unsigned w, h, bw;
+                X11::getGeometry(_window, &w, &h, &bw);
+                if (rect->x > 0 || rect->y > 0 || rect->width < w+bw
+                                || rect->height < h+bw) {
+                   _shape_input = true;
+                }
+            } else {
+                _shape_input = true;
+            }
+            XFree(rect);
+        }
     }
+#endif // HAVE_SHAPE
 
     XAddToSaveSet(X11::getDpy(), _window);
     XSetWindowBorderWidth(X11::getDpy(), _window, 0);
@@ -100,7 +121,6 @@ Client::Client(Window new_client, bool is_new)
 
     getWMNormalHints();
     readName();
-    readIconName();
 
     // cyclic dependency, getting the name requires quiering autoprops
     _class_hint->title = _title.getReal();
@@ -139,20 +159,21 @@ Client::Client(Window new_client, bool is_new)
     // Finished creating the client, so now adding it to the client list.
     woListAdd(this);
     _wo_map[_window] = this;
-    _client_list.push_back(this);
+    _clients.push_back(this);
 }
 
 //! @brief Client destructor
 Client::~Client(void)
 {
     // Remove from lists
-    if (_transient) {
-        _transient->_transient_clients.remove(this);
-        _transient->removeObserver(this);
+    if (_transient_for) {
+        vector<Client *> &tc(_transient_for->_transients);
+        tc.erase(std::remove(tc.begin(), tc.end(), this), tc.end());
+        _transient_for->removeObserver(this);
     }
     _wo_map.erase(_window);
     woListRemove(this);
-    _client_list.remove(this);
+    _clients.erase(std::remove(_clients.begin(), _clients.end(), this), _clients.end());
 
     returnClientID(_id);
 
@@ -166,9 +187,9 @@ Client::~Client(void)
     }
 
     // Focus the parent if we had focus before
-    if (focus && _transient) {
-        Frame *trans_frame = static_cast<Frame*>(_transient->getParent());
-        if (trans_frame->getActiveChild() == _transient) {
+    if (focus && _transient_for) {
+        Frame *trans_frame = static_cast<Frame*>(_transient_for->getParent());
+        if (trans_frame->getActiveChild() == _transient_for) {
             trans_frame->giveInputFocus();
         }
     }
@@ -298,7 +319,7 @@ Client::findPreviousFrame(void)
     }
 
     long id;
-    if (AtomUtil::getLong(_window, Atoms::getAtom(PEKWM_FRAME_ID), id)) {
+    if (X11::getLong(_window, PEKWM_FRAME_ID, id)) {
         _parent = Frame::findFrameFromID(id);
         if (_parent) {
             Frame *frame = static_cast<Frame*>(_parent);
@@ -384,8 +405,8 @@ Client::setMappedStateAndFocus(bool is_new, AutoProperty *autoproperty)
             _parent->giveInputFocus();
 
             // Check if we are transient, and if we want to focus
-        } else if (_transient && _transient->isFocused()
-                   && Config::instance()->isFocusNewChild()) {
+        } else if (_transient_for && _transient_for->isFocused()
+                                  && Config::instance()->isFocusNewChild()) {
             _parent->giveInputFocus();
         }
     }
@@ -398,15 +419,15 @@ Client::setMappedStateAndFocus(bool is_new, AutoProperty *autoproperty)
 void
 Client::findAndRaiseIfTransient(void)
 {
-    if (_transient_window != None && ! _transient) {
+    if (_transient_for_window != None && ! _transient_for) {
         getTransientForHint();
     }
 
-    if (_transient) {
-        Frame *frame = static_cast<Frame*>(getParent());
-        Frame *frame_transient = static_cast<Frame*>(_transient->getParent());
-        if (frame->getActiveChild() == this) {
-            Workspaces::stackAbove(this, frame_transient->getWindow());
+    if (_transient_for) {
+        // Ensure layer is at least as high as the parent
+        if (_transient_for->getLayer() > getLayer()) {
+            setLayer(_transient_for->getLayer());
+            updateParentLayerAndRaiseIfActive();
         }
     }
 }
@@ -427,7 +448,7 @@ Client::mapWindow(void)
         updateEwmhStates();
     }
 
-    if(! _transient) {
+    if(! _transient_for) {
         // Unmap our transient windows if we have any
         mapOrUnmapTransients(_window, false);
     }
@@ -468,7 +489,7 @@ Client::iconify(void)
     }
 
     _iconified = true;
-    if (! _transient) {
+    if (! _transient_for) {
         mapOrUnmapTransients(_window, true);
     }
 
@@ -538,9 +559,9 @@ Client::setWorkspace(uint workspace)
         _workspace = workspace;
 
         if (_sticky) {
-            AtomUtil::setLong(_window, Atoms::getAtom(NET_WM_DESKTOP), NET_WM_STICKY_WINDOW);
+            X11::setLong(_window, NET_WM_DESKTOP, NET_WM_STICKY_WINDOW);
         } else {
-            AtomUtil::setLong(_window, Atoms::getAtom(NET_WM_DESKTOP), _workspace);
+            X11::setLong(_window, NET_WM_DESKTOP, _workspace);
         }
     }
 }
@@ -574,6 +595,15 @@ Client::reparent(PWinObj *parent, int x, int y)
                          PropertyChangeMask|StructureNotifyMask|FocusChangeMask);
 }
 
+ActionEvent*
+Client::handleMapRequest(XMapRequestEvent *ev)
+{
+       if (_parent && dynamic_cast<PDecor *>(_parent)) {
+           dynamic_cast<PDecor*>(_parent)->deiconify();
+       }
+       return 0;
+}
+
 ActionEvent*
 Client::handleUnmapEvent(XUnmapEvent *ev)
 {
@@ -587,12 +617,12 @@ Client::handleUnmapEvent(XUnmapEvent *ev)
 
     // Extended Window Manager Hints 1.3 specifies that a window manager
     // should remove the _NET_WM_STATE property when a window is withdrawn.
-    AtomUtil::unsetProperty(_window, Atoms::getAtom(STATE));
+    X11::unsetProperty(_window, STATE);
 
     // Extended Window Manager Hints 1.3 specifies that a window manager
     // should remove the _NET_WM_DESKTOP property when a window is withdrawn.
     // (to allow legacy applications to reuse a withdrawn window)
-    AtomUtil::unsetProperty(_window, Atoms::getAtom(NET_WM_DESKTOP));
+    X11::unsetProperty(_window, NET_WM_DESKTOP);
 
 #ifdef DEBUG
     cerr << __FILE__ << "@" << __LINE__ << ": "
@@ -607,17 +637,26 @@ Client::handleUnmapEvent(XUnmapEvent *ev)
     return 0;
 }
 
-ActionEvent*
-Client::handleMapRequest(XMapRequestEvent *ev)
-{
-       if (_parent && dynamic_cast<PDecor *>(_parent)) {
-           dynamic_cast<PDecor*>(_parent)->deiconify();
-       }
-       return 0;
-}
-
 // END - PWinObj interface.
 
+#ifdef HAVE_SHAPE
+void
+Client::handleShapeEvent(XShapeEvent *ev)
+{
+    if (ev->kind == ShapeBounding) {
+        _shape_bounding = ev->shaped;
+    } else if (ev->kind == ShapeInput) {
+        _shape_input = ev->shaped;
+    } else {
+        return;
+    }
+
+    if (_parent) {
+        static_cast<Frame *>(_parent)->handleShapeEvent(ev);
+    }
+}
+#endif // HAVE_SHAPE
+
 // START - Observer interface
 
 void
@@ -625,21 +664,17 @@ Client::notify(Observable *observable, Observation *observation)
 {
     if (observation) {
         LayerObservation *layer_observation = dynamic_cast<LayerObservation*>(observation);
-        if (layer_observation) {
-            setLayer(layer_observation->layer + 1);
-
-            Frame *frame = static_cast<Frame*>(getParent());
-            if (frame->getActiveChild() == this) {
-                frame->setLayer(layer_observation->layer + 1);
-                frame->raise();
-            }
+        if (layer_observation && layer_observation->layer > getLayer()) {
+            setLayer(layer_observation->layer);
+            updateParentLayerAndRaiseIfActive();
         }
     } else {
         Client *client = static_cast<Client*>(observable);
-        if (client == _transient) {
-            _transient = 0;
+        if (client == _transient_for) {
+            _transient_for = 0;
         } else {
-            _transient_clients.remove(client);
+            _transients.erase(std::remove(_transients.begin(), _transients.end(), this),
+                              _transients.end());
         }
     }
 }
@@ -657,8 +692,8 @@ Client::findClient(Window win)
         return 0;
     }
 
-    list<Client*>::iterator it(_client_list.begin());
-    for (; it != _client_list.end(); ++it) {
+    vector<Client*>::const_iterator it(_clients.begin());
+    for (; it != _clients.end(); ++it) {
         if (win == (*it)->getWindow()
             || ((*it)->getParent() && (*((*it)->getParent()) == win))) {
             return (*it);
@@ -678,8 +713,8 @@ Client::findClientFromWindow(Window win)
         return 0;
     }
 
-    list<Client*>::iterator it(_client_list.begin());
-    for(; it != _client_list.end(); ++it) {
+    vector<Client*>::const_iterator it(_clients.begin());
+    for(; it != _clients.end(); ++it) {
         if (win == (*it)->getWindow()) {
             return (*it);
         }
@@ -694,8 +729,8 @@ Client::findClientFromWindow(Window win)
 Client*
 Client::findClientFromHint(const ClassHint *class_hint)
 {
-    list<Client*>::iterator it(_client_list.begin());
-    for (; it != _client_list.end(); ++it) {
+    vector<Client*>::const_iterator it(_clients.begin());
+    for (; it != _clients.end(); ++it) {
         if (*class_hint == *(*it)->getClassHint()) {
             return *it;
         }
@@ -710,8 +745,8 @@ Client::findClientFromHint(const ClassHint *class_hint)
 Client*
 Client::findClientFromID(uint id)
 {
-    list<Client*>::iterator it(_client_list.begin());
-    for (; it != _client_list.end(); ++it) {
+    vector<Client*>::const_iterator it(_clients.begin());
+    for (; it != _clients.end(); ++it) {
         if ((*it)->getClientID() == id) {
             return *it;
         }
@@ -724,11 +759,11 @@ Client::findClientFromID(uint id)
  * Insert all clients with the transient for set to win.
  */
 void
-Client::findFamilyFromWindow(std::list<Client*> &client_list, Window win)
+Client::findFamilyFromWindow(vector<Client*> &client_list, Window win)
 {
-    list<Client*>::iterator it(Client::client_begin());
+    vector<Client*>::const_iterator it(Client::client_begin());
     for (; it != Client::client_end(); ++it) {
-        if ((*it)->getTransientClientWindow() == win) {
+        if ((*it)->getTransientForClientWindow() == win) {
             client_list.push_back(*it);
         }
     }
@@ -741,10 +776,10 @@ Client::findFamilyFromWindow(std::list<Client*> &client_list, Window win)
 void
 Client::mapOrUnmapTransients(Window win, bool hide)
 {
-    list<Client*> client_list;
+    vector<Client*> client_list;
     findFamilyFromWindow(client_list, win);
 
-    list<Client*>::iterator it(client_list.begin());
+    vector<Client*>::const_iterator it(client_list.begin());
     for (; it != client_list.end(); ++it) {
         if (static_cast<Frame*>((*it)->getParent())->getActiveChild() == *it) {
             if (hide) {
@@ -789,8 +824,8 @@ Client::grabButtons(void)
     // Make sure we don't have any buttons grabbed.
     X11::ungrabButton(_window);
 
-    list<ActionEvent> *actions = Config::instance()->getMouseActionList(MOUSE_ACTION_LIST_CHILD_FRAME);
-    list<ActionEvent>::iterator it(actions->begin());
+    vector<ActionEvent> *actions = Config::instance()->getMouseActionList(MOUSE_ACTION_LIST_CHILD_FRAME);
+    vector<ActionEvent>::const_iterator it(actions->begin());
     for (; it != actions->end(); ++it) {
         if ((it->type == MOUSE_EVENT_PRESS) || (it->type == MOUSE_EVENT_RELEASE)) {
             // No need to grab mod less events, replied with the frame
@@ -858,7 +893,7 @@ Client::readClassRoleHints(void)
 
     // wm window role
     string role;
-    AtomUtil::getString(_window, Atoms::getAtom(WM_WINDOW_ROLE), role);
+    X11::getString(_window, WM_WINDOW_ROLE, role);
 
     _class_hint->h_role = Util::to_wide_str(role);
 }
@@ -869,10 +904,10 @@ Client::readEwmhHints(void)
 {
     // which workspace do we belong to?
     long workspace = -1;
-    AtomUtil::getLong(_window, Atoms::getAtom(NET_WM_DESKTOP), workspace);
+    X11::getLong(_window, NET_WM_DESKTOP, workspace);
     if (workspace < 0) {
         _workspace = Workspaces::getActive();
-        AtomUtil::setLong(_window, Atoms::getAtom(NET_WM_DESKTOP), _workspace);
+        X11::setLong(_window, NET_WM_DESKTOP, _workspace);
     } else {
         _workspace = workspace;
     }
@@ -921,7 +956,7 @@ Client::readMwmHints(void)
 
             _actions.resize   = (mwm_hints->functions&MWM_FUNC_RESIZE)  ? state : ! state;
             _actions.move     = (mwm_hints->functions&MWM_FUNC_MOVE)    ? state : ! state;
-            _actions.minimize = (mwm_hints->functions&MWM_FUNC_ICONIFY) ? state : ! state;
+            _actions.iconify  = (mwm_hints->functions&MWM_FUNC_ICONIFY) ? state : ! state;
             _actions.close    = (mwm_hints->functions&MWM_FUNC_CLOSE)   ? state : ! state;
             _actions.maximize_vert = (mwm_hints->functions&MWM_FUNC_MAXIMIZE) ? state : ! state;
             _actions.maximize_horz = (mwm_hints->functions&MWM_FUNC_MAXIMIZE) ? state : ! state;
@@ -954,16 +989,16 @@ Client::readPekwmHints(void)
     string str;
 
     // Get decor state
-    if (AtomUtil::getLong(_window, Atoms::getAtom(PEKWM_FRAME_DECOR), value)) {
+    if (X11::getLong(_window, PEKWM_FRAME_DECOR, value)) {
         _state.decor = value;
     }
     // Get skip state
-    if (AtomUtil::getLong(_window, Atoms::getAtom(PEKWM_FRAME_SKIP), value)) {
+    if (X11::getLong(_window, PEKWM_FRAME_SKIP, value)) {
         _state.skip = value;
     }
 
     // Get custom title
-    if (AtomUtil::getString(_window, Atoms::getAtom(PEKWM_TITLE), str)) {
+    if (X11::getString(_window, PEKWM_TITLE, str)) {
         _title.setUser(Util::to_wide_str(str));
     }
 
@@ -1001,7 +1036,7 @@ AutoProperty*
 Client::readAutoprops(ApplyOn type)
 {
     AutoProperty *data =
-        AutoProperties::instance()->findAutoProperty(_class_hint, _workspace, type);
+        AutoProperties::instance()->findAutoProperty(_class_hint, Workspaces::getActive(), type);
 
     if (data) {
         // Make sure transient state matches
@@ -1075,8 +1110,8 @@ Client::applyActionAccessMask(uint mask, bool value)
     if (mask & ACTION_ACCESS_RESIZE) {
         _actions.resize = value;
     }
-    if (mask & ACTION_ACCESS_MINIMIZE) {
-        _actions.minimize = value;
+    if (mask & ACTION_ACCESS_ICONIFY) {
+        _actions.iconify = value;
     }
     if (mask & ACTION_ACCESS_SHADE) {
         _actions.shade = value;
@@ -1094,7 +1129,7 @@ Client::applyActionAccessMask(uint mask, bool value)
         _actions.fullscreen = value;
     }
     if (mask & ACTION_ACCESS_CHANGE_DESKTOP) {
-        _actions.change_desktop = value;
+        _actions.change_ws = value;
     }
     if (mask & ACTION_ACCESS_CLOSE) {
         _actions.close = value;
@@ -1107,7 +1142,7 @@ Client::applyActionAccessMask(uint mask, bool value)
 void
 Client::readClientPid(void)
 {
-    AtomUtil::getLong(_window, Atoms::getAtom(NET_WM_PID), _pid);
+    X11::getLong(_window, NET_WM_PID, _pid);
 }
 
 /**
@@ -1118,7 +1153,7 @@ void
 Client::readClientRemote(void)
 {
     string client_machine;
-    if (AtomUtil::getTextProperty(_window, XA_WM_CLIENT_MACHINE, client_machine)) {
+    if (X11::getTextProperty(_window, XA_WM_CLIENT_MACHINE, client_machine)) {
         _is_remote = Util::getHostname() != client_machine;
     }
 }
@@ -1130,13 +1165,13 @@ Client::findClientID(void)
 {
     uint id = 0;    
 
-    if (_clientid_list.size()) {
+    if (_clientids.size()) {
         // Check for used Frame IDs
-        id = _clientid_list.back();
-        _clientid_list.pop_back();
+        id = _clientids.back();
+        _clientids.pop_back();
     } else {
         // No free, get next number (Client is not in list when this is called.)
-        id = _client_list.size() + 1;
+        id = _clients.size() + 1;
     }
 
     return id;
@@ -1147,10 +1182,10 @@ Client::findClientID(void)
 void
 Client::returnClientID(uint id)
 {
-    vector<uint>::iterator it(_clientid_list.begin());
-    for (; it != _clientid_list.end() && id < *it; ++it)
+    vector<uint>::iterator it(_clientids.begin());
+    for (; it != _clientids.end() && id < *it; ++it)
         ;
-    _clientid_list.insert(it, id);
+    _clientids.insert(it, id);
 }
 
 /**
@@ -1160,27 +1195,26 @@ void
 Client::readName(void)
 {
     // Read title, bail out if it fails.
-    wstring title;
-    if (! AtomUtil::getUtf8String(_window, Atoms::getAtom(NET_WM_NAME), title)) {
-        string mb_title;
-        if (! AtomUtil::getTextProperty(_window, XA_WM_NAME, mb_title)) {
+    string title;
+    if (! X11::getUtf8String(_window, NET_WM_NAME, title)) {
+        if (! X11::getTextProperty(_window, XA_WM_NAME, title)) {
             return;
         }
-        title = Util::to_wide_str(mb_title);
     }
 
+    std::wstring wtitle = Util::to_wide_str(title);
     // Mirror it on the visible
     _title.setCustom(L"");
-    _title.setCount(titleFindID(title));
-    _title.setReal(title);
+    _title.setCount(titleFindID(wtitle));
+    _title.setReal(wtitle);
 
     // Apply title rules and find unique name, doesn't apply on
     // user-set titles
-    if (titleApplyRule(title)) {
-        _title.setCustom(title);
-        AtomUtil::setUtf8String(_window, Atoms::getAtom(NET_WM_VISIBLE_NAME), title);
+    if (titleApplyRule(wtitle)) {
+        _title.setCustom(wtitle);
+        X11::setUtf8String(_window, NET_WM_VISIBLE_NAME, Util::to_utf8_str(wtitle));
     } else {
-        AtomUtil::unsetProperty(_window, Atoms::getAtom(NET_WM_VISIBLE_NAME));
+        X11::unsetProperty(_window, NET_WM_VISIBLE_NAME);
     }
 }
 
@@ -1212,10 +1246,10 @@ Client::titleFindID(std::wstring &title)
     }
 
     uint id_found = 0;
-    list<uint> ids_used;
+    vector<uint> ids_used;
 
-    list<Client*>::iterator it = _client_list.begin();
-    for (; it != _client_list.end(); ++it) {
+    vector<Client*>::const_iterator it = _clients.begin();
+    for (; it != _clients.end(); ++it) {
         if (*it != this) {
             if ((*it)->getTitle()->getReal() == title) {
                 ids_used.push_back((*it)->getTitle()->getCount());
@@ -1225,9 +1259,9 @@ Client::titleFindID(std::wstring &title)
 
     // more than one client having this name
     if (ids_used.size() > 0) {
-        ids_used.sort();
+        std::sort(ids_used.begin(), ids_used.end());
 
-        list<uint>::iterator ui_it( ids_used.begin());
+        vector<uint>::const_iterator ui_it(ids_used.begin());
         for (uint i = 0; ui_it != ids_used.end(); ++i, ++ui_it) {
             if (i < *ui_it) {
                 id_found = i;
@@ -1243,33 +1277,6 @@ Client::titleFindID(std::wstring &title)
     return id_found;
 }
 
-/**
- * Get the clients icon name to be displayed when the client is
- * iconified.
- */
-void
-Client::readIconName(void)
-{
-    wstring icon_name;
-
-    if (! AtomUtil::getUtf8String(_window, Atoms::getAtom(NET_WM_ICON_NAME), icon_name)) {
-        string mb_icon_name;
-        if (AtomUtil::getTextProperty(_window, XA_WM_ICON_NAME, mb_icon_name)) {
-            icon_name = Util::to_wide_str(mb_icon_name);
-        }
-    }
-
-    // Set real name
-    _icon_name.setReal(icon_name);
-    _icon_name.setCustom(icon_name);
-
-    if (_icon_name.getVisible() == _icon_name.getReal()) {
-        AtomUtil::unsetProperty(_window, Atoms::getAtom(NET_WM_VISIBLE_ICON_NAME));
-    } else {
-        AtomUtil::setUtf8String(_window, Atoms::getAtom(NET_WM_VISIBLE_ICON_NAME), icon_name);
-    }
-}
-
 //! @brief Sets the WM_STATE of the client to state
 //! @param state State to set.
 void
@@ -1281,8 +1288,8 @@ Client::setWmState(ulong state)
     data[1] = None; // No Icon
 
     XChangeProperty(X11::getDpy(), _window,
-                    Atoms::getAtom(WM_STATE),
-                    Atoms::getAtom(WM_STATE),
+                    X11::getAtom(WM_STATE),
+                    X11::getAtom(WM_STATE),
                     32, PropModeReplace, (uchar*) data, 2);
 }
 
@@ -1301,8 +1308,8 @@ Client::getWmState(void)
     uchar *udata;
 
     int status =
-        XGetWindowProperty(X11::getDpy(), _window, Atoms::getAtom(WM_STATE),
-                           0L, 2L, False, Atoms::getAtom(WM_STATE),
+        XGetWindowProperty(X11::getDpy(), _window, X11::getAtom(WM_STATE),
+                           0L, 2L, False, X11::getAtom(WM_STATE),
                            &real_type, &real_format, &items_read, &items_left,
                            &udata);
     if ((status  == Success) && items_read) {
@@ -1344,17 +1351,15 @@ void Client::sendTakeFocusMessage(void)
     if (_send_focus_message) {
         {
             XEvent ev;
-            XChangeProperty(X11::getDpy(),
-                            RootWindow(X11::getDpy(), DefaultScreen(X11::getDpy())),
+            XChangeProperty(X11::getDpy(), X11::getRoot(),
                             XA_PRIMARY, XA_STRING, 8, PropModeAppend, 0, 0);
-            XWindowEvent(X11::getDpy(),
-                         RootWindow(X11::getDpy(), DefaultScreen(X11::getDpy())),
+            XWindowEvent(X11::getDpy(), X11::getRoot(),
                          PropertyChangeMask, &ev);
             X11::setLastEventTime(ev.xproperty.time);
         }
         X11::sendEvent(_window,
-                       Atoms::getAtom(WM_PROTOCOLS), NoEventMask,
-                       Atoms::getAtom(WM_TAKE_FOCUS),
+                       X11::getAtom(WM_PROTOCOLS), NoEventMask,
+                       X11::getAtom(WM_TAKE_FOCUS),
                        X11::getLastEventTime());
     }
 }
@@ -1410,16 +1415,21 @@ void
 Client::setSkip(uint skip)
 {
     _state.skip = skip;
-    AtomUtil::setLong(_window, Atoms::getAtom(PEKWM_FRAME_SKIP), _state.skip);
+    X11::setLong(_window, PEKWM_FRAME_SKIP, _state.skip);
 }
 
 //! @brief Sends an WM_DELETE message to the client, else kills it.
 void
 Client::close(void)
 {
+    // Check for DisallowedActions="Close".
+    if (! allowClose()) {
+        return;
+    }
+
     if (_send_close_message) {
-        X11::sendEvent(_window, Atoms::getAtom(WM_PROTOCOLS), NoEventMask,
-                     Atoms::getAtom(WM_DELETE_WINDOW), CurrentTime);
+        X11::sendEvent(_window, X11::getAtom(WM_PROTOCOLS), NoEventMask,
+                     X11::getAtom(WM_DELETE_WINDOW), CurrentTime);
     } else {
         kill();
     }
@@ -1566,7 +1576,7 @@ Client::getMwmHints(Window win)
     MwmHints *data = 0;
     uchar *udata;
 
-    Atom hints_atom = Atoms::getAtom(MOTIF_WM_HINTS);
+    Atom hints_atom = X11::getAtom(MOTIF_WM_HINTS);
 
     int status = XGetWindowProperty(X11::getDpy(), win, hints_atom, 0L, 20L, False, hints_atom,
                                     &real_type, &real_format, &items_read, &items_left, &udata);
@@ -1612,40 +1622,38 @@ Client::getEwmhStates(NetWMStates &win_states)
 {
     int num = 0;
     Atom *states;
-    states = (Atom*)
-             AtomUtil::getEwmhPropData(_window, Atoms::getAtom(STATE),
-                                       XA_ATOM, num);
+    states = (Atom*) X11::getEwmhPropData(_window, STATE, XA_ATOM, num);
 
     if (states) {
         for (int i = 0; i < num; ++i) {
-            if (states[i] == Atoms::getAtom(STATE_MODAL)) {
+            if (states[i] == X11::getAtom(STATE_MODAL)) {
                 win_states.modal = true;
-            } else if (states[i] == Atoms::getAtom(STATE_STICKY)) {
+            } else if (states[i] == X11::getAtom(STATE_STICKY)) {
                 win_states.sticky = true;
-            } else if (states[i] == Atoms::getAtom(STATE_MAXIMIZED_VERT)
+            } else if (states[i] == X11::getAtom(STATE_MAXIMIZED_VERT)
                        && ! isCfgDeny(CFG_DENY_STATE_MAXIMIZED_VERT)) {
                 win_states.max_vert = true;
-            } else if (states[i] == Atoms::getAtom(STATE_MAXIMIZED_HORZ)
+            } else if (states[i] == X11::getAtom(STATE_MAXIMIZED_HORZ)
                        && ! isCfgDeny(CFG_DENY_STATE_MAXIMIZED_HORZ)) {
                 win_states.max_horz = true;
-            } else if (states[i] == Atoms::getAtom(STATE_SHADED)) {
+            } else if (states[i] == X11::getAtom(STATE_SHADED)) {
                 win_states.shaded = true;
-            } else if (states[i] == Atoms::getAtom(STATE_SKIP_TASKBAR)) {
+            } else if (states[i] == X11::getAtom(STATE_SKIP_TASKBAR)) {
                 win_states.skip_taskbar = true;
-            } else if (states[i] == Atoms::getAtom(STATE_SKIP_PAGER)) {
+            } else if (states[i] == X11::getAtom(STATE_SKIP_PAGER)) {
                 win_states.skip_pager = true;
-            } else if (states[i] == Atoms::getAtom(STATE_DEMANDS_ATTENTION)) {
+            } else if (states[i] == X11::getAtom(STATE_DEMANDS_ATTENTION)) {
                 win_states.demands_attention = true;
-            } else if (states[i] == Atoms::getAtom(STATE_HIDDEN)
+            } else if (states[i] == X11::getAtom(STATE_HIDDEN)
                        && ! isCfgDeny(CFG_DENY_STATE_HIDDEN)) {
                 win_states.hidden = true;
-            } else if (states[i] == Atoms::getAtom(STATE_FULLSCREEN)
+            } else if (states[i] == X11::getAtom(STATE_FULLSCREEN)
                        && ! isCfgDeny(CFG_DENY_STATE_FULLSCREEN)) {
                 win_states.fullscreen = true;
-            } else if (states[i] == Atoms::getAtom(STATE_ABOVE)
+            } else if (states[i] == X11::getAtom(STATE_ABOVE)
                        && ! isCfgDeny(CFG_DENY_STATE_ABOVE)) {
                 win_states.above = true;
-            } else if (states[i] == Atoms::getAtom(STATE_BELOW)
+            } else if (states[i] == X11::getAtom(STATE_BELOW)
                        && ! isCfgDeny(CFG_DENY_STATE_BELOW)) {
                 win_states.below = true;
             }
@@ -1663,38 +1671,38 @@ Client::getEwmhStates(NetWMStates &win_states)
 void
 Client::updateEwmhStates(void)
 {
-    list<Atom> states;
+    vector<Atom> states;
 
     if (false) // we don't yet support modal state
-        states.push_back(Atoms::getAtom(STATE_MODAL));
+        states.push_back(X11::getAtom(STATE_MODAL));
     if (_sticky)
-        states.push_back(Atoms::getAtom(STATE_STICKY));
+        states.push_back(X11::getAtom(STATE_STICKY));
     if (_state.maximized_vert)
-        states.push_back(Atoms::getAtom(STATE_MAXIMIZED_VERT));
+        states.push_back(X11::getAtom(STATE_MAXIMIZED_VERT));
     if (_state.maximized_horz)
-        states.push_back(Atoms::getAtom(STATE_MAXIMIZED_HORZ));
+        states.push_back(X11::getAtom(STATE_MAXIMIZED_HORZ));
     if (_state.shaded)
-        states.push_back(Atoms::getAtom(STATE_SHADED));
+        states.push_back(X11::getAtom(STATE_SHADED));
     if (isSkip(SKIP_TASKBAR))
-        states.push_back(Atoms::getAtom(STATE_SKIP_TASKBAR));
+        states.push_back(X11::getAtom(STATE_SKIP_TASKBAR));
     if (isSkip(SKIP_PAGER))
-        states.push_back(Atoms::getAtom(STATE_SKIP_PAGER));
+        states.push_back(X11::getAtom(STATE_SKIP_PAGER));
     if (_iconified)
-        states.push_back(Atoms::getAtom(STATE_HIDDEN));
+        states.push_back(X11::getAtom(STATE_HIDDEN));
     if (_state.fullscreen)
-        states.push_back(Atoms::getAtom(STATE_FULLSCREEN));
+        states.push_back(X11::getAtom(STATE_FULLSCREEN));
     if (getLayer() == LAYER_ABOVE_DOCK) {
-        states.push_back(Atoms::getAtom(STATE_ABOVE));
+        states.push_back(X11::getAtom(STATE_ABOVE));
     }
     if (getLayer() == LAYER_BELOW) {
-        states.push_back(Atoms::getAtom(STATE_BELOW));
+        states.push_back(X11::getAtom(STATE_BELOW));
     }
 
     Atom *atoms = new Atom[(states.size() > 0) ? states.size() : 1];
     if (states.size() > 0) {
         copy(states.begin(), states.end(), atoms);
     }
-    AtomUtil::setAtoms(_window, Atoms::getAtom(STATE), atoms, states.size());
+    X11::setAtoms(_window, STATE, atoms, states.size());
     delete [] atoms;
 }
 
@@ -1706,24 +1714,24 @@ Client::updateWinType(bool set)
     Atom *atoms = 0;
 
     _window_type = WINDOW_TYPE;
-    atoms = (Atom*) AtomUtil::getEwmhPropData(_window, Atoms::getAtom(WINDOW_TYPE), XA_ATOM, items);
+    atoms = (Atom*) X11::getEwmhPropData(_window, WINDOW_TYPE, XA_ATOM, items);
     if (atoms) {
         for (int i = 0; _window_type == WINDOW_TYPE && i < items; ++i) {
-            if (atoms[i] == Atoms::getAtom(WINDOW_TYPE_DESKTOP)) {
+            if (atoms[i] == X11::getAtom(WINDOW_TYPE_DESKTOP)) {
                 _window_type = WINDOW_TYPE_DESKTOP;
-            } else if (atoms[i] == Atoms::getAtom(WINDOW_TYPE_DOCK)) {
+            } else if (atoms[i] == X11::getAtom(WINDOW_TYPE_DOCK)) {
                 _window_type = WINDOW_TYPE_DOCK;
-            } else if (atoms[i] == Atoms::getAtom(WINDOW_TYPE_TOOLBAR)) {
+            } else if (atoms[i] == X11::getAtom(WINDOW_TYPE_TOOLBAR)) {
                 _window_type = WINDOW_TYPE_TOOLBAR;
-            } else if (atoms[i] == Atoms::getAtom(WINDOW_TYPE_MENU)) {
+            } else if (atoms[i] == X11::getAtom(WINDOW_TYPE_MENU)) {
                 _window_type = WINDOW_TYPE_MENU;
-            } else if (atoms[i] == Atoms::getAtom(WINDOW_TYPE_UTILITY)) {
+            } else if (atoms[i] == X11::getAtom(WINDOW_TYPE_UTILITY)) {
                 _window_type = WINDOW_TYPE_UTILITY;
-            } else if (atoms[i] == Atoms::getAtom(WINDOW_TYPE_DIALOG)) {
+            } else if (atoms[i] == X11::getAtom(WINDOW_TYPE_DIALOG)) {
                 _window_type = WINDOW_TYPE_DIALOG;
-            } else if (atoms[i] == Atoms::getAtom(WINDOW_TYPE_SPLASH)) {
+            } else if (atoms[i] == X11::getAtom(WINDOW_TYPE_SPLASH)) {
                 _window_type = WINDOW_TYPE_SPLASH;
-            } else if (atoms[i] == Atoms::getAtom(WINDOW_TYPE_NORMAL)) {
+            } else if (atoms[i] == X11::getAtom(WINDOW_TYPE_NORMAL)) {
                 _window_type = WINDOW_TYPE_NORMAL;
             }
         }
@@ -1734,7 +1742,7 @@ Client::updateWinType(bool set)
     if (_window_type == WINDOW_TYPE) {
         _window_type = WINDOW_TYPE_NORMAL;
         if (set)
-            AtomUtil::setAtom(_window, Atoms::getAtom(WINDOW_TYPE), Atoms::getAtom(WINDOW_TYPE_NORMAL));
+            X11::setAtom(_window, WINDOW_TYPE, WINDOW_TYPE_NORMAL);
     }
 }
 
@@ -1814,9 +1822,9 @@ Client::getWMProtocols(void)
 
     if (XGetWMProtocols(X11::getDpy(), _window, &protocols, &count) != 0) {
         for (int i = 0; i < count; ++i) {
-            if (protocols[i] == Atoms::getAtom(WM_TAKE_FOCUS)) {
+            if (protocols[i] == X11::getAtom(WM_TAKE_FOCUS)) {
                 _send_focus_message = true;
-            } else if (protocols[i] == Atoms::getAtom(WM_DELETE_WINDOW)) {
+            } else if (protocols[i] == X11::getAtom(WM_DELETE_WINDOW)) {
                 _send_close_message = true;
             }
         }
@@ -1831,16 +1839,35 @@ Client::getWMProtocols(void)
 void
 Client::getTransientForHint(void)
 {
-    if (! _transient) {
-        XGetTransientForHint(X11::getDpy(), _window, &_transient_window);
+    if (_transient_for) {
+        return;
+    }
 
-        if (_transient_window != None) {
-            _transient = findClientFromWindow(_transient_window);
-            if (_transient) {
-               _transient->_transient_clients.push_back(this);
-               _transient->addObserver(this);
-            }
+    XGetTransientForHint(X11::getDpy(), _window, &_transient_for_window);
+
+    if (_transient_for_window != None) {
+        if (_transient_for_window == _window) {
+            ERR("Client set transient hint for itself.");
+            _transient_for_window = None;
+            return;
         }
+
+        _transient_for = findClientFromWindow(_transient_for_window);
+        if (_transient_for) {
+            // Observe for changes
+            _transient_for->_transients.push_back(this);
+            _transient_for->addObserver(this);
+        }
+    }
+}
+
+void
+Client::updateParentLayerAndRaiseIfActive(void)
+{
+    Frame *frame = static_cast<Frame*>(getParent());
+    if (frame->getActiveChild() == this) {
+        frame->setLayer(getLayer());
+        frame->raise();
     }
 }
 
@@ -1855,9 +1882,8 @@ Client::getStrutHint(void)
     removeStrutHint();
 
     int num = 0;
-    long *strut = static_cast<long*>(AtomUtil::getEwmhPropData(_window,
-                                                               Atoms::getAtom(NET_WM_STRUT),
-                                                               XA_CARDINAL, num));
+    long *strut = static_cast<long*>(X11::getEwmhPropData(_window, NET_WM_STRUT,
+                                                          XA_CARDINAL, num));
     if (strut) {
         _strut = new Strut();
         *_strut = strut;
@@ -1892,7 +1918,7 @@ long
 Client::getPekwmFrameOrder(void)
 {
     long num = -1;
-    AtomUtil::getLong(_window, Atoms::getAtom(PEKWM_FRAME_ORDER), num);
+    X11::getLong(_window, PEKWM_FRAME_ORDER, num);
     return num;
 }
 
@@ -1902,7 +1928,7 @@ Client::getPekwmFrameOrder(void)
 void
 Client::setPekwmFrameOrder(long num)
 {
-    AtomUtil::setLong(_window, Atoms::getAtom(PEKWM_FRAME_ORDER), num);
+    X11::setLong(_window, PEKWM_FRAME_ORDER, num);
 }
 
 /**
@@ -1913,7 +1939,7 @@ bool
 Client::getPekwmFrameActive(void)
 {
     long act = 0;
-    return (AtomUtil::getLong(_window, Atoms::getAtom(PEKWM_FRAME_ACTIVE), act)
+    return (X11::getLong(_window, PEKWM_FRAME_ACTIVE, act)
             && act == 1);
 }
 
@@ -1923,7 +1949,7 @@ Client::getPekwmFrameActive(void)
 void
 Client::setPekwmFrameActive(bool act)
 {
-    AtomUtil::setLong(_window, Atoms::getAtom(PEKWM_FRAME_ACTIVE), act ? 1 : 0);
+    X11::setLong(_window, PEKWM_FRAME_ACTIVE, act ? 1 : 0);
 }
 
 /**
