@@ -31,7 +31,6 @@ using std::cerr;
 using std::endl;
 using std::string;
 using std::wstring;
-using std::list;
 using std::map;
 using std::vector;
 using std::pair;
@@ -137,6 +136,7 @@ Config::Config(void) :
     _action_map["Close"] = pair<ActionType, uint>(ACTION_CLOSE, FRAME_MASK);
     _action_map["CloseFrame"] = pair<ActionType, uint>(ACTION_CLOSE_FRAME, FRAME_MASK);
     _action_map["Kill"] = pair<ActionType, uint>(ACTION_KILL, FRAME_MASK);
+    _action_map["SetGeometry"] = pair<ActionType, uint>(ACTION_SET_GEOMETRY, FRAME_MASK);
     _action_map["Raise"] = pair<ActionType, uint>(ACTION_RAISE, FRAME_MASK);
     _action_map["Lower"] = pair<ActionType, uint>(ACTION_LOWER, FRAME_MASK);
     _action_map["ActivateOrRaise"] = pair<ActionType, uint>(ACTION_ACTIVATE_OR_RAISE, FRAME_MASK);
@@ -149,6 +149,7 @@ Config::Config(void) :
     _action_map["MoveResize"] = pair<ActionType, uint>(ACTION_MOVE_RESIZE, KEYGRABBER_OK);
     _action_map["GroupingDrag"] = pair<ActionType, uint>(ACTION_GROUPING_DRAG, FRAME_OK|CLIENT_OK);
     _action_map["WarpToWorkspace"] = pair<ActionType, uint>(ACTION_WARP_TO_WORKSPACE, SCREEN_EDGE_OK);
+    _action_map["MoveToHead"] = pair<ActionType, uint>(ACTION_MOVE_TO_HEAD, FRAME_MASK);
     _action_map["MoveToEdge"] = pair<ActionType, uint>(ACTION_MOVE_TO_EDGE, KEYGRABBER_OK);
     _action_map["NextFrame"] = pair<ActionType, uint>(ACTION_NEXT_FRAME, KEYGRABBER_OK|ROOTCLICK_OK|SCREEN_EDGE_OK);
     _action_map["PrevFrame"] = pair<ActionType, uint>(ACTION_PREV_FRAME, KEYGRABBER_OK|ROOTCLICK_OK|SCREEN_EDGE_OK);
@@ -176,11 +177,12 @@ Config::Config(void) :
     _action_map["Dynamic"] = pair<ActionType, uint>(ACTION_MENU_DYN, ROOTMENU_OK|WINDOWMENU_OK);
     _action_map["SendKey"] = pair<ActionType, uint>(ACTION_SEND_KEY, ANY_MASK);
     _action_map["SetOpacity"] = pair<ActionType, uint>(ACTION_SET_OPACITY, FRAME_MASK);
+    _action_map["Debug"] = pair<ActionType, uint>(ACTION_DEBUG, ANY_MASK);
 
     _action_access_mask_map[""] = ACTION_ACCESS_NO;
     _action_access_mask_map["MOVE"] = ACTION_ACCESS_MOVE;
     _action_access_mask_map["RESIZE"] = ACTION_ACCESS_RESIZE;
-    _action_access_mask_map["ICONIFY"] = ACTION_ACCESS_MINIMIZE;
+    _action_access_mask_map["ICONIFY"] = ACTION_ACCESS_ICONIFY;
     _action_access_mask_map["SHADE"] = ACTION_ACCESS_SHADE;
     _action_access_mask_map["STICK"] = ACTION_ACCESS_STICK;
     _action_access_mask_map["MAXIMIZEHORIZONTAL"] = ACTION_ACCESS_MAXIMIZE_HORZ;
@@ -312,6 +314,7 @@ Config::Config(void) :
     _action_state_map["Sticky"] = ACTION_STATE_STICKY;
     _action_state_map["AlwaysOnTop"] = ACTION_STATE_ALWAYS_ONTOP;
     _action_state_map["AlwaysBelow"] = ACTION_STATE_ALWAYS_BELOW;
+    _action_state_map["Decor"] = ACTION_STATE_DECOR;
     _action_state_map["DecorBorder"] = ACTION_STATE_DECOR_BORDER;
     _action_state_map["DecorTitlebar"] = ACTION_STATE_DECOR_TITLEBAR;
     _action_state_map["Iconified"] = ACTION_STATE_ICONIFIED;
@@ -358,27 +361,27 @@ Config::Config(void) :
     _harbour_orientation_map["RIGHTTOLEFT"] = BOTTOM_TO_TOP;
 
     // fill the mouse action map
-    _mouse_action_map[MOUSE_ACTION_LIST_TITLE_FRAME] = new list<ActionEvent>;
-    _mouse_action_map[MOUSE_ACTION_LIST_TITLE_OTHER] = new list<ActionEvent>;
-    _mouse_action_map[MOUSE_ACTION_LIST_CHILD_FRAME] = new list<ActionEvent>;
-    _mouse_action_map[MOUSE_ACTION_LIST_CHILD_OTHER] = new list<ActionEvent>;
-    _mouse_action_map[MOUSE_ACTION_LIST_ROOT] = new list<ActionEvent>;
-    _mouse_action_map[MOUSE_ACTION_LIST_MENU] = new list<ActionEvent>;
-    _mouse_action_map[MOUSE_ACTION_LIST_OTHER] = new list<ActionEvent>;
+    _mouse_action_map[MOUSE_ACTION_LIST_TITLE_FRAME] = new vector<ActionEvent>;
+    _mouse_action_map[MOUSE_ACTION_LIST_TITLE_OTHER] = new vector<ActionEvent>;
+    _mouse_action_map[MOUSE_ACTION_LIST_CHILD_FRAME] = new vector<ActionEvent>;
+    _mouse_action_map[MOUSE_ACTION_LIST_CHILD_OTHER] = new vector<ActionEvent>;
+    _mouse_action_map[MOUSE_ACTION_LIST_ROOT] = new vector<ActionEvent>;
+    _mouse_action_map[MOUSE_ACTION_LIST_MENU] = new vector<ActionEvent>;
+    _mouse_action_map[MOUSE_ACTION_LIST_OTHER] = new vector<ActionEvent>;
 
-    _mouse_action_map[MOUSE_ACTION_LIST_EDGE_T] = new list<ActionEvent>;
-    _mouse_action_map[MOUSE_ACTION_LIST_EDGE_B] = new list<ActionEvent>;
-    _mouse_action_map[MOUSE_ACTION_LIST_EDGE_L] = new list<ActionEvent>;
-    _mouse_action_map[MOUSE_ACTION_LIST_EDGE_R] = new list<ActionEvent>;
+    _mouse_action_map[MOUSE_ACTION_LIST_EDGE_T] = new vector<ActionEvent>;
+    _mouse_action_map[MOUSE_ACTION_LIST_EDGE_B] = new vector<ActionEvent>;
+    _mouse_action_map[MOUSE_ACTION_LIST_EDGE_L] = new vector<ActionEvent>;
+    _mouse_action_map[MOUSE_ACTION_LIST_EDGE_R] = new vector<ActionEvent>;
 
-    _mouse_action_map[MOUSE_ACTION_LIST_BORDER_TL] = new list<ActionEvent>;
-    _mouse_action_map[MOUSE_ACTION_LIST_BORDER_T] = new list<ActionEvent>;
-    _mouse_action_map[MOUSE_ACTION_LIST_BORDER_TR] = new list<ActionEvent>;
-    _mouse_action_map[MOUSE_ACTION_LIST_BORDER_L] = new list<ActionEvent>;
-    _mouse_action_map[MOUSE_ACTION_LIST_BORDER_R] = new list<ActionEvent>;
-    _mouse_action_map[MOUSE_ACTION_LIST_BORDER_BL] = new list<ActionEvent>;
-    _mouse_action_map[MOUSE_ACTION_LIST_BORDER_B] = new list<ActionEvent>;
-    _mouse_action_map[MOUSE_ACTION_LIST_BORDER_BR] = new list<ActionEvent>;
+    _mouse_action_map[MOUSE_ACTION_LIST_BORDER_TL] = new vector<ActionEvent>;
+    _mouse_action_map[MOUSE_ACTION_LIST_BORDER_T] = new vector<ActionEvent>;
+    _mouse_action_map[MOUSE_ACTION_LIST_BORDER_TR] = new vector<ActionEvent>;
+    _mouse_action_map[MOUSE_ACTION_LIST_BORDER_L] = new vector<ActionEvent>;
+    _mouse_action_map[MOUSE_ACTION_LIST_BORDER_R] = new vector<ActionEvent>;
+    _mouse_action_map[MOUSE_ACTION_LIST_BORDER_BL] = new vector<ActionEvent>;
+    _mouse_action_map[MOUSE_ACTION_LIST_BORDER_B] = new vector<ActionEvent>;
+    _mouse_action_map[MOUSE_ACTION_LIST_BORDER_BR] = new vector<ActionEvent>;
 }
 
 //! @brief Destructor for Config class
@@ -386,7 +389,7 @@ Config::~Config(void)
 {
     _instance = 0;
 
-    map<MouseActionListName, list<ActionEvent>* >::iterator it;
+    map<MouseActionListName, vector<ActionEvent>* >::iterator it;
     for (it = _mouse_action_map.begin(); it != _mouse_action_map.end(); ++it) {
         delete it->second;
     }
@@ -446,7 +449,7 @@ Config::setDesktopNamesUTF8(char *names, ulong length)
 bool
 Config::load(const std::string &config_file)
 {
-    if (! Util::requireReload(_cfg_state, config_file)) {
+    if (! _cfg_files.requireReload(config_file)) {
         return false;
     }
 
@@ -458,9 +461,9 @@ Config::load(const std::string &config_file)
     // Make sure config is reloaded next time as content is dynamically
     // generated from the configuration file.
     if (! success || cfg.is_dynamic_content()) {
-        _cfg_state.clear();
+        _cfg_files.clear();
     } else {
-        _cfg_state = cfg.get_file_list();
+        _cfg_files = cfg.getCfgFiles();
     }
 
     if (! success) {
@@ -523,20 +526,20 @@ Config::loadFiles(CfgParser::Entry *section)
         return;
     }
 
-    list<CfgParserKey*> key_list;
-    key_list.push_back(new CfgParserKeyPath("KEYS", _files_keys, SYSCONFDIR "/keys"));
-    key_list.push_back(new CfgParserKeyPath("MOUSE", _files_mouse, SYSCONFDIR "/mouse"));
-    key_list.push_back(new CfgParserKeyPath("MENU", _files_menu, SYSCONFDIR "/menu"));
-    key_list.push_back(new CfgParserKeyPath("START", _files_start, SYSCONFDIR "/start"));
-    key_list.push_back(new CfgParserKeyPath("AUTOPROPS", _files_autoprops, SYSCONFDIR "/autoproperties"));
-    key_list.push_back(new CfgParserKeyPath("THEME", _files_theme, DATADIR "/pekwm/themes/default/theme"));
-    key_list.push_back(new CfgParserKeyPath("ICONS", _files_icon_path, DATADIR "/pekwm/icons"));
+    vector<CfgParserKey*> keys;
+    keys.push_back(new CfgParserKeyPath("KEYS", _files_keys, SYSCONFDIR "/keys"));
+    keys.push_back(new CfgParserKeyPath("MOUSE", _files_mouse, SYSCONFDIR "/mouse"));
+    keys.push_back(new CfgParserKeyPath("MENU", _files_menu, SYSCONFDIR "/menu"));
+    keys.push_back(new CfgParserKeyPath("START", _files_start, SYSCONFDIR "/start"));
+    keys.push_back(new CfgParserKeyPath("AUTOPROPS", _files_autoprops, SYSCONFDIR "/autoproperties"));
+    keys.push_back(new CfgParserKeyPath("THEME", _files_theme, DATADIR "/pekwm/themes/default/theme"));
+    keys.push_back(new CfgParserKeyPath("ICONS", _files_icon_path, DATADIR "/pekwm/icons"));
 
     // Parse
-    section->parse_key_values(key_list.begin(), key_list.end());
+    section->parse_key_values(keys.begin(), keys.end());
 
     // Free up resources
-    for_each(key_list.begin(), key_list.end(), Util::Free<CfgParserKey*>());
+    for_each(keys.begin(), keys.end(), Util::Free<CfgParserKey*>());
 }
 
 //! @brief Loads MOVERESIZE section of main configuration
@@ -548,19 +551,19 @@ Config::loadMoveResize(CfgParser::Entry *section)
         return;
     }
 
-    list<CfgParserKey*> key_list;
-    key_list.push_back(new CfgParserKeyNumeric<int>("EDGEATTRACT", _moveresize_edgeattract, 0, 0));
-    key_list.push_back(new CfgParserKeyNumeric<int>("EDGERESIST", _moveresize_edgeresist, 0, 0));
-    key_list.push_back(new CfgParserKeyNumeric<int>("WINDOWATTRACT",_moveresize_woattract, 0, 0));
-    key_list.push_back(new CfgParserKeyNumeric<int>("WINDOWRESIST", _moveresize_woresist, 0, 0));
-    key_list.push_back(new CfgParserKeyBool("OPAQUEMOVE", _moveresize_opaquemove));
-    key_list.push_back(new CfgParserKeyBool("OPAQUERESIZE", _moveresize_opaqueresize));
+    vector<CfgParserKey*> keys;
+    keys.push_back(new CfgParserKeyNumeric<int>("EDGEATTRACT", _moveresize_edgeattract, 0, 0));
+    keys.push_back(new CfgParserKeyNumeric<int>("EDGERESIST", _moveresize_edgeresist, 0, 0));
+    keys.push_back(new CfgParserKeyNumeric<int>("WINDOWATTRACT",_moveresize_woattract, 0, 0));
+    keys.push_back(new CfgParserKeyNumeric<int>("WINDOWRESIST", _moveresize_woresist, 0, 0));
+    keys.push_back(new CfgParserKeyBool("OPAQUEMOVE", _moveresize_opaquemove));
+    keys.push_back(new CfgParserKeyBool("OPAQUERESIZE", _moveresize_opaqueresize));
 
     // Parse data
-    section->parse_key_values(key_list.begin(), key_list.end());
+    section->parse_key_values(keys.begin(), keys.end());
 
     // Free up resources
-    for_each(key_list.begin(), key_list.end(), Util::Free<CfgParserKey*>());
+    for_each(keys.begin(), keys.end(), Util::Free<CfgParserKey*>());
 }
 
 //! @brief Loads SCREEN section of main configuration
@@ -576,40 +579,40 @@ Config::loadScreen(CfgParser::Entry *section)
     string edge_size, workspace_names, trim_title;
     CfgParser::Entry *value;
 
-    list<CfgParserKey*> key_list;
-    key_list.push_back(new CfgParserKeyNumeric<int>("WORKSPACES", _screen_workspaces, 4, 1));
-    key_list.push_back(new CfgParserKeyNumeric<int>("PIXMAPCACHESIZE", _screen_pixmap_cache_size));
-    key_list.push_back(new CfgParserKeyNumeric<int>("WORKSPACESPERROW", _screen_workspaces_per_row, 0, 0));
-    key_list.push_back(new CfgParserKeyString("WORKSPACENAMES", workspace_names));
-    key_list.push_back(new CfgParserKeyString("EDGESIZE", edge_size));
-    key_list.push_back(new CfgParserKeyBool("EDGEINDENT", _screen_edge_indent));
-    key_list.push_back(new CfgParserKeyNumeric<int>("DOUBLECLICKTIME", _screen_doubleclicktime, 250, 0));
-    key_list.push_back(new CfgParserKeyString("TRIMTITLE", trim_title));
-    key_list.push_back(new CfgParserKeyBool("FULLSCREENABOVE", _screen_fullscreen_above, true));
-    key_list.push_back(new CfgParserKeyBool("FULLSCREENDETECT", _screen_fullscreen_detect, true));
-    key_list.push_back(new CfgParserKeyBool("SHOWFRAMELIST", _screen_showframelist));
-    key_list.push_back(new CfgParserKeyBool("SHOWSTATUSWINDOW", _screen_show_status_window));
-    key_list.push_back(new CfgParserKeyBool("SHOWSTATUSWINDOWCENTEREDONROOT", _screen_show_status_window_on_root, false));
-    key_list.push_back(new CfgParserKeyBool("SHOWCLIENTID", _screen_show_client_id));
-    key_list.push_back(new CfgParserKeyNumeric<int>("SHOWWORKSPACEINDICATOR",
+    vector<CfgParserKey*> keys;
+    keys.push_back(new CfgParserKeyNumeric<int>("WORKSPACES", _screen_workspaces, 4, 1));
+    keys.push_back(new CfgParserKeyNumeric<int>("PIXMAPCACHESIZE", _screen_pixmap_cache_size));
+    keys.push_back(new CfgParserKeyNumeric<int>("WORKSPACESPERROW", _screen_workspaces_per_row, 0, 0));
+    keys.push_back(new CfgParserKeyString("WORKSPACENAMES", workspace_names));
+    keys.push_back(new CfgParserKeyString("EDGESIZE", edge_size));
+    keys.push_back(new CfgParserKeyBool("EDGEINDENT", _screen_edge_indent));
+    keys.push_back(new CfgParserKeyNumeric<int>("DOUBLECLICKTIME", _screen_doubleclicktime, 250, 0));
+    keys.push_back(new CfgParserKeyString("TRIMTITLE", trim_title));
+    keys.push_back(new CfgParserKeyBool("FULLSCREENABOVE", _screen_fullscreen_above, true));
+    keys.push_back(new CfgParserKeyBool("FULLSCREENDETECT", _screen_fullscreen_detect, true));
+    keys.push_back(new CfgParserKeyBool("SHOWFRAMELIST", _screen_showframelist));
+    keys.push_back(new CfgParserKeyBool("SHOWSTATUSWINDOW", _screen_show_status_window));
+    keys.push_back(new CfgParserKeyBool("SHOWSTATUSWINDOWCENTEREDONROOT", _screen_show_status_window_on_root, false));
+    keys.push_back(new CfgParserKeyBool("SHOWCLIENTID", _screen_show_client_id));
+    keys.push_back(new CfgParserKeyNumeric<int>("SHOWWORKSPACEINDICATOR",
                                            _screen_show_workspace_indicator, 500, 0));
-    key_list.push_back(new CfgParserKeyNumeric<int>("WORKSPACEINDICATORSCALE",
+    keys.push_back(new CfgParserKeyNumeric<int>("WORKSPACEINDICATORSCALE",
                                            _screen_workspace_indicator_scale, 16, 2)); 
-    key_list.push_back(new CfgParserKeyNumeric<uint>("WORKSPACEINDICATOROPACITY",
+    keys.push_back(new CfgParserKeyNumeric<uint>("WORKSPACEINDICATOROPACITY",
                                            _screen_workspace_indicator_opacity, 100, 0, 100));
-    key_list.push_back(new CfgParserKeyBool("PLACENEW", _screen_place_new));
-    key_list.push_back(new CfgParserKeyBool("FOCUSNEW", _screen_focus_new));
-    key_list.push_back(new CfgParserKeyBool("FOCUSNEWCHILD", _screen_focus_new_child, true));
-    key_list.push_back(new CfgParserKeyBool("HONOURRANDR", _screen_honour_randr, true));
-    key_list.push_back(new CfgParserKeyBool("HONOURASPECTRATIO", _screen_honour_aspectratio, true));
-    key_list.push_back(new CfgParserKeyBool("REPORTALLCLIENTS", _screen_report_all_clients, false));
+    keys.push_back(new CfgParserKeyBool("PLACENEW", _screen_place_new));
+    keys.push_back(new CfgParserKeyBool("FOCUSNEW", _screen_focus_new));
+    keys.push_back(new CfgParserKeyBool("FOCUSNEWCHILD", _screen_focus_new_child, true));
+    keys.push_back(new CfgParserKeyBool("HONOURRANDR", _screen_honour_randr, true));
+    keys.push_back(new CfgParserKeyBool("HONOURASPECTRATIO", _screen_honour_aspectratio, true));
+    keys.push_back(new CfgParserKeyBool("REPORTALLCLIENTS", _screen_report_all_clients, false));
 
     // Parse data
-    section->parse_key_values(key_list.begin(), key_list.end());
+    section->parse_key_values(keys.begin(), keys.end());
 
     // Free up resources
-    for_each(key_list.begin(), key_list.end(), Util::Free<CfgParserKey*>());
-    key_list.clear();
+    for_each(keys.begin(), keys.end(), Util::Free<CfgParserKey*>());
+    keys.clear();
 
     // Convert input data
     _screen_trim_title = Util::to_wide_str(trim_title);
@@ -663,18 +666,18 @@ Config::loadScreen(CfgParser::Entry *section)
 
         CfgParser::Entry *sub_2 = sub->find_section("SMART");
         if (sub_2) {
-            key_list.push_back(new CfgParserKeyBool("ROW", _screen_placement_row));
-            key_list.push_back(new CfgParserKeyBool("LEFTTORIGHT", _screen_placement_ltr));
-            key_list.push_back(new CfgParserKeyBool("TOPTOBOTTOM", _screen_placement_ttb));
-            key_list.push_back(new CfgParserKeyNumeric<int>("OFFSETX", _screen_placement_offset_x, 0, 0));
-            key_list.push_back(new CfgParserKeyNumeric<int>("OFFSETY", _screen_placement_offset_y, 0, 0));
+            keys.push_back(new CfgParserKeyBool("ROW", _screen_placement_row));
+            keys.push_back(new CfgParserKeyBool("LEFTTORIGHT", _screen_placement_ltr));
+            keys.push_back(new CfgParserKeyBool("TOPTOBOTTOM", _screen_placement_ttb));
+            keys.push_back(new CfgParserKeyNumeric<int>("OFFSETX", _screen_placement_offset_x, 0, 0));
+            keys.push_back(new CfgParserKeyNumeric<int>("OFFSETY", _screen_placement_offset_y, 0, 0));
 
             // Do the parsing
-            sub_2->parse_key_values(key_list.begin(), key_list.end());
+            sub_2->parse_key_values(keys.begin(), keys.end());
 
             // Freeup resources
-            for_each(key_list.begin(), key_list.end(), Util::Free<CfgParserKey*>());
-            key_list.clear();
+            for_each(keys.begin(), keys.end(), Util::Free<CfgParserKey*>());
+            keys.clear();
         }
     }
 
@@ -685,16 +688,16 @@ Config::loadScreen(CfgParser::Entry *section)
 
     sub = section->find_section("UNIQUENAMES");
     if (sub) {
-        key_list.push_back(new CfgParserKeyBool("SETUNIQUE", _screen_client_unique_name));
-        key_list.push_back(new CfgParserKeyString("PRE", _screen_client_unique_name_pre));
-        key_list.push_back(new CfgParserKeyString("POST", _screen_client_unique_name_post));
+        keys.push_back(new CfgParserKeyBool("SETUNIQUE", _screen_client_unique_name));
+        keys.push_back(new CfgParserKeyString("PRE", _screen_client_unique_name_pre));
+        keys.push_back(new CfgParserKeyString("POST", _screen_client_unique_name_post));
 
         // Parse data
-        sub->parse_key_values(key_list.begin(), key_list.end());
+        sub->parse_key_values(keys.begin(), keys.end());
 
         // Free up resources
-        for_each(key_list.begin(), key_list.end(), Util::Free<CfgParserKey*>());
-        key_list.clear();
+        for_each(keys.begin(), keys.end(), Util::Free<CfgParserKey*>());
+        keys.clear();
     }
 }
 
@@ -707,26 +710,26 @@ Config::loadMenu(CfgParser::Entry *section)
         return;
     }
 
-    list<CfgParserKey*> key_list;
+    vector<CfgParserKey*> keys;
     string value_select, value_enter, value_exec;
 
-    key_list.push_back(new CfgParserKeyString("SELECT", value_select, "MOTION", 0));
-    key_list.push_back(new CfgParserKeyString("ENTER", value_enter, "BUTTONPRESS", 0));
-    key_list.push_back(new CfgParserKeyString("EXEC", value_exec, "BUTTONRELEASE", 0));
-    key_list.push_back(new CfgParserKeyBool("DISPLAYICONS", _menu_display_icons, true));
-    key_list.push_back(new CfgParserKeyNumeric<uint>("FOCUSOPACITY", _menu_focus_opacity, 100, 0, 100));
-    key_list.push_back(new CfgParserKeyNumeric<uint>("UNFOCUSOPACITY", _menu_unfocus_opacity, 100, 0, 100));
+    keys.push_back(new CfgParserKeyString("SELECT", value_select, "MOTION", 0));
+    keys.push_back(new CfgParserKeyString("ENTER", value_enter, "BUTTONPRESS", 0));
+    keys.push_back(new CfgParserKeyString("EXEC", value_exec, "BUTTONRELEASE", 0));
+    keys.push_back(new CfgParserKeyBool("DISPLAYICONS", _menu_display_icons, true));
+    keys.push_back(new CfgParserKeyNumeric<uint>("FOCUSOPACITY", _menu_focus_opacity, 100, 0, 100));
+    keys.push_back(new CfgParserKeyNumeric<uint>("UNFOCUSOPACITY", _menu_unfocus_opacity, 100, 0, 100));
 
     // Parse data
-    section->parse_key_values(key_list.begin(), key_list.end());
+    section->parse_key_values(keys.begin(), keys.end());
 
     _menu_select_mask = getMenuMask(value_select);
     _menu_enter_mask = getMenuMask(value_enter);
     _menu_exec_mask = getMenuMask(value_exec);
 
     // Free up resources
-    for_each(key_list.begin(), key_list.end(), Util::Free<CfgParserKey*>());
-    key_list.clear();
+    for_each(keys.begin(), keys.end(), Util::Free<CfgParserKey*>());
+    keys.clear();
 
     // Parse icon size limits
     CfgParser::iterator it(section->begin());
@@ -751,16 +754,16 @@ Config::loadMenuIcons(CfgParser::Entry *section)
         return;
     }
 
-    list<CfgParserKey*> key_list;
+    vector<CfgParserKey*> keys;
     string minimum, maximum;
 
-    key_list.push_back(new CfgParserKeyString("MINIMUM", minimum, "16x16", 3));
-    key_list.push_back(new CfgParserKeyString("MAXIMUM", maximum, "16x16", 3));
+    keys.push_back(new CfgParserKeyString("MINIMUM", minimum, "16x16", 3));
+    keys.push_back(new CfgParserKeyString("MAXIMUM", maximum, "16x16", 3));
 
     // Parse data
-    section->parse_key_values(key_list.begin(), key_list.end());
+    section->parse_key_values(keys.begin(), keys.end());
 
-    for_each(key_list.begin(), key_list.end(), Util::Free<CfgParserKey*>());
+    for_each(keys.begin(), keys.end(), Util::Free<CfgParserKey*>());
 
     SizeLimits limits;
     if (limits.parse(minimum, maximum)) {
@@ -774,20 +777,20 @@ Config::loadMenuIcons(CfgParser::Entry *section)
 void
 Config::loadCmdDialog(CfgParser::Entry *section)
 {
-  if (! section) {
-    return;
-  }
+    if (! section) {
+        return;
+    }
 
-  list<CfgParserKey*> key_list;
+    vector<CfgParserKey*> keys;
 
-  key_list.push_back(new CfgParserKeyBool("HISTORYUNIQUE", _cmd_dialog_history_unique));
-  key_list.push_back(new CfgParserKeyNumeric<int>("HISTORYSIZE", _cmd_dialog_history_size, 1024, 1));
-  key_list.push_back(new CfgParserKeyPath("HISTORYFILE", _cmd_dialog_history_file, "~/.pekwm/history"));
-  key_list.push_back(new CfgParserKeyNumeric<int>("HISTORYSAVEINTERVAL", _cmd_dialog_history_save_interval, 16, 0));
+    keys.push_back(new CfgParserKeyBool("HISTORYUNIQUE", _cmd_dialog_history_unique));
+    keys.push_back(new CfgParserKeyNumeric<int>("HISTORYSIZE", _cmd_dialog_history_size, 1024, 1));
+    keys.push_back(new CfgParserKeyPath("HISTORYFILE", _cmd_dialog_history_file, "~/.pekwm/history"));
+    keys.push_back(new CfgParserKeyNumeric<int>("HISTORYSAVEINTERVAL", _cmd_dialog_history_save_interval, 16, 0));
 
-  section->parse_key_values(key_list.begin(), key_list.end());
+    section->parse_key_values(keys.begin(), keys.end());
 
-  for_each(key_list.begin(), key_list.end(), Util::Free<CfgParserKey*>());
+    for_each(keys.begin(), keys.end(), Util::Free<CfgParserKey*>());
 }
 
 //! @brief Loads the HARBOUR section of the main configuration
@@ -798,22 +801,22 @@ Config::loadHarbour(CfgParser::Entry *section)
         return;
     }
 
-    list<CfgParserKey*> key_list;
+    vector<CfgParserKey*> keys;
     string value_placement, value_orientation;
 
-    key_list.push_back(new CfgParserKeyBool("ONTOP", _harbour_ontop, true));
-    key_list.push_back(new CfgParserKeyBool("MAXIMIZEOVER", _harbour_maximize_over, false));
-    key_list.push_back(new CfgParserKeyNumeric<int>("HEAD", _harbour_head_nr, 0, 0));
-    key_list.push_back(new CfgParserKeyString("PLACEMENT", value_placement, "RIGHT", 0));
-    key_list.push_back(new CfgParserKeyString("ORIENTATION", value_orientation, "TOPTOBOTTOM", 0));
-    key_list.push_back(new CfgParserKeyNumeric<uint>("OPACITY", _harbour_opacity, 100, 0, 100));
+    keys.push_back(new CfgParserKeyBool("ONTOP", _harbour_ontop, true));
+    keys.push_back(new CfgParserKeyBool("MAXIMIZEOVER", _harbour_maximize_over, false));
+    keys.push_back(new CfgParserKeyNumeric<int>("HEAD", _harbour_head_nr, 0, 0));
+    keys.push_back(new CfgParserKeyString("PLACEMENT", value_placement, "RIGHT", 0));
+    keys.push_back(new CfgParserKeyString("ORIENTATION", value_orientation, "TOPTOBOTTOM", 0));
+    keys.push_back(new CfgParserKeyNumeric<uint>("OPACITY", _harbour_opacity, 100, 0, 100));
 
     // Parse data
-    section->parse_key_values(key_list.begin(), key_list.end());
+    section->parse_key_values(keys.begin(), keys.end());
 
     // Free up resources
-    for_each(key_list.begin(), key_list.end(), Util::Free<CfgParserKey*>());
-    key_list.clear();
+    for_each(keys.begin(), keys.end(), Util::Free<CfgParserKey*>());
+    keys.clear();
 
     // Convert opacity from percent to absolute value
     CONV_OPACITY(_harbour_opacity);
@@ -829,15 +832,15 @@ Config::loadHarbour(CfgParser::Entry *section)
 
     CfgParser::Entry *sub = section->find_section("DOCKAPP");
     if (sub) {
-        key_list.push_back(new CfgParserKeyNumeric<int>("SIDEMIN", _harbour_da_min_s, 64, 0));
-        key_list.push_back(new CfgParserKeyNumeric<int>("SIDEMAX", _harbour_da_max_s, 64, 0));
+        keys.push_back(new CfgParserKeyNumeric<int>("SIDEMIN", _harbour_da_min_s, 64, 0));
+        keys.push_back(new CfgParserKeyNumeric<int>("SIDEMAX", _harbour_da_max_s, 64, 0));
 
         // Parse data
-        sub->parse_key_values(key_list.begin(), key_list.end());
+        sub->parse_key_values(keys.begin(), keys.end());
 
         // Free up resources
-        for_each(key_list.begin(), key_list.end(), Util::Free<CfgParserKey*>());
-        key_list.clear();
+        for_each(keys.begin(), keys.end(), Util::Free<CfgParserKey*>());
+        keys.clear();
     }
 }
 
@@ -981,11 +984,24 @@ Config::parseAction(const std::string &action_string, Action &action, uint mask)
                 case ACTION_SHOW_SEARCH_DIALOG:
                 case ACTION_SEND_KEY:
                 case ACTION_MENU_DYN:
+                case ACTION_DEBUG:
                     action.setParamS(tok[1]);
                     break;
+                case ACTION_SET_GEOMETRY:
+                    // Add optional head parameter, -1 means screen.
+                    Util::splitString(tok[1], tok, " \t", 2);
+                    if (tok.size() == 4) {
+                        action.setParamS(tok[2]);
+                        action.setParamI(0, strtol(tok[3].c_str(), 0, 10));
+                    } else {
+                        action.setParamS(tok[1]);
+                        action.setParamI(0, -1);
+                    }
+                    break;
                 case ACTION_ACTIVATE_CLIENT_REL:
                 case ACTION_MOVE_CLIENT_REL:
                 case ACTION_GOTO_CLIENT_ID:
+                case ACTION_MOVE_TO_HEAD:
                     action.setParamI(0, strtol(tok[1].c_str(), 0, 10));
                     break;
                 case ACTION_SET:
@@ -1329,6 +1345,7 @@ Config::parseInputDialogActions(const std::string &actions, ActionEvent &ae)
     vector<string> tok;
     vector<string>::iterator it;
     Action action;
+    string::size_type first, last;
 
     // reset the action event
     ae.action_list.clear();
@@ -1336,6 +1353,11 @@ Config::parseInputDialogActions(const std::string &actions, ActionEvent &ae)
     // chop the string up separating the actions
     if (Util::splitString(actions, tok, ";")) {
         for (it = tok.begin(); it != tok.end(); ++it) {
+            first = (*it).find_first_not_of(" \t\n");
+            if (first == string::npos)
+                continue;
+            last = (*it).find_last_not_of(" \t\n");
+            (*it) = (*it).substr(first, last-first+1);
             if (parseInputDialogAction(*it, action)) {
                 ae.action_list.push_back(action);
                 action.clear();
@@ -1631,26 +1653,25 @@ Config::copyConfigFiles(void)
 bool
 Config::loadMouseConfig(const std::string &mouse_file)
 {
-    if (! Util::requireReload(_mouse_state, mouse_file)) {
+    if (! _cfg_files_mouse.requireReload(mouse_file)) {
         return false;
     }
     
     CfgParser mouse_cfg;
     if (! mouse_cfg.parse(mouse_file, CfgParserSource::SOURCE_FILE, true)
         && ! mouse_cfg.parse(SYSCONFDIR "/mouse", CfgParserSource::SOURCE_FILE, true)) {
-        _mouse_state.clear();
+        _cfg_files_mouse.clear();
         return false;
     }
     
-    // Clear state if load failed or dynamic content.
     if (mouse_cfg.is_dynamic_content()) {
-        _mouse_state.clear();
+        _cfg_files_mouse.clear();
     } else {
-        _mouse_state = mouse_cfg.get_file_list();
+        _cfg_files_mouse = mouse_cfg.getCfgFiles();
     }
 
     // Make sure old actions get unloaded.
-    map<MouseActionListName, list<ActionEvent>* >::iterator it;
+    map<MouseActionListName, vector<ActionEvent>* >::iterator it;
     for (it = _mouse_action_map.begin(); it != _mouse_action_map.end(); ++it) {
         it->second->clear();
     }
@@ -1715,7 +1736,7 @@ Config::loadMouseConfig(const std::string &mouse_file)
 
 //! @brief Parses mouse config section, like FRAME
 void
-Config::parseButtons(CfgParser::Entry *section, std::list<ActionEvent>* mouse_list, ActionOk action_ok)
+Config::parseButtons(CfgParser::Entry *section, vector<ActionEvent>* mouse_list, ActionOk action_ok)
 {
     if (! section || ! mouse_list) {
         return;
@@ -1753,10 +1774,10 @@ Config::parseButtons(CfgParser::Entry *section, std::list<ActionEvent>* mouse_li
 
 // frame border configuration
 
-list<ActionEvent>*
+vector<ActionEvent>*
 Config::getBorderListFromPosition(uint pos)
 {
-    list<ActionEvent> *ret = 0;
+    vector<ActionEvent> *ret = 0;
 
     switch (pos) {
     case BORDER_TOP_LEFT:
@@ -1788,10 +1809,10 @@ Config::getBorderListFromPosition(uint pos)
     return ret;
 }
 
-list<ActionEvent>*
+vector<ActionEvent>*
 Config::getEdgeListFromPosition(uint pos)
 {
-    list<ActionEvent> *ret = 0;
+    vector<ActionEvent> *ret = 0;
 
     switch (pos) {
     case SCREEN_EDGE_TOP:
