@@ -12,7 +12,6 @@
 
 #include <algorithm>
 #include <cstdio>
-#include <cwchar>
 #include <functional>
 #include <iostream>
 
@@ -20,6 +19,7 @@ extern "C" {
 #include <X11/Xatom.h>
 }
 
+#include "Debug.hh"
 #include "PWinObj.hh"
 #include "PDecor.hh"
 #include "Frame.hh"
@@ -35,21 +35,13 @@ extern "C" {
 #include "Workspaces.hh"
 #include "WindowManager.hh"
 #include "KeyGrabber.hh"
-#include "PMenu.hh"
 #include "Theme.hh"
 
-using std::cerr;
-using std::endl;
-using std::find;
-using std::list;
-using std::mem_fun;
 using std::string;
-using std::vector;
 using std::wstring;
-using std::swprintf;
 
-list<Frame*> Frame::_frame_list = list<Frame*>();
-vector<uint> Frame::_frameid_list = vector<uint>();
+vector<Frame*> Frame::_frames;
+vector<uint> Frame::_frameid_list;
 
 Frame* Frame::_tag_frame = 0;
 bool Frame::_tag_behind = false;
@@ -80,7 +72,7 @@ Frame::Frame(Client *client, AutoProperty *ap)
     // get unique id of the frame, if the client didn't have an id
     if (! WindowManager::instance()->isStartup()) {
         long id;
-        if (AtomUtil::getLong(client->getWindow(), Atoms::getAtom(PEKWM_FRAME_ID), id)) {
+        if (X11::getLong(client->getWindow(), PEKWM_FRAME_ID, id)) {
             _id = id;
         }
 
@@ -143,7 +135,7 @@ Frame::Frame(Client *client, AutoProperty *ap)
 
     // still need a position?
     if (place) {
-        Workspaces::placeWo(this, client->getTransientClientWindow());
+        Workspaces::placeWo(this, client->getTransientForClientWindow());
     }
 
     _old_gm = _gm;
@@ -161,7 +153,7 @@ Frame::Frame(Client *client, AutoProperty *ap)
 
     // I add these to the list before I insert the client into the frame to
     // be able to skip an extra updateClientList
-    _frame_list.push_back(this);
+    _frames.push_back(this);
     WindowManager::instance()->addToMRUBack(this);
     Workspaces::insert(this);
 
@@ -192,7 +184,7 @@ Frame::~Frame(void)
     // remove from lists
     _wo_map.erase(_window);
     woListRemove(this);
-    _frame_list.remove(this);
+    _frames.erase(std::remove(_frames.begin(), _frames.end(), this), _frames.end());
     Workspaces::remove(this);
     WindowManager::instance()->removeFromFrameList(this);
     if (_tag_frame == this) {
@@ -236,23 +228,6 @@ Frame::stick(void)
     PDecor::setWorkspace(Workspaces::getActive());
 }
 
-void
-Frame::raise(void)
-{
-    PDecor::raise();
-
-    if (_client->transient_size()) {
-        Frame *frame;
-        list<Client*>::iterator it(_client->transient_begin());
-        for (; it != _client->transient_end(); ++it) {
-            frame = static_cast<Frame*>((*it)->getParent());
-            if (frame != this && frame->getActiveClient() == *it) {
-                frame->raise();
-            }
-        }
-    }
-}
-
 //! @brief Sets workspace on frame, wrapper to allow autoproperty loading
 void
 Frame::setWorkspace(unsigned int workspace)
@@ -260,6 +235,11 @@ Frame::setWorkspace(unsigned int workspace)
     // Duplicate the behavior done in PDecor::setWorkspace to have a sane
     // value on _workspace and not NET_WM_STICKY_WINDOW.
     if (workspace != NET_WM_STICKY_WINDOW) {
+        // Check for DisallowedActions="SetWorkspace".
+        if (_client && ! _client->allowChangeWorkspace()) {
+            return;
+        }
+
         // First we set the workspace, then load autoproperties for possible
         // overrun of workspace and then set the workspace.
         _workspace = workspace;
@@ -271,14 +251,15 @@ Frame::setWorkspace(unsigned int workspace)
 }
 
 void
-Frame::setLayer(unsigned int layer)
+Frame::setLayer(Layer layer)
 {
     PDecor::setLayer(layer);
 
     if (_client) {
-        LayerObservation *observation = new LayerObservation(Layer(layer));
-        _client->notifyObservers(observation);
-        delete observation;
+        _client->setLayer(layer);
+
+        LayerObservation observation(static_cast<Layer>(layer));
+        _client->notifyObservers(&observation);
     }
 }
 
@@ -311,7 +292,7 @@ Frame::handleMotionEvent(XMotionEvent *ev)
         }
         
         if (pos != BORDER_NO_POS) {
-            list<ActionEvent> *bl = Config::instance()->getBorderListFromPosition(pos);
+            vector<ActionEvent> *bl = Config::instance()->getBorderListFromPosition(pos);
             ae = ActionHandler::findMouseAction(button, ev->state, MOUSE_EVENT_MOTION, bl);
         }
     }
@@ -336,14 +317,12 @@ Frame::handleEnterEvent(XCrossingEvent *ev)
     PDecor::handleEnterEvent(ev);
 
     ActionEvent *ae = 0;
-    list<ActionEvent> *al = 0;
+    vector<ActionEvent> *al = 0;
 
-    if (ev->window == getTitleWindow()) {
+    if (ev->window == getTitleWindow() || findButton(ev->window)) {
         al = Config::instance()->getMouseActionList(MOUSE_ACTION_LIST_TITLE_FRAME);
-
     } else if (ev->subwindow == _client->getWindow()) {
         al = Config::instance()->getMouseActionList(MOUSE_ACTION_LIST_CHILD_FRAME);
-
     } else {
         uint pos = getBorderPosition(ev->window);
         if (pos != BORDER_NO_POS) {
@@ -387,10 +366,7 @@ Frame::handleMapRequest(XMapRequestEvent *ev)
     }
 
     if (! _sticky && (_workspace != Workspaces::getActive())) {
-#ifdef DEBUG
-        cerr << __FILE__ << "@" << __LINE__ << ": "
-             << "Ignoring MapRequest, not on current workspace!" << endl;
-#endif // DEBUG
+        LOG("Ignoring MapRequest, not on current workspace!");
         return 0;
     }
 
@@ -402,8 +378,8 @@ Frame::handleMapRequest(XMapRequestEvent *ev)
 ActionEvent*
 Frame::handleUnmapEvent(XUnmapEvent *ev)
 {
-    list<PWinObj*>::iterator it(_child_list.begin());
-    for (; it != _child_list.end(); ++it) {
+    vector<PWinObj*>::const_iterator it(_children.begin());
+    for (; it != _children.end(); ++it) {
         if (*(*it) == ev->window) {
             (*it)->handleUnmapEvent(ev);
             break;
@@ -415,16 +391,16 @@ Frame::handleUnmapEvent(XUnmapEvent *ev)
 
 // END - PWinObj interface.
 
-void
-Frame::handleShapeEvent(XAnyEvent *ev)
-{
 #ifdef HAVE_SHAPE
-    if (! _client || (ev->window != _client->getWindow())) {
+void
+Frame::handleShapeEvent(XShapeEvent *ev)
+{
+    if (! _client || ev->window != _client->getWindow()) {
         return;
     }
-    _client->setShaped(setShape());
-#endif // HAVE_SHAPE
+    applyBorderShape(ev->kind);
 }
+#endif // HAVE_SHAPE
 
 // START - PDecor interface.
 
@@ -449,10 +425,10 @@ Frame::getActiveClient(void)
 
 //! @brief Adds child to the frame.
 void
-Frame::addChild(PWinObj *child, std::list<PWinObj*>::iterator *it)
+Frame::addChild(PWinObj *child, vector<PWinObj*>::iterator *it)
 {
     PDecor::addChild(child, it);
-    AtomUtil::setLong(child->getWindow(), Atoms::getAtom(PEKWM_FRAME_ID), _id);
+    X11::setLong(child->getWindow(), PEKWM_FRAME_ID, _id);
     child->lower();
 
     Client *client = dynamic_cast<Client*>(child);
@@ -464,12 +440,12 @@ Frame::addChild(PWinObj *child, std::list<PWinObj*>::iterator *it)
  * Add child preserving order from the previous pekwm run.
  */
 void
-Frame::addChildOrdered (Client *child)
+Frame::addChildOrdered(Client *child)
 {
     Client *client;
     
-    list<PWinObj*>::iterator it(_child_list.begin());
-    for (; it != _child_list.end(); ++it) {
+    vector<PWinObj*>::iterator it(_children.begin());
+    for (; it != _children.end(); ++it) {
         client = static_cast<Client*>(*it);
         if (child->getInitialFrameOrder() < client->getInitialFrameOrder()) {
             break;
@@ -511,11 +487,14 @@ Frame::activateChild(PWinObj *child)
     }
     PDecor::activateChild(child);
 
-    // setShape uses current active child, so we need to activate the
-    // child before setting shape
+#ifdef HAVE_SHAPE
+    // applyBorderShape() uses current active child, so we need to activate
+    // the child before setting shape
     if (X11::hasExtensionShape()) {
-        _client->setShaped(setShape());
+        applyBorderShape(ShapeBounding);
+        applyBorderShape(ShapeInput);
     }
+#endif // HAVE_SHAPE
 
     setOpacity(_client);
 
@@ -540,8 +519,8 @@ Frame::updatedChildOrder(void)
     titleClear();
 
     Client *client;
-    list<PWinObj*>::iterator it(_child_list.begin());
-    for (long num = 0; it != _child_list.end(); ++num, ++it) {
+    vector<PWinObj*>::const_iterator it(_children.begin());
+    for (long num = 0; it != _children.end(); ++num, ++it) {
         client = static_cast<Client*>(*it);
         client->setPekwmFrameOrder(num);
         titleAdd(client->getTitle());
@@ -558,10 +537,12 @@ Frame::updatedActiveChild(void)
 {
     titleSetActive(0);
 
-    list<PWinObj*>::iterator it(_child_list.begin());
-    for (uint i = 0; it != _child_list.end(); ++i, ++it) {
-        static_cast<Client*>(*it)->setPekwmFrameActive(_child == *it);
-        if (_child == *it) {
+    Client *client;
+    uint size = _children.size();
+    for (uint i = 0; i < size; ++i) {
+        client = static_cast<Client*>(_children[i]);
+        client->setPekwmFrameActive(_child == client);
+        if (_child == client) {
             titleSetActive(i);
         }
     }
@@ -600,6 +581,12 @@ void
 Frame::setShaded(StateAction sa)
 {
     bool shaded = isShaded();
+
+    // Check for DisallowedActions="Shade"
+    if (_client && ! _client->allowShade()) {
+        sa = STATE_UNSET;
+    }
+
     PDecor::setShaded(sa);
     if (shaded != isShaded()) {
         _client->setShade(isShaded());
@@ -695,10 +682,9 @@ Frame::setId(uint id)
 {
     _id = id;
 
-    list<PWinObj*>::iterator it(_child_list.begin());
-    long atom = Atoms::getAtom(PEKWM_FRAME_ID);
-    for (; it != _child_list.end(); ++it) {
-        AtomUtil::setLong((*it)->getWindow(), atom, id);
+    vector<PWinObj*>::const_iterator it(_children.begin());
+    for (; it != _children.end(); ++it) {
+        X11::setLong((*it)->getWindow(), PEKWM_FRAME_ID, id);
     }
 }
 
@@ -800,8 +786,8 @@ Frame::findFrameFromWindow(Window win)
         return 0;
     }
 
-    list<Frame*>::iterator it(_frame_list.begin());
-    for(; it !=_frame_list.end(); ++it) {
+    vector<Frame*>::const_iterator it(_frames.begin());
+    for(; it !=_frames.end(); ++it) {
         if (win == (*it)->getWindow()) { // operator == does more than that
             return (*it);
         }
@@ -816,8 +802,8 @@ Frame::findFrameFromWindow(Window win)
 Frame*
 Frame::findFrameFromID(uint id)
 {
-    list<Frame*>::iterator it(Frame::frame_begin());
-    for (; it != Frame::frame_end(); ++it) {
+    vector<Frame*>::const_iterator it(_frames.begin());
+    for (; it != _frames.end(); ++it) {
         if ((*it)->getId() == id) {
             return (*it);
         }
@@ -826,7 +812,6 @@ Frame::findFrameFromID(uint id)
     return 0;
 }
 
-//! @brief
 void
 Frame::setupAPGeometry(Client *client, AutoProperty *ap)
 {
@@ -835,7 +820,7 @@ Frame::setupAPGeometry(Client *client, AutoProperty *ap)
     // get client geometry
     if (ap->isMask(AP_CLIENT_GEOMETRY)) {
         Geometry gm(client->_gm);
-        applyAPGeometry(gm, ap->client_gm, ap->client_gm_mask);
+        applyGeometry(gm, ap->client_gm, ap->client_gm_mask);
 
         if (ap->client_gm_mask&(XValue|YValue)) {
             moveChild(gm.x, gm.y);
@@ -847,28 +832,29 @@ Frame::setupAPGeometry(Client *client, AutoProperty *ap)
 
     // get frame geometry
     if (ap->isMask(AP_FRAME_GEOMETRY)) {
-        applyAPGeometry(_gm, ap->frame_gm, ap->frame_gm_mask);
-        if (ap->frame_gm_mask&(XValue|YValue)) {
-            move(_gm.x, _gm.y);
-        }
-        if (ap->frame_gm_mask&(WidthValue|HeightValue)) {
-            resize(_gm.width, _gm.height);
-        }
+        setGeometry(ap->frame_gm, ap->frame_gm_mask);
     }
 }
 
-//! @brief Apply autoproperties geometry.
+void
+Frame::applyGeometry(Geometry &gm, const Geometry &ap_gm, int mask)
+{
+    applyGeometry(gm, ap_gm, mask, X11::getScreenGeometry());
+}
+
+//! @brief Apply geometry.
 //! @param gm Geometry to modify.
 //! @param ap_gm Geometry to get values from.
 //! @param mask Geometry mask.
+//! @param screen_gm Geometry of the screen/head for position.
 void
-Frame::applyAPGeometry(Geometry &gm, const Geometry &ap_gm, int mask)
+Frame::applyGeometry(Geometry &gm, const Geometry &ap_gm, int mask, const Geometry &screen_gm)
 {
     // Read size before position so negative position works, if size is
     // < 1 consider it to be full screen size.
     if (mask&WidthValue) {
         if (ap_gm.width < 1) {
-            gm.width = X11::getWidth();
+            gm.width = screen_gm.width;
         } else {
             gm.width = ap_gm.width;
         }
@@ -876,6 +862,7 @@ Frame::applyAPGeometry(Geometry &gm, const Geometry &ap_gm, int mask)
     
     if (mask&HeightValue) {
         if (ap_gm.height < 1) {
+            gm.height = screen_gm.height;
         } else {
             gm.height = ap_gm.height;
         }
@@ -883,15 +870,15 @@ Frame::applyAPGeometry(Geometry &gm, const Geometry &ap_gm, int mask)
 
     // Read position
     if (mask&XValue) {
-        gm.x = ap_gm.x;
+        gm.x = screen_gm.x + ap_gm.x;
         if (mask&XNegative) {
-            gm.x += X11::getWidth() - gm.width;
+            gm.x += screen_gm.width - gm.width;
         }
     }
     if (mask&YValue) {
-        gm.y = ap_gm.y;
+        gm.y = screen_gm.y + ap_gm.y;
         if (mask&YNegative) {
-            gm.y += X11::getHeight() - gm.height;
+            gm.y += screen_gm.height - gm.height;
         }
     }
 }
@@ -909,7 +896,7 @@ Frame::findFrameID(void)
         _frameid_list.pop_back();
     } else {
         // No free, get next number (Frame is not in list when this is called.)
-        id = _frame_list.size() + 1;
+        id = _frames.size() + 1;
     }
 
     return id;
@@ -958,8 +945,8 @@ Frame::getClientDecorName(Client *client)
 void
 Frame::resetFrameIDs(void)
 {
-    list<Frame*>::iterator it(_frame_list.begin());
-    for (uint id = 1; it != _frame_list.end(); ++id, ++it) {
+    vector<Frame*>::const_iterator it(_frames.begin());
+    for (uint id = 1; it != _frames.end(); ++id, ++it) {
         (*it)->setId(id);
     }
 }
@@ -973,7 +960,7 @@ Frame::detachClient(Client *client)
         return;
     }
 
-    if (_child_list.size() > 1) {
+    if (_children.size() > 1) {
         removeChild(client);
 
         client->move(_gm.x, _gm.y + borderTop());
@@ -1096,7 +1083,7 @@ Frame::doGroupingDrag(XMotionEvent *ev, Client *client, bool behind) // FIXME: r
 
                 // if we currently have focus and the frame exists after we remove
                 // this client we need to redraw it as unfocused
-                bool focus = behind ? false : (_child_list.size() > 1);
+                bool focus = behind ? false : (_children.size() > 1);
 
                 removeChild(client);
 
@@ -1111,7 +1098,7 @@ Frame::doGroupingDrag(XMotionEvent *ev, Client *client, bool behind) // FIXME: r
                     setFocused(false);
                 }
 
-            }  else if (_child_list.size() > 1) {
+            }  else if (_children.size() > 1) {
                 // if we have more than one client in the frame detach this one
                 removeChild(client);
 
@@ -1368,6 +1355,34 @@ Frame::recalcResizeDrag(int nx, int ny, bool left, bool top)
     _gm.y = top ? (_click_y - _gm.height) : _click_y;
 }
 
+void
+Frame::moveToHead(int head_nr)
+{
+    int curr_head_nr = getNearestHead();
+    if (curr_head_nr == head_nr || head_nr >= X11::getNumHeads()) {
+        return;
+    }
+
+    Geometry old_gm, new_gm;
+    X11::getHeadInfoWithEdge(curr_head_nr, old_gm);
+    X11::getHeadInfoWithEdge(head_nr, new_gm);
+
+    // Ensure the window fits in the new head.
+    _gm.x = new_gm.x + (_gm.x - old_gm.x);
+    _gm.y = new_gm.y + (_gm.y - old_gm.y);
+    _gm.width = std::min(_gm.width, new_gm.width);
+    _gm.height = std::min(_gm.height, new_gm.height);
+
+    if ((_gm.x + _gm.width) > (new_gm.x + new_gm.width)) {
+        _gm.x = new_gm.x + new_gm.width - _gm.width;
+    }
+    if ((_gm.y + _gm.height) > (new_gm.y + new_gm.height)) {
+        _gm.y = new_gm.y + new_gm.height - _gm.height;
+    }
+
+    moveResize(_gm.x, _gm.y, _gm.width, _gm.height);
+}
+
 //! @brief Moves the Frame to the screen edge ori ( considering struts )
 void
 Frame::moveToEdge(OrientationType ori)
@@ -1443,8 +1458,8 @@ Frame::updateInactiveChildInfo(void)
         return;
     }
     
-    list<PWinObj*>::iterator it(_child_list.begin());
-    for (; it != _child_list.end(); ++it) {
+    vector<PWinObj*>::const_iterator it(_children.begin());
+    for (; it != _children.end(); ++it) {
         if (*it != _client) {
             applyState(static_cast<Client*>(*it));
             (*it)->resize(getChildWidth(), getChildHeight());
@@ -1563,6 +1578,11 @@ Frame::setStateMaximized(StateAction sa, bool horz, bool vert, bool fill)
 void
 Frame::setStateFullscreen(StateAction sa)
 {
+    // Check for DisallowedActions="Fullscreen".
+    if (_client && ! _client->allowFullscreen()) {
+        sa = STATE_UNSET;
+    }
+
     if (! ActionUtil::needToggle(sa, _fullscreen)) {
         return;
     }
@@ -1599,7 +1619,7 @@ Frame::setStateFullscreen(StateAction sa)
     moveResize(_gm.x, _gm.y, _gm.width, _gm.height);
 
     // Re-stack window if fullscreen is above other windows.
-    if (Config::instance()->isFullscreenAbove()) {
+    if (Config::instance()->isFullscreenAbove() && _client->getLayer() != LAYER_DESKTOP) {
         setLayer(_fullscreen ? LAYER_ABOVE_DOCK : _non_fullscreen_layer);
         raise();
     }
@@ -1614,6 +1634,11 @@ Frame::setStateFullscreen(StateAction sa)
 void
 Frame::setStateSticky(StateAction sa)
 {
+    // Check for DisallowedActions="Stick".
+    if (_client && ! _client->allowStick()) {
+        sa = STATE_UNSET;
+    }
+
     if (ActionUtil::needToggle(sa, _sticky)) {
         stick();
     }
@@ -1659,9 +1684,8 @@ Frame::setStateDecorBorder(StateAction sa)
         _client->setBorder(hasBorder());
 
         // update the _PEKWM_FRAME_DECOR hint
-        AtomUtil::setLong(_client->getWindow(),
-                          Atoms::getAtom(PEKWM_FRAME_DECOR),
-                          _client->getDecorState());
+        X11::setLong(_client->getWindow(), PEKWM_FRAME_DECOR,
+                     _client->getDecorState());
     }
 }
 
@@ -1678,9 +1702,8 @@ Frame::setStateDecorTitlebar(StateAction sa)
     if (titlebar != hasTitlebar()) {
         _client->setTitlebar(hasTitlebar());
 
-        AtomUtil::setLong(_client->getWindow(),
-                          Atoms::getAtom(PEKWM_FRAME_DECOR),
-                          _client->getDecorState());
+        X11::setLong(_client->getWindow(), PEKWM_FRAME_DECOR,
+                     _client->getDecorState());
     }
 }
 
@@ -1688,6 +1711,11 @@ Frame::setStateDecorTitlebar(StateAction sa)
 void
 Frame::setStateIconified(StateAction sa)
 {
+    // Check for DisallowedActions="Iconify".
+    if (_client && ! _client->allowIconify()) {
+        sa = STATE_UNSET;
+    }
+
     if (! ActionUtil::needToggle(sa, _iconified)) {
         return;
     }
@@ -1748,10 +1776,8 @@ Frame::setStateTitle(StateAction sa, Client *client, const std::wstring &title)
     }
 
     // Set PEKWM_TITLE atom to preserve title on client between sessions.
-    AtomUtil::setString(client->getWindow(),
-                        Atoms::getAtom(PEKWM_TITLE),
-                        Util::to_mb_str(client->getTitle()->getUser()));
-
+    X11::setString(client->getWindow(), PEKWM_TITLE,
+                   Util::to_mb_str(client->getTitle()->getUser()));
 
     renderTitle();
 }
@@ -1792,8 +1818,8 @@ Frame::getMaxBounds(int &max_x,int &max_r, int &max_y, int &max_b)
     f_r = getRX();
     f_b = getBY();
 
-    list<Frame*>::iterator it = _frame_list.begin();
-    for (; it != _frame_list.end(); ++it) {
+    vector<Frame*>::const_iterator it = _frames.begin();
+    for (; it != _frames.end(); ++it) {
         if (! (*it)->isMapped()) {
             continue;
         }
@@ -1820,7 +1846,33 @@ Frame::getMaxBounds(int &max_x,int &max_r, int &max_y, int &max_b)
     }
 }
 
-//! @brief
+void
+Frame::setGeometry(const std::string geometry, int head)
+{
+    Geometry gm;
+    int mask = XParseGeometry(geometry.c_str(), &gm.x, &gm.y, &gm.width, &gm.height);
+    setGeometry(gm, mask, head);
+}
+ 
+void
+Frame::setGeometry(const Geometry &geometry, int gm_mask, int head)
+{
+    Geometry screen_gm;
+    if (head == -1) {
+        screen_gm = X11::getScreenGeometry();
+    } else {
+        screen_gm = X11::getHeadGeometry(head);
+    }
+
+    applyGeometry(_gm, geometry, gm_mask, screen_gm);
+    if (gm_mask&(XValue|YValue)) {
+        move(_gm.x, _gm.y);
+    }
+    if (gm_mask&(WidthValue|HeightValue)) {
+        resize(_gm.width, _gm.height);
+    }
+}
+
 void
 Frame::growDirection(uint direction)
 {
@@ -1855,8 +1907,8 @@ Frame::growDirection(uint direction)
 void
 Frame::close(void)
 {
-    list<PWinObj*>::iterator it(_child_list.begin());
-    for (; it != _child_list.end(); ++it) {
+    vector<PWinObj*>::const_iterator it(_children.begin());
+    for (; it != _children.end(); ++it) {
         static_cast<Client*>(*it)->close();
     }
 }
@@ -2072,36 +2124,20 @@ Frame::handleConfigureRequest(XConfigureRequestEvent *ev, Client *client)
     handleConfigureRequestGeometry(ev, client);
 
     // update the stacking
-    if (! client->isCfgDeny(CFG_DENY_STACKING)) {
-        if (ev->value_mask&CWStackMode) {
-            if (ev->value_mask&CWSibling) {
-                switch(ev->detail) {
-                case Above:
-                    Workspaces::stackAbove(this, ev->above);
-                    break;
-                case Below:
-                    Workspaces::stackBelow(this, ev->above);
-                    break;
-                case TopIf:
-                case BottomIf:
-                    // FIXME: What does occlude mean?
-                    break;
-                }
-            } else {
-                switch(ev->detail) { // FIXME: Is this broken?
-                case Above:
-                    raise();
-                    break;
-                case Below:
-                    lower();
-                    break;
-                case TopIf:
-                case BottomIf:
-                    // FIXME: Why does the manual say that it should care about siblings
-                    // even if we don't have any specified?
-                    break;
-                }
-            }
+    if (! client->isCfgDeny(CFG_DENY_STACKING) && ev->value_mask&CWStackMode) {
+        // ev->value_mask&CWSibling should not happen
+        switch(ev->detail) { // FIXME: Is this broken?
+        case Above:
+            raise();
+            break;
+        case Below:
+            lower();
+            break;
+        case TopIf:
+        case BottomIf:
+            // FIXME: Why does the manual say that it should care about siblings
+            // even if we don't have any specified?
+            break;
         }
     }
 }
@@ -2122,7 +2158,7 @@ Frame::handleConfigureRequestGeometry(XConfigureRequestEvent *ev, Client *client
     if (! client->isCfgDeny(CFG_DENY_SIZE)
         && (ev->value_mask & (CWWidth|CWHeight))) {
         resizeChild(ev->width, ev->height);
-        _client->setShaped(setShape());
+        applyBorderShape();
         change_geometry = true;
     }
 
@@ -2177,9 +2213,9 @@ Frame::isRequestGeometryFullscreen(XConfigureRequestEvent *ev, Client *client)
 void
 Frame::handleClientMessage(XClientMessageEvent *ev, Client *client)
 {
-    if (ev->message_type == Atoms::getAtom(STATE)) {
+    if (ev->message_type == X11::getAtom(STATE)) {
         handleClientStateMessage(ev, client);
-    } else if (ev->message_type == Atoms::getAtom(NET_ACTIVE_WINDOW)) {
+    } else if (ev->message_type == X11::getAtom(NET_ACTIVE_WINDOW)) {
         if (! client->isCfgDeny(CFG_DENY_ACTIVE_WINDOW)) {
             // Active child if it's not the active child
             if (client != _client) {
@@ -2199,13 +2235,13 @@ Frame::handleClientMessage(XClientMessageEvent *ev, Client *client)
             raise();
             giveInputFocus();
         }
-    } else if (ev->message_type == Atoms::getAtom(NET_CLOSE_WINDOW)) {
+    } else if (ev->message_type == X11::getAtom(NET_CLOSE_WINDOW)) {
         client->close();
-    } else if (ev->message_type == Atoms::getAtom(NET_WM_DESKTOP)) {
+    } else if (ev->message_type == X11::getAtom(NET_WM_DESKTOP)) {
         if (client == _client) {
             setWorkspace(ev->data.l[0]);
         }
-    } else if (ev->message_type == Atoms::getAtom(WM_CHANGE_STATE) &&
+    } else if (ev->message_type == X11::getAtom(WM_CHANGE_STATE) &&
                (ev->format == 32) && (ev->data.l[0] == IconicState)) {
         if (client == _client) {
             iconify();
@@ -2279,33 +2315,33 @@ Frame::handleStateAtom(StateAction sa, Atom atom, Client *client)
 void
 Frame::handleCurrentClientStateAtom(StateAction sa, Atom atom, Client *client)
 {
-    if (atom == Atoms::getAtom(STATE_STICKY)) {
+    if (atom == X11::getAtom(STATE_STICKY)) {
         setStateSticky(sa);
     }
-    if (atom == Atoms::getAtom(STATE_MAXIMIZED_HORZ)
+    if (atom == X11::getAtom(STATE_MAXIMIZED_HORZ)
             && ! client->isCfgDeny(CFG_DENY_STATE_MAXIMIZED_HORZ)) {
         setStateMaximized(sa, true, false, false);
     }
-    if (atom == Atoms::getAtom(STATE_MAXIMIZED_VERT)
+    if (atom == X11::getAtom(STATE_MAXIMIZED_VERT)
             && ! client->isCfgDeny(CFG_DENY_STATE_MAXIMIZED_VERT)) {
         setStateMaximized(sa, false, true, false);
     }
-    if (atom == Atoms::getAtom(STATE_SHADED)) {
+    if (atom == X11::getAtom(STATE_SHADED)) {
         setShaded(sa);
     }
-    if (atom == Atoms::getAtom(STATE_HIDDEN)
+    if (atom == X11::getAtom(STATE_HIDDEN)
             && ! client->isCfgDeny(CFG_DENY_STATE_HIDDEN)) {
         setStateIconified(sa);
     }
-    if (atom == Atoms::getAtom(STATE_FULLSCREEN)
+    if (atom == X11::getAtom(STATE_FULLSCREEN)
             && ! client->isCfgDeny(CFG_DENY_STATE_FULLSCREEN)) {
         setStateFullscreen(sa);
     }
-    if (atom == Atoms::getAtom(STATE_ABOVE)
+    if (atom == X11::getAtom(STATE_ABOVE)
             && ! client->isCfgDeny(CFG_DENY_STATE_ABOVE)) {
         setStateAlwaysOnTop(sa);
     }
-    if (atom == Atoms::getAtom(STATE_BELOW)
+    if (atom == X11::getAtom(STATE_BELOW)
             && ! client->isCfgDeny(CFG_DENY_STATE_BELOW)) {
         setStateAlwaysBelow(sa);
     }
@@ -2315,25 +2351,25 @@ Frame::handleCurrentClientStateAtom(StateAction sa, Atom atom, Client *client)
 void
 Frame::handlePropertyChange(XPropertyEvent *ev, Client *client)
 {
-    if (ev->atom == Atoms::getAtom(NET_WM_DESKTOP)) {
+    if (ev->atom == X11::getAtom(NET_WM_DESKTOP)) {
         if (client == _client) {
             long workspace;
 
-            if (AtomUtil::getLong(client->getWindow(),
-                                  Atoms::getAtom(NET_WM_DESKTOP), workspace)) {
+            if (X11::getLong(client->getWindow(),
+                             NET_WM_DESKTOP, workspace)) {
                 if (workspace != signed(_workspace))
                     setWorkspace(workspace);
             }
         }
-    } else if (ev->atom == Atoms::getAtom(NET_WM_STRUT)) {
+    } else if (ev->atom == X11::getAtom(NET_WM_STRUT)) {
         client->getStrutHint();
-    } else if (ev->atom == Atoms::getAtom(NET_WM_NAME) || ev->atom == XA_WM_NAME) {
+    } else if (ev->atom == X11::getAtom(NET_WM_NAME) || ev->atom == XA_WM_NAME) {
         handleTitleChange(client);
     } else if (ev->atom == XA_WM_NORMAL_HINTS) {
         client->getWMNormalHints();
     } else if (ev->atom == XA_WM_TRANSIENT_FOR) {
         client->getTransientForHint();
-    } else if (ev->atom == Atoms::getAtom(WM_HINTS)) {
+    } else if (ev->atom == X11::getAtom(WM_HINTS)) {
         bool nstate, ostate = client->demandsAttention();
         client->getWMHints();
         nstate = client->demandsAttention();
