@@ -17,13 +17,15 @@
 #include "CfgParser.hh"
 #include "Util.hh"
 
+extern "C" {
+#include <X11/Xutil.h>
+}
+
 #include <iostream>
 
 using std::cerr;
 using std::endl;
 using std::string;
-using std::list;
-using std::vector;
 using std::find;
 
 KeyGrabber* KeyGrabber::_instance = 0;
@@ -44,7 +46,7 @@ KeyGrabber::Chain::~Chain(void)
 void
 KeyGrabber::Chain::unload(void)
 {
-    list<KeyGrabber::Chain*>::iterator it(_chains.begin());
+    vector<KeyGrabber::Chain*>::iterator it(_chains.begin());
     for (; it != _chains.end(); ++it) {
         delete *it;
     }
@@ -56,7 +58,7 @@ KeyGrabber::Chain::unload(void)
 KeyGrabber::Chain*
 KeyGrabber::Chain::findChain(XKeyEvent *ev)
 {
-    list<KeyGrabber::Chain*>::iterator it(_chains.begin());
+    vector<KeyGrabber::Chain*>::iterator it(_chains.begin());
 
     for (; it != _chains.end(); ++it) {
         if ((((*it)->getMod() == MOD_ANY) || ((*it)->getMod() == ev->state)) &&
@@ -72,7 +74,7 @@ KeyGrabber::Chain::findChain(XKeyEvent *ev)
 ActionEvent*
 KeyGrabber::Chain::findAction(XKeyEvent *ev)
 {
-    list<ActionEvent>::iterator it(_keys.begin());
+    vector<ActionEvent>::iterator it(_keys.begin());
 
     for (; it != _keys.end(); ++it) {
         if (((it->mod == MOD_ANY) || (it->mod == ev->state)) &&
@@ -115,13 +117,13 @@ KeyGrabber::~KeyGrabber(void)
 bool
 KeyGrabber::load(const std::string &file, bool force)
 {
-    if (! force && ! Util::requireReload(_cfg_state, file)) {
+    if (! force && ! _cfg_files.requireReload(file)) {
         return false;
     }
 
     CfgParser key_cfg;
     if (! key_cfg.parse(file, CfgParserSource::SOURCE_FILE)) {
-        _cfg_state.clear();
+        _cfg_files.clear();
         if (! key_cfg.parse(SYSCONFDIR "/keys", CfgParserSource::SOURCE_FILE, true)) {
             cerr << __FILE__ << "@" << __LINE__ << "Error: no keyfile at " << file
                  << " or " << SYSCONFDIR "/keys" << endl;
@@ -130,9 +132,9 @@ KeyGrabber::load(const std::string &file, bool force)
     }
 
     if (key_cfg.is_dynamic_content()) {
-        _cfg_state.clear();
+        _cfg_files.clear();
     } else {
-        _cfg_state = key_cfg.get_file_list();
+        _cfg_files = key_cfg.getCfgFiles();
     }
 
     CfgParser::Entry *section;
@@ -264,13 +266,13 @@ KeyGrabber::parseMenuChain(CfgParser::Entry *section, KeyGrabber::Chain *chain)
 void
 KeyGrabber::grabKeys(Window win)
 {
-    const list<KeyGrabber::Chain*> chains = _global_chain.getChainList();
-    list<KeyGrabber::Chain*>::const_iterator c_it = chains.begin();
+    const vector<KeyGrabber::Chain*> &chains(_global_chain.getChains());
+    vector<KeyGrabber::Chain*>::const_iterator c_it = chains.begin();
     for (; c_it != chains.end(); ++c_it)
         grabKey(win, (*c_it)->getMod(), (*c_it)->getKey());
 
-    const list<ActionEvent> keys = _global_chain.getKeyList();
-    list<ActionEvent>::const_iterator k_it = keys.begin();
+    const vector<ActionEvent> &keys(_global_chain.getKeys());
+    vector<ActionEvent>::const_iterator k_it = keys.begin();
     for (; k_it != keys.end(); ++k_it)
         grabKey(win, k_it->mod, k_it->sym);
 }
