@@ -12,7 +12,6 @@
 
 #include "Config.hh"
 #include "ActionHandler.hh"
-#include "Atoms.hh"
 #include "ManagerWindows.hh"
 #include "Util.hh"
 
@@ -65,8 +64,8 @@ HintWO::HintWO(Window root, bool replace) throw (std::string&)
     XChangeWindowAttributes(X11::getDpy(), _window, CWEventMask|CWOverrideRedirect, &attr);
 
     // Set hints not being updated
-    AtomUtil::setString(_window, Atoms::getAtom(NET_WM_NAME), WM_NAME);
-    AtomUtil::setWindow(_window, Atoms::getAtom(NET_SUPPORTING_WM_CHECK), _window);
+    X11::setString(_window, NET_WM_NAME, WM_NAME);
+    X11::setWindow(_window, NET_SUPPORTING_WM_CHECK, _window);
 
     if (! claimDisplay(replace)) {
         throw string("unable to claim display");
@@ -95,7 +94,7 @@ HintWO::getTime(void)
 
     // Generate event on ourselves
     XChangeProperty(X11::getDpy(), _window,
-		    Atoms::getAtom(WM_CLASS), Atoms::getAtom(STRING),
+		    X11::getAtom(WM_CLASS), X11::getAtom(STRING),
                     8, PropModeAppend, 0, 0);
     XWindowEvent(X11::getDpy(), _window, PropertyChangeMask, &event);
 
@@ -190,10 +189,10 @@ HintWO::claimDisplayOwner(Window session_atom, Time timestamp)
 {
     XEvent event;
     // FIXME: One should use _root_wo here?
-    Window root = RootWindow(X11::getDpy(), DefaultScreen(X11::getDpy()));
+    Window root = X11::getRoot();
 
     event.xclient.type = ClientMessage;
-    event.xclient.message_type = Atoms::getAtom(MANAGER);
+    event.xclient.message_type = X11::getAtom(MANAGER);
     event.xclient.display = X11::getDpy();
     event.xclient.window = root;
     event.xclient.format = 32;
@@ -236,16 +235,18 @@ RootWO::RootWO(Window root)
 
     // Set hits on the hint window, these are not updated so they are
     // set in the constructor.
-    AtomUtil::setLong(_window, Atoms::getAtom(NET_WM_PID), static_cast<long>(getpid()));
-    AtomUtil::setString(_window, Atoms::getAtom(WM_CLIENT_MACHINE), Util::getHostname());
+    X11::setLong(_window, NET_WM_PID, static_cast<long>(getpid()));
+    X11::setString(_window, WM_CLIENT_MACHINE, Util::getHostname());
 
-    AtomUtil::setWindow(_window, Atoms::getAtom(NET_SUPPORTING_WM_CHECK), HintWO::instance()->getWindow());
-    Atoms::setEwmhAtomsSupport(_window);
-    AtomUtil::setLong(_window, Atoms::getAtom(NET_NUMBER_OF_DESKTOPS), Config::instance()->getWorkspaces());
-    AtomUtil::setLong(_window, Atoms::getAtom(NET_CURRENT_DESKTOP), 0);
+    X11::setWindow(_window, NET_SUPPORTING_WM_CHECK, HintWO::instance()->getWindow());
+    X11::setEwmhAtomsSupport(_window);
+    X11::setLong(_window, NET_NUMBER_OF_DESKTOPS, Config::instance()->getWorkspaces());
+    X11::setLong(_window, NET_CURRENT_DESKTOP, 0);
 
-    long desktop_geometry[2] = { _gm.width, _gm.height };
-    AtomUtil::setLongs(_window, Atoms::getAtom(NET_DESKTOP_GEOMETRY), desktop_geometry, 2);
+    long desktop_geometry[2];
+    desktop_geometry[0] = _gm.width;
+    desktop_geometry[1] = _gm.height;
+    X11::setLongs(_window, NET_DESKTOP_GEOMETRY, desktop_geometry, 2);
 
     woListAdd(this);
     _wo_map[_window] = this;
@@ -257,8 +258,8 @@ RootWO::RootWO(Window root)
 RootWO::~RootWO(void)
 {
     // Remove atoms, PID will not be valid on shutdown.
-    AtomUtil::unsetProperty(_window, Atoms::getAtom(NET_WM_PID));
-    AtomUtil::unsetProperty(_window, Atoms::getAtom(WM_CLIENT_MACHINE));
+    X11::unsetProperty(_window, NET_WM_PID);
+    X11::unsetProperty(_window, WM_CLIENT_MACHINE);
 
     _wo_map.erase(_window);
     woListRemove(this);
@@ -339,8 +340,10 @@ RootWO::handleLeaveEvent(XCrossingEvent *ev)
 void
 RootWO::setEwmhWorkarea(const Geometry &workarea)
 {
-    long workarea_array[4] = { workarea.x, workarea.y, workarea.width, workarea.height };
-    AtomUtil::setLongs(_window, Atoms::getAtom(NET_WORKAREA), workarea_array, 4);
+    long workarea_array[4] = { workarea.x, workarea.y, 0, 0 };
+    workarea_array[2] = workarea.width;
+    workarea_array[3] = workarea.height;
+    X11::setLongs(_window, NET_WORKAREA, workarea_array, 4);
 }
 
 /**
@@ -351,7 +354,7 @@ RootWO::setEwmhWorkarea(const Geometry &workarea)
 void
 RootWO::setEwmhActiveWindow(Window win)
 {
-    AtomUtil::setWindow(X11::getRoot(), Atoms::getAtom(NET_ACTIVE_WINDOW), win);
+    X11::setWindow(X11::getRoot(), NET_ACTIVE_WINDOW, win);
 }
 
 /**
@@ -362,10 +365,9 @@ RootWO::readEwmhDesktopNames(void)
 {
     uchar *data;
     ulong data_length;
-    if (AtomUtil::getProperty(X11::getRoot(),
-                              Atoms::getAtom(NET_DESKTOP_NAMES),
-                              Atoms::getAtom(UTF8_STRING),
-                              EXPECTED_DESKTOP_NAMES_LENGTH, &data, &data_length)) {
+    if (X11::getProperty(X11::getRoot(), NET_DESKTOP_NAMES,
+                         X11::getAtom(UTF8_STRING),
+                         EXPECTED_DESKTOP_NAMES_LENGTH, &data, &data_length)) {
         Config::instance()->setDesktopNamesUTF8(reinterpret_cast<char *>(data), data_length);
 
         XFree(data);
@@ -383,8 +385,8 @@ RootWO::setEwmhDesktopNames(void)
     Config::instance()->getDesktopNamesUTF8(&desktopnames, &length);
 
     if (desktopnames) {
-        AtomUtil::setUtf8StringArray(X11::getRoot(),
-                                     Atoms::getAtom(NET_DESKTOP_NAMES), desktopnames, length);
+        X11::setUtf8StringArray(X11::getRoot(), NET_DESKTOP_NAMES,
+                                desktopnames, length);
         delete [] desktopnames;
     }
 }
