@@ -10,18 +10,16 @@
 #include "config.h"
 #endif // HAVE_CONFIG_H
 
-#include <cassert>
-#include <cstdio>
-#include <cstdlib>
 #include <functional>
-#include <iostream>
-#include <cwchar>
+#include <algorithm>
+using std::mem_fun;
 
 extern "C" {
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
 }
 
+#include "Debug.hh"
 #include "Compat.hh"
 #include "Config.hh"
 #include "PWinObj.hh"
@@ -38,15 +36,10 @@ extern "C" {
 #include "PixmapHandler.hh"
 #include "Workspaces.hh"
 
-using std::cerr;
-using std::endl;
 using std::find;
-using std::list;
 using std::map;
-using std::mem_fun;
 using std::string;
 using std::vector;
-using std::swprintf;
 
 // PDecor::Button
 
@@ -87,7 +80,7 @@ PDecor::Button::~Button(void)
 ActionEvent*
 PDecor::Button::findAction(XButtonEvent *ev)
 {
-    list<ActionEvent>::iterator it(_data->begin());
+    vector<ActionEvent>::iterator it(_data->begin());
     for (; it != _data->end(); ++it) {
         if (it->mod == ev->state && it->sym == ev->button)
             return &*it;
@@ -113,17 +106,19 @@ PDecor::Button::setState(ButtonState state)
         texture->render(_bg, 0, 0, _gm.width, _gm.height);
 
 #ifdef HAVE_SHAPE
-        // Get shape mask
-        bool need_free;
-        Pixmap shape = _data->getTexture(state)->getMask(0, 0, need_free);
-        if (shape != None) {
-            X11::shapeSetMask(_window, shape);
-            if (need_free) {
-                ScreenResources::instance()->getPixmapHandler()->returnPixmap(shape);
+        if (_data->setShape()) {
+            // Get shape mask
+            bool need_free;
+            Pixmap shape = _data->getTexture(state)->getMask(0, 0, need_free);
+            if (shape != None) {
+                X11::shapeSetMask(_window, ShapeBounding, shape);
+                if (need_free) {
+                    ScreenResources::instance()->getPixmapHandler()->returnPixmap(shape);
+                }
+            } else {
+                XRectangle rect = {0 /* x */, 0 /* y */, (unsigned short)_gm.width, (unsigned short)_gm.height };
+                X11::shapeSetRect(_window, &rect);
             }
-        } else {
-            XRectangle rect = {0 /* x */, 0 /* y */, _gm.width, _gm.height };
-            X11::shapeSetRect(_window, &rect);
         }
 #endif // HAVE_SHAPE
         clear();
@@ -175,7 +170,7 @@ const string PDecor::DEFAULT_DECOR_NAME_BORDERLESS = string("BORDERLESS");
 const string PDecor::DEFAULT_DECOR_NAME_TITLEBARLESS = string("TITLEBARLESS");
 const string PDecor::DEFAULT_DECOR_NAME_ATTENTION = string("ATTENTION");
 
-list<PDecor*> PDecor::_pdecor_list = list<PDecor*>();
+vector<PDecor*> PDecor::_pdecors;
 
 //! @brief PDecor constructor
 //! @param dpy Display
@@ -195,7 +190,7 @@ PDecor::PDecor(Theme *theme,
       _maximized_vert(false), _maximized_horz(false),
       _fullscreen(false), _skip(0), _data(0), 
       _border(true), _titlebar(true), _shaded(false),
-      _need_shape(false), _need_client_shape(false),
+      _need_shape(false),
       _dirty_resized(true), _real_height(1),
       _title_wo(), _title_bg(None),
       _title_active(0), _titles_left(0), _titles_right(1)
@@ -216,7 +211,7 @@ PDecor::PDecor(Theme *theme,
     createParentWindow(window_params);
     if (window_params.mask & CWColormap) {
         window_params.depth = X11::getDepth();
-        window_params.visual = X11::getVisual()->getXVisual();
+        window_params.visual = X11::getVisual();
         window_params.attr.colormap = X11::getColormap();
     }
     createTitle(window_params);
@@ -228,7 +223,7 @@ PDecor::PDecor(Theme *theme,
     // map title and border windows
     XMapSubwindows(X11::getDpy(), _window);
 
-    _pdecor_list.push_back(this);
+    _pdecors.push_back(this);
 }
 
 /**
@@ -325,12 +320,10 @@ PDecor::createBorder(CreateWindowParams &params)
 //! @brief PDecor destructor
 PDecor::~PDecor(void)
 {
-    _pdecor_list.remove(this);
+    _pdecors.erase(std::remove(_pdecors.begin(), _pdecors.end(), this), _pdecors.end());
 
-    if (_child_list.size() > 0) {
-        while (_child_list.size() != 0) {
-            removeChild(_child_list.back(), false); // Don't call delete this.
-        }
+    while (! _children.empty()) {
+        removeChild(_children.back(), false); // Don't call delete this.
     }
 
     // Make things look smoother, buttons will be noticed as deleted
@@ -367,7 +360,7 @@ PDecor::mapWindow(void)
 {
     if (! _mapped) {
         PWinObj::mapWindow();
-        for_each(_child_list.begin(), _child_list.end(),
+        for_each(_children.begin(), _children.end(),
                  mem_fun(&PWinObj::mapWindow));
     }
 }
@@ -391,10 +384,10 @@ PDecor::unmapWindow(void)
 {
     if (_mapped) {
         if (_iconified) {
-            for_each(_child_list.begin(), _child_list.end(),
+            for_each(_children.begin(), _children.end(),
                      mem_fun(&PWinObj::iconify));
         } else {
-            for_each(_child_list.begin(), _child_list.end(),
+            for_each(_children.begin(), _children.end(),
                      mem_fun(&PWinObj::unmapWindow));
         }
         PWinObj::unmapWindow();
@@ -479,10 +472,10 @@ PDecor::moveResize(int x, int y, uint width, uint height)
                           getChildWidth(), getChildHeight());
 
         // The client window may have its window gravity set to something different
-        // than NorthWestGravity (see Xlib manual chapter 3.2.3). Therefore the 
+        // than NorthWestGravity (see Xlib manual chapter 3.2.3). Therefore the
         // X server may not move its top left corner along with the decoration.
-        // We correct these cases by calling alignChild(). It is called only for 
-        // _child and not all members of _child_list, because activateChild.*() 
+        // We correct these cases by calling alignChild(). It is called only for
+        // _child and not all members of _children, because activateChild.*()
         // does it itself.
         alignChild(_child);
     }
@@ -555,20 +548,16 @@ PDecor::setWorkspace(uint workspace)
 {
     if (workspace != NET_WM_STICKY_WINDOW) {
         if (workspace >= Workspaces::size()) {
-#ifdef DEBUG
-            cerr << __FILE__ << "@" << __LINE__ << ": "
-                 << "PDecor(" << this << ")::setWorkspace(" << workspace << ")"
-                 << endl << " *** workspace > number of workspaces:"
-                 << Workspaces::size () << endl;
-#endif // DEBUG
+            LOG("this == " << this << " workspace == " << workspace
+                << " >= number of workspaces == " << Workspaces::size());
             workspace = Workspaces::size() - 1;
         }
         _workspace = workspace;
     }
 
-    list<PWinObj*>::iterator it (_child_list.begin ());
-    for (; it != _child_list.end (); ++it) {
-        (*it)->setWorkspace (workspace);
+    vector<PWinObj*>::const_iterator it(_children.begin());
+    for (; it != _children.end(); ++it) {
+        (*it)->setWorkspace(workspace);
     }
 
     if (! _mapped && ! _iconified) {
@@ -587,10 +576,7 @@ PDecor::giveInputFocus(void)
     if (_mapped && _child) {
         _child->giveInputFocus();
     } else {
-#ifdef DEBUG
-        cerr << __FILE__ << "@" << __LINE__ << ": "
-             << "PDecor::giveInputFocus(" << this << ")" << endl << " *** reverting to root" << endl;
-#endif // DEBUG
+        LOG("this == " << this << " - reverting to root");
         PWinObj::getRootPWinObj()->giveInputFocus();
     }
 }
@@ -602,7 +588,7 @@ ActionEvent*
 PDecor::handleButtonPress(XButtonEvent *ev)
 {
     ActionEvent *ae = 0;
-    list<ActionEvent> *actions = 0;
+    vector<ActionEvent> *actions = 0;
 
     // Remove state modifiers from event
     X11::stripStateModifiers(&ev->state);
@@ -685,7 +671,7 @@ ActionEvent*
 PDecor::handleButtonRelease(XButtonEvent *ev)
 {
     ActionEvent *ae = 0;
-    list<ActionEvent> *actions = 0;
+    vector<ActionEvent> *actions = 0;
     MouseEventType mb = MOUSE_EVENT_RELEASE;
 
     // Remove state modifiers from event
@@ -819,20 +805,17 @@ void
 PDecor::addDecor(PDecor *decor)
 {
     if (this == decor) {
-#ifdef DEBUG
-        cerr << __FILE__ << "@" << __LINE__ << ": "
-             << "PDecor(" << this << ")::addDecor(" << decor << ")" << " *** this == decor" << endl;
-#endif // DEBUG
+        LOG("this == decor (" << this << ")");
         return;
     }
 
-    list<PWinObj*>::iterator it(decor->begin());
+    vector<PWinObj*>::const_iterator it(decor->begin());
     for (; it != decor->end(); ++it) {
         addChild(*it);
         (*it)->setWorkspace(_workspace);
     }
 
-    decor->_child_list.clear();
+    decor->_children.clear();
     delete decor;
 }
 
@@ -883,17 +866,17 @@ PDecor::loadDecor(void)
     if (! _data) {
         _data = _theme->getPDecorData(DEFAULT_DECOR_NAME);
     }
-    assert(_data);
+    LOG_IF(!_data, "_data == 0");
 
     // Load decor.
-    list<Theme::PDecorButtonData*>::iterator b_it(_data->buttonBegin());
+    vector<Theme::PDecorButtonData*>::const_iterator b_it(_data->buttonBegin());
     for (; b_it != _data->buttonEnd(); ++b_it) {
         uint width = std::max(static_cast<uint>(1), (*b_it)->getWidth() ? (*b_it)->getWidth() : getTitleHeight());
         uint height = std::max(static_cast<uint>(1), (*b_it)->getHeight() ? (*b_it)->getHeight() : getTitleHeight());
 
-        _button_list.push_back(new PDecor::Button(&_title_wo, *b_it, width, height));
-        _button_list.back()->mapWindow();
-        addChildWindow(_button_list.back()->getWindow());
+        _buttons.push_back(new PDecor::Button(&_title_wo, *b_it, width, height));
+        _buttons.back()->mapWindow();
+        addChildWindow(_buttons.back()->getWindow());
     }
 
     // Update title position.
@@ -906,8 +889,8 @@ PDecor::loadDecor(void)
     }
 
     // Update child positions.
-    list<PWinObj*>::iterator c_it(_child_list.begin());
-    for (; c_it != _child_list.end(); ++c_it) {
+    vector<PWinObj*>::const_iterator c_it(_children.begin());
+    for (; c_it != _children.end(); ++c_it) {
         alignChild(*c_it);
     }
 
@@ -934,20 +917,20 @@ PDecor::unloadDecor(void)
     // the current buttons.
     _button = 0;
 
-    list<PDecor::Button*>::iterator it(_button_list.begin());
-    for (; it != _button_list.end(); ++it) {
+    vector<PDecor::Button*>::const_iterator it(_buttons.begin());
+    for (; it != _buttons.end(); ++it) {
         removeChildWindow((*it)->getWindow());
         delete *it;
     }
-    _button_list.clear();
+    _buttons.clear();
 }
 
 //! @brief
 PDecor::Button*
 PDecor::findButton(Window win)
 {
-    list<PDecor::Button*>::iterator it(_button_list.begin());
-    for (; it != _button_list.end(); ++it) {
+    vector<PDecor::Button*>::const_iterator it(_buttons.begin());
+    for (; it != _buttons.end(); ++it) {
         if (**it == win) {
             return *it;
         }
@@ -960,28 +943,27 @@ PDecor::findButton(Window win)
 PWinObj*
 PDecor::getChildFromPos(int x)
 {
-    if (! _child_list.size() || (_child_list.size() != _title_list.size()))
+    if (_children.empty() || _children.size() != _titles.size()) {
         return 0;
-    if (_child_list.size() == 1)
-        return _child_list.front();
+    }
+    if (_children.size() == 1)
+        return _children.front();
 
     if (x < static_cast<int>(_titles_left)) {
-        return _child_list.front();
+        return _children.front();
     } else if (x > static_cast<int>(_title_wo.getWidth() - _titles_right)) {
-        return _child_list.back();
+        return _children.back();
     }
 
     PTexture *t_sep = _data->getTextureSeparator(getFocusedState(false));
     uint sepw = t_sep?t_sep->getWidth():0;
-    list<PWinObj*>::iterator c_it(_child_list.begin());
-    list<PDecor::TitleItem*>::iterator t_it(_title_list.begin());
 
     uint pos = _titles_left, xx = x;
-    for (uint i = 0; i < _title_list.size(); ++i, ++t_it, ++c_it) {
-        if (xx >= pos && xx <= pos + (*t_it)->getWidth() + sepw) {
-            return *c_it;
+    for (uint i = 0; i < _titles.size(); ++i) {
+        if (xx >= pos && xx <= pos + _titles[i]->getWidth() + sepw) {
+            return _children[i];
         }
-        pos += (*t_it)->getWidth() + sepw;
+        pos += _titles[i]->getWidth() + sepw;
     }
 
     return 0;
@@ -1065,13 +1047,13 @@ PDecor::getTitleHeight(void) const
 
 //! @brief Adds a child to the decor, reparenting the window
 void
-PDecor::addChild(PWinObj *child, std::list<PWinObj*>::iterator *it)
+PDecor::addChild(PWinObj *child, vector<PWinObj*>::iterator *it)
 {
     child->reparent(this, borderLeft(), borderTop() + getTitleHeight());
     if (it == 0) {
-        _child_list.push_back(child);
+        _children.push_back(child);
     } else {
-        _child_list.insert(*it, child);
+        _children.insert(*it, child);
     }
 
     updatedChildOrder();
@@ -1079,7 +1061,7 @@ PDecor::addChild(PWinObj *child, std::list<PWinObj*>::iterator *it)
     // Sync focused state if it is the first child, the child will be
     // activated later on. If there are children here already fit the
     // child into the decor.
-    if (_child_list.size() == 1) {
+    if (_children.size() == 1) {
         _focused = ! _focused;
         setFocused(! _focused);
     } else {
@@ -1098,21 +1080,18 @@ PDecor::removeChild(PWinObj *child, bool do_delete)
                     _gm.x + borderLeft(),
                     _gm.y + borderTop() + getTitleHeight());
 
-    list<PWinObj*>::iterator it(find(_child_list.begin(), _child_list.end(), child));
-    if (it == _child_list.end()) {
-#ifdef DEBUG
-        cerr << __FILE__ << "@" << __LINE__ << ": "
-             << "PDecor(" << this << ")::removeChild(" << child << ")" << endl
-             << " *** child not in _child_list, bailing out." << endl;
-#endif // DEBUG
+    vector<PWinObj*>::iterator it(find(_children.begin(), _children.end(), child));
+    if (it == _children.end()) {
+        LOG("this == " << this << " child == " << child
+            << " - child not in _child_list, bailing out");
         return;
     }
 
-    it = _child_list.erase(it);
+    it = _children.erase(it);
 
-    if (_child_list.size() > 0) {
+    if (! _children.empty()) {
         if (_child == child) {
-            if (it == _child_list.end()) {
+            if (it == _children.end()) {
                 --it;
             }
 
@@ -1157,30 +1136,36 @@ PDecor::getDecorInfo(wchar_t *buf, uint size)
 void
 PDecor::activateChildNum(uint num)
 {
-    if (num >= _child_list.size()) {
-#ifdef DEBUG
-        cerr << __FILE__ << "@" << __LINE__ << ": "
-             << "PDecor(" << this << ")::activeChildNum(" << num << ")" << endl
-             << " *** num > _child_list.size()" << endl;
-#endif // DEBUG
+    if (num >= _children.size()) {
+        LOG("this == " << this << " num == " << num
+             << " >= _children.size() == " << _children.size());
         return;
     }
 
-    list<PWinObj*>::iterator it(_child_list.begin());
-    for (uint i = 0; i < num; ++i, ++it);
-
-    activateChild(*it);
+    activateChild(_children[num]);
 }
 
 //! @brief
 void
 PDecor::activateChildRel(int off)
 {
-    PWinObj *child = getChildRel(off);
-    if (! child) {
-        child = _child_list.front();
+    vector<PWinObj*>::size_type cur=0, size = _children.size();
+    for (; cur < size; ++cur) {
+        if (_child == _children[cur]) {
+            break;
+        }
     }
-    activateChild(child);
+
+    if (cur == size) {
+        _child = _children.front();
+        cur = 0;
+    }
+
+    off = (off+signed(cur))%signed(size);
+    if (off < 0) {
+        off += size;
+    }
+    activateChild(_children[off]);
 }
 
 //! @brief Moves the current child off steps
@@ -1188,73 +1173,32 @@ PDecor::activateChildRel(int off)
 void
 PDecor::moveChildRel(int off)
 {
-    PWinObj *child = getChildRel(off);
-    if (! child || (child == _child)) {
-        return;
+    vector<PWinObj*>::size_type idx=0, size = _children.size();
+    for (; idx < size; ++idx) {
+        if (_child == _children[idx]) {
+            break;
+        }
     }
 
-    bool at_begin = (_child == _child_list.front());
-    bool at_end = (_child == _child_list.back());
-
-    // switch place
-    _child_list.remove(_child);
-    list<PWinObj*>::iterator it(find(_child_list.begin(), _child_list.end(), child));
-    if (off > 0) {
-        // when wrapping, we want to be able to insert at first place
-        if (at_begin || (*it != _child_list.front())) {
-            ++it;
+    if (idx == size) {
+        if (_children.empty()) {
+            ERR("_children is empty! off == " << off << " Please report.");
+            return;
         }
-
-        _child_list.insert(it, _child);
-    } else {
-        if (! at_end && (*it == _child_list.back())) {
-            ++it;
-        }
-        _child_list.insert(it, _child);
+        _child = _children.front();
+        idx = 0;
     }
 
+    off = (off+idx)%size;
+    if (off < 0) {
+        off += size;
+    }
+
+    _children.erase(std::remove(_children.begin(), _children.end(), _child), _children.end());
+    _children.insert(_children.begin()+off, _child);
     updatedChildOrder();
 }
 
-//! @brief
-PWinObj*
-PDecor::getChildRel(int off)
-{
-    if ((off == 0) || (_child_list.size() < 2)) {
-#ifdef DEBUG
-        cerr << __FILE__ << "@" << __LINE__ << ": "
-             << "PDecor(" << this << ")::getChildRel(" << off << ")" << endl
-             << " *** off == 0 or not enough children" << endl;
-#endif // DEBUG
-        return 0;
-    }
-
-    list<PWinObj*>::iterator it(find(_child_list.begin(), _child_list.end(), _child));
-    // if no active child, use first
-    if (it == _child_list.end()) {
-        it = _child_list.begin();
-    }
-
-    int dir = (off > 0) ? 1 : -1;
-    off = abs(off);
-
-    for (int i = 0; i < off; ++i) {
-        if (dir == 1) { // forward
-            if (++it == _child_list.end()) {
-                it = _child_list.begin();
-            }
-
-        } else { // backward
-            if (it == _child_list.begin()) {
-                it = _child_list.end();
-            }
-            --it;
-        }
-    }
-
-    return *it;
-}
-
 //! @brief Do move of Decor with the mouse.
 //! @param x_root X position of the pointer when event was triggered.
 //! @param y_root Y position of the pointer when event was triggered.
@@ -1413,7 +1357,7 @@ PDecor::doKeyboardMoveResize(void)
     bool outline = (! Config::instance()->getOpaqueMove() ||
                     ! Config::instance()->getOpaqueResize());
     ActionEvent *ae;
-    list<Action>::iterator it;
+    vector<Action>::iterator it;
 
     wchar_t buf[128];
     getDecorInfo(buf, 128);
@@ -1605,36 +1549,35 @@ PDecor::renderTitle(void)
     uint x = _titles_left; // Position
     uint pad_horiz =  _data->getPad(PAD_LEFT) + _data->getPad(PAD_RIGHT); // Amount of horizontal padding
 
-    list<PDecor::TitleItem*>::iterator it(_title_list.begin());
-    for (uint i = 0; it != _title_list.end(); ++i, ++it) {
+    uint size = _titles.size();
+    for (uint i = 0; i < size; ++i) {
         sel = (_title_active == i);
 
         // render tab
         _data->getTextureTab(getFocusedState(sel))->render(_title_bg, x, 0,
-                                                           (*it)->getWidth(), _title_wo.getHeight());
+                                                           _titles[i]->getWidth(),
+                                                           _title_wo.getHeight());
 
         font = getFont(getFocusedState(sel));
         font->setColor(_data->getFontColor(getFocusedState(sel)));
 
-        if ((*it)) {
-            PFont::TrimType trim = PFont::FONT_TRIM_MIDDLE;
-            if ((*it)->isCustom() || (*it)->isUserSet()) {
-                trim = PFont::FONT_TRIM_END;
-            }
-
-            font->draw(_title_bg,
-                       x + _data->getPad(PAD_LEFT), // X position
-                       _data->getPad(PAD_UP), // Y position
-                       (*it)->getVisible(), 0, // Text and max chars
-                       (*it)->getWidth() - pad_horiz, // Available width
-                       trim); // Type of trim
+        PFont::TrimType trim = PFont::FONT_TRIM_MIDDLE;
+        if (_titles[i]->isCustom() || _titles[i]->isUserSet()) {
+            trim = PFont::FONT_TRIM_END;
         }
 
+        font->draw(_title_bg,
+                   x + _data->getPad(PAD_LEFT), // X position
+                   _data->getPad(PAD_UP), // Y position
+                   _titles[i]->getVisible(), 0, // Text and max chars
+                   _titles[i]->getWidth() - pad_horiz, // Available width
+                   trim); // Type of trim
+
         // move to next tab (or separator if any)
-        x += (*it)->getWidth();
+        x += _titles[i]->getWidth();
 
         // draw separator
-        if ((_title_list.size() > 1) && (i < (_title_list.size() - 1))) {
+        if (size > 1 && i < size - 1) {
             t_sep->render(_title_bg, x, 0, 0, 0);
             x += t_sep->getWidth();
         }
@@ -1647,8 +1590,8 @@ PDecor::renderTitle(void)
 void
 PDecor::renderButtons(void)
 {
-    list<PDecor::Button*>::iterator it(_button_list.begin());
-    for (; it != _button_list.end(); ++it) {
+    vector<PDecor::Button*>::const_iterator it(_buttons.begin());
+    for (; it != _buttons.end(); ++it) {
         (*it)->setState(_focused ? BUTTON_STATE_FOCUSED : BUTTON_STATE_UNFOCUSED);
     }
 }
@@ -1726,7 +1669,7 @@ PDecor::setBorderShape(void)
                                                                   do_free);
         if (pix != None) {
             _need_shape = true;
-            X11::shapeSetMask(_border_win[it->first], pix);
+            X11::shapeSetMask(_border_win[it->first], ShapeBounding, pix);
             if (do_free) {
                 ScreenResources::instance()->getPixmapHandler()->returnPixmap(pix);
             }
@@ -1860,36 +1803,6 @@ PDecor::checkEdgeSnap(void)
 }
 
 
-//! @brief Set shapes of decor
-//! @return true if shape is on, else false.
-bool
-PDecor::setShape(void)
-{
-#ifdef HAVE_SHAPE
-    if (! _child) {
-        return false;
-    }
-
-    int num;
-    XRectangle *rect;
-
-    rect = X11::shapeGetRects(_child->getWindow(), &num);
-    // If there is more than one Rectangle it has an irregular shape.
-    _need_client_shape = (num > 1);
-
-    // Will set or unset shape depending on _need_shape and _need_client_shape.
-    applyBorderShape();
-
-    if (rect) {
-        XFree(rect);
-    }
-
-    return (num > 1);
-#else // !HAVE_SHAPE
-    return false;
-#endif // HAVE_SHAPE
-}
-
 /**
  * Move child into position with regards to title and border.
  */
@@ -1918,8 +1831,8 @@ PDecor::placeButtons(void)
     _titles_left = 0;
     _titles_right = 0;
 
-    list<PDecor::Button*>::iterator it(_button_list.begin());
-    for (; it != _button_list.end(); ++it) {
+    vector<PDecor::Button*>::const_iterator it(_buttons.begin());
+    for (; it != _buttons.end(); ++it) {
         if ((*it)->isLeft()) {
             (*it)->move(_titles_left, 0);
             _titles_left += (*it)->getWidth();
@@ -1990,21 +1903,22 @@ PDecor::placeBorder(void)
     applyBorderShape();
 }
 
+#ifdef HAVE_SHAPE
 /**
  * Apply shaping on the window based on the shape of the client and
  * the borders.
  */
 void
-PDecor::applyBorderShape(void)
+PDecor::applyBorderShape(int kind)
 {
-#ifdef HAVE_SHAPE
+    bool client_shape = _child?_child->hasShapeRegion(kind):false;
     XRectangle rect_square;
     rect_square.x = 0;
     rect_square.y = 0;
     rect_square.width = _gm.width;
     rect_square.height = _gm.height;
 
-    if (_need_shape || _need_client_shape) {
+    if ((_need_shape && kind == ShapeBounding) || client_shape) {
         // if we have tab min == 0 then we have full width title and place the
         // border ontop, else we put the border under the title
         uint bt_off = (_data->getTitleWidthMin() > 0) ? getTitleHeight() : 0;
@@ -2014,50 +1928,54 @@ PDecor::applyBorderShape(void)
                                     0, 0, _gm.width, _gm.height, 0, 0, 0);
 
         if (_child && ! _shaded) {
-            X11::shapeCombine(shape, borderLeft(), borderTop() + getTitleHeight(),
-                               _child->getWindow(), ShapeSet);
+            X11::shapeCombine(shape, kind,
+                              borderLeft(), borderTop() + getTitleHeight(),
+                              _child->getWindow(), ShapeSet);
         }
 
         // Apply border shape. Need to be carefull wheter or not to include it.
         if (_border
                 && ! (_shaded && bt_off) // Shaded in non-full-width mode removes border.
-                && ! (_need_client_shape)) { // Shaped clients should appear bordeless.
+                && ! client_shape) { // Shaped clients should appear bordeless.
             // top
             if (borderTop() > 0) {
-                X11::shapeCombine(shape, 0, bt_off, _border_win[BORDER_TOP_LEFT],
-                                  ShapeUnion);
+                X11::shapeCombine(shape, kind, 0, bt_off,
+                                  _border_win[BORDER_TOP_LEFT], ShapeUnion);
 
-                X11::shapeCombine(shape, borderTopLeft(), bt_off, _border_win[BORDER_TOP],
-                                  ShapeUnion);
+                X11::shapeCombine(shape, kind, borderTopLeft(), bt_off,
+                                  _border_win[BORDER_TOP], ShapeUnion);
 
-                X11::shapeCombine(shape, _gm.width - borderTopRight(), bt_off,
-                                  _border_win[BORDER_TOP_RIGHT], ShapeUnion);
+                X11::shapeCombine(shape, kind, _gm.width - borderTopRight(),
+                                  bt_off, _border_win[BORDER_TOP_RIGHT], ShapeUnion);
             }
 
             bool use_bt_off = bt_off || borderTop();
             // Left border
             if (borderLeft() > 0) {
-                X11::shapeCombine(shape, 0, 
+                X11::shapeCombine(shape, kind, 0,
                                   use_bt_off ? bt_off + borderTopLeftHeight() : getTitleHeight(),
                                   _border_win[BORDER_LEFT], ShapeUnion);
             }
 
             // Right border
             if (borderRight() > 0) {
-                X11::shapeCombine(shape, _gm.width - borderRight(), 
+                X11::shapeCombine(shape, kind, _gm.width - borderRight(),
                                   use_bt_off ? bt_off + borderTopRightHeight() : getTitleHeight(),
                                   _border_win[BORDER_RIGHT], ShapeUnion);
             }
 
             // bottom
             if (borderBottom() > 0) {
-                X11::shapeCombine(shape, 0, _gm.height - borderBottomLeftHeight(), 
+                X11::shapeCombine(shape, kind,
+                                  0, _gm.height - borderBottomLeftHeight(),
                                   _border_win[BORDER_BOTTOM_LEFT], ShapeUnion);
 
-                X11::shapeCombine(shape, borderBottomLeft(), _gm.height - borderBottom(),
+                X11::shapeCombine(shape, kind,
+                                  borderBottomLeft(), _gm.height - borderBottom(),
                                   _border_win[BORDER_BOTTOM], ShapeUnion);
 
-                X11::shapeCombine(shape, _gm.width - borderBottomRight(), 
+                X11::shapeCombine(shape, kind,
+                                  _gm.width - borderBottomRight(),
                                   _gm.height - borderBottomRightHeight(),
                                   _border_win[BORDER_BOTTOM_RIGHT],
                                   ShapeUnion);
@@ -2065,7 +1983,7 @@ PDecor::applyBorderShape(void)
         }
         if (_titlebar) {
             // apply title shape
-            X11::shapeCombine(shape, _title_wo.getX(), _title_wo.getY(),
+            X11::shapeCombine(shape, kind, _title_wo.getX(), _title_wo.getY(),
                               _title_wo.getWindow(), ShapeUnion);
         }
 
@@ -2075,16 +1993,15 @@ PDecor::applyBorderShape(void)
         X11::shapeIntersectRect(shape, &rect_square);
 
         // Apply the shape mask to the window
-        X11::shapeCombine(_window, 0, 0, shape, ShapeSet);
+        X11::shapeCombine(_window, kind, 0, 0, shape, ShapeSet);
 
         XDestroyWindow(X11::getDpy(), shape);
     } else {
-        // No shaping is required, apply a square shape to remove
-        // possible stale shaping.
-        X11::shapeSetRect(_window, &rect_square);
+        // Reinstate default region
+        X11::shapeSetMask(_window, kind, None);
     }
-#endif // HAVE_SHAPE
 }
+#endif // HAVE_SHAPE
 
 //! @brief Restacks child, title and border windows.
 void
@@ -2185,10 +2102,7 @@ PDecor::getBorderSize(BorderPosition pos, uint &width, uint &height)
         }
         break;
     default:
-#ifdef DEBUG
-        cerr << __FILE__ << "@" << __LINE__ << ": "
-             << "PDecor(" << this << ")::getBorderSize()" << endl << " *** invalid border position" << endl;
-#endif // DEBUG
+        LOG("this == " << this << " - invalid border position");
         width = 1;
         height = 1;
         break;
@@ -2217,8 +2131,8 @@ PDecor::calcTitleWidth(void)
 
         if (_data->isTitleWidthSymetric()) {
             // Symetric mode, get max tab width, multiply with number of tabs
-            list<PDecor::TitleItem*>::iterator it(_title_list.begin());
-            for (; it != _title_list.end(); ++it) {
+            vector<PDecor::TitleItem*>::const_iterator it(_titles.begin());
+            for (; it != _titles.end(); ++it) {
                 width = font->getWidth((*it)->getVisible());
                 if (width > width_max) {
                     width_max = width;
@@ -2226,19 +2140,19 @@ PDecor::calcTitleWidth(void)
             }
 
             width = width_max + _data->getPad(PAD_LEFT) + _data->getPad(PAD_RIGHT);
-            width *= _title_list.size();
+            width *= _titles.size();
 
         } else {
             // Asymetric mode, get individual widths
-            list<PDecor::TitleItem*>::iterator it(_title_list.begin());
-            for (; it != _title_list.end(); ++it) {
+            vector<PDecor::TitleItem*>::const_iterator it(_titles.begin());
+            for (; it != _titles.end(); ++it) {
                 width += font->getWidth((*it)->getVisible())
                     + _data->getPad(PAD_LEFT) + _data->getPad(PAD_RIGHT);
             }
         }
 
         // Add width of separators and buttons
-        width += (_title_list.size() - 1)
+        width += (_titles.size() - 1)
                  * _data->getTextureSeparator(getFocusedState(false))->getWidth();
         width += _titles_left + _titles_right;
 
@@ -2261,7 +2175,7 @@ PDecor::calcTitleWidth(void)
 void
 PDecor::calcTabsWidth(void)
 {
-    if (! _title_list.size()) {
+    if (! _titles.size()) {
         return;
     }
 
@@ -2291,17 +2205,17 @@ PDecor::calcTabsGetAvailAndTabWidth(uint &width_avail, uint &tab_width, int &off
     }
  
     // Remove separators if enough space is available
-    uint sep_width = (_title_list.size() - 1)
+    uint sep_width = (_titles.size() - 1)
         * _data->getTextureSeparator(getFocusedState(false))->getWidth();
     if (width_avail > sep_width) {
         width_avail -= sep_width;
     }
 
-    tab_width = width_avail / _title_list.size();
-    off = width_avail % _title_list.size();
+    tab_width = width_avail / _titles.size();
+    off = width_avail % _titles.size();
 }
 
-//! @brief Calculate tab width symetric, sets up _title_list widths.
+//! @brief Calculate tab width symetric, sets up _titles widths.
 void
 PDecor::calcTabsWidthSymetric(void)
 {
@@ -2310,8 +2224,8 @@ PDecor::calcTabsWidthSymetric(void)
     calcTabsGetAvailAndTabWidth(width_avail, tab_width, off);
 
     // Assign width to elements
-    list<PDecor::TitleItem*>::iterator it(_title_list.begin());
-    for (; it != _title_list.end(); ++it) {
+    vector<PDecor::TitleItem*>::const_iterator it(_titles.begin());
+    for (; it != _titles.end(); ++it) {
         (*it)->setWidth(tab_width + ((off-- > 0) ? 1 : 0));
     }
 }
@@ -2337,8 +2251,8 @@ PDecor::calcTabsWidthAsymetric(void)
 
     // 1. give tabs their required width.
     uint width_total = 0;
-    list<PDecor::TitleItem*>::iterator it(_title_list.begin());
-    for (; it != _title_list.end(); ++it) {
+    vector<PDecor::TitleItem*>::const_iterator it(_titles.begin());
+    for (; it != _titles.end(); ++it) {
         // This should set the tab width to be only the size needed
         width = font->getWidth((*it)->getVisible().c_str())
             + _data->getPad(PAD_LEFT) + _data->getPad(PAD_RIGHT) + ((off-- > 0) ? 1 : 0);
@@ -2361,9 +2275,9 @@ void
 PDecor::calcTabsWidthAsymetricShrink(uint width_avail, uint tab_width)
 {
     // 2. Tabs did not fit
-    uint tabs_left = _title_list.size();
-    list<PDecor::TitleItem*>::iterator it(_title_list.begin());
-    for (; it != _title_list.end(); ++it) {
+    uint tabs_left = _titles.size();
+    vector<PDecor::TitleItem*>::const_iterator it(_titles.begin());
+    for (; it != _titles.end(); ++it) {
         if ((*it)->getWidth() < tab_width) {
             // 3. Add width of tabs requiring less space to the average.
             tabs_left--;
@@ -2374,7 +2288,7 @@ PDecor::calcTabsWidthAsymetricShrink(uint width_avail, uint tab_width)
     // 4. Re-assign width equally to tabs using more than average
     tab_width = width_avail / tabs_left;
     uint off = width_avail % tabs_left;
-    for (it = _title_list.begin(); it != _title_list.end(); ++it) {
+    for (it = _titles.begin(); it != _titles.end(); ++it) {
         if ((*it)->getWidth() >= tab_width) {
             (*it)->setWidth(tab_width + ((off-- > 0) ? 1 : 0));
         }
