@@ -24,10 +24,9 @@ extern "C" {
 
 using std::cerr;
 using std::endl;
-using std::list;
 using std::string;
 
-list<PImageLoader*> PImage::_loader_list = list<PImageLoader*>();
+vector<PImageLoader*> PImage::_loaders;
 
 /**
  * PImage constructor, loads image if one is specified.
@@ -64,8 +63,8 @@ PImage::load(const std::string &file)
         return false;
     }
 
-    list<PImageLoader*>::iterator it(_loader_list.begin());
-    for (; it != _loader_list.end(); ++it) {
+    vector<PImageLoader*>::const_iterator it(_loaders.begin());
+    for (; it != _loaders.end(); ++it) {
         if (! strcasecmp((*it)->getExt(), ext.c_str())) {
             _data = (*it)->load(file, _width, _height, _has_alpha, _use_alpha);
             if (_data) {
@@ -315,7 +314,7 @@ PImage::drawAlphaFixed(Drawable dest, int x, int y, uint width, uint height, uch
     }
 
     // Get mask from visual
-    Visual *visual = X11::getVisual()->getXVisual();
+    Visual *visual = X11::getVisual();
     dest_image->red_mask = visual->red_mask;
     dest_image->green_mask = visual->green_mask;
     dest_image->blue_mask = visual->blue_mask;
@@ -424,7 +423,7 @@ PImage::createMask(uchar *data, uint width, uint height)
 
     // Create XImage
     XImage *ximage;
-    ximage = XCreateImage(X11::getDpy(), X11::getVisual()->getXVisual(),
+    ximage = XCreateImage(X11::getDpy(), X11::getVisual(),
                           1, ZPixmap, 0, 0, width, height, 32, 0);
     if (! ximage) {
         cerr << " *** WARNING: unable to create XImage!" << endl;
@@ -471,7 +470,7 @@ PImage::createXImage(uchar *data, uint width, uint height)
 {
     // Create XImage
     XImage *ximage;
-    ximage = XCreateImage(X11::getDpy(), X11::getVisual()->getXVisual(),
+    ximage = XCreateImage(X11::getDpy(), X11::getVisual(),
                           X11::getDepth(), ZPixmap, 0, 0,
                           width, height, 32, 0);
     if (! ximage) {
