@@ -1,6 +1,6 @@
 //
 // RegexString.cc for pekwm
-// Copyright ©  2003-2008 Claes Nästén <me{@}pekdon{.}net>
+// Copyright © 2003-2009 Claes Nästén <me{@}pekdon{.}net>
 //
 // This program is licensed under the GNU GPL.
 // See the LICENSE file for more information.
@@ -16,41 +16,37 @@
 #include "RegexString.hh"
 #include "Util.hh"
 
-#ifdef HAVE_GETTEXT
-#include <libintl.h>
-#define _(S) gettext(S)
-#else // !HAVE_GETTEXT
-#define _(S) S
-#endif // HAVE_GETTEXT
-
+using std::vector;
 using std::cerr;
 using std::endl;
-using std::list;
 using std::string;
 using std::wstring;
+using std::strtol;
+
+const char RegexString::SEPARATOR = '/';
 
 //! @brief RegexString constructor.
-RegexString::RegexString (void)
-    : _reg_ok(false), _ref_max(1)
+RegexString::RegexString(void)
+    : _reg_ok(false), _reg_inverted(false), _ref_max(1)
 {
 }
 
 //! @brief RegexString constructor with default search
-RegexString::RegexString (const std::wstring &str, bool full)
-  : _reg_ok(false), _ref_max(1)
+RegexString::RegexString(const std::wstring &str, bool full)
+  : _reg_ok(false), _reg_inverted(false), _ref_max(1)
 {
   parse_match(str, full);
 }
 
 //! @brief RegexString destructor.
-RegexString::~RegexString (void)
+RegexString::~RegexString(void)
 {
     free_regex();
 }
 
 //! @brief Simple ed s command lookalike.
 bool
-RegexString::ed_s (std::wstring &str)
+RegexString::ed_s(std::wstring &str)
 {
     if (! _reg_ok) {
         return false;
@@ -70,8 +66,8 @@ RegexString::ed_s (std::wstring &str)
     string result;
     uint ref, size;
 
-    list<RegexString::Part>::iterator it(_ref_list.begin ());
-    for (; it != _ref_list.end(); ++it) {
+    vector<RegexString::Part>::iterator it(_refs.begin());
+    for (; it != _refs.end(); ++it) {
         if (it->get_reference() >= 0) {
             ref = it->get_reference();
 
@@ -94,49 +90,49 @@ RegexString::ed_s (std::wstring &str)
 }
 
 //! @brief Parses match part of regular expression.
-//! @param or_match Expression.
+//! @param match Expression.
 //! @param full Full expression if true (including flags). Defaults to false.
 bool
-RegexString::parse_match(const std::wstring &or_match, bool full)
+RegexString::parse_match(const std::wstring &match, bool full)
 {
     // Free resources
     if (_reg_ok) {
         free_regex();
     }
 
-    if (or_match.size()) {
+    if (match.size()) {
         int flags = REG_EXTENDED;
         string expression;
         wstring expression_str;
 
-        if (full) {
-            // Full regular expression syntax, parse out flags etc
-            char sep = or_match[0];
+        // Full regular expression syntax, parse out flags etc
+        string::size_type pos;
+        if (match[0] == SEPARATOR
+            && (pos = match.find_last_of(SEPARATOR)) != string::npos) {
+            // Main expression
+            expression_str = match.substr(1, pos - 1);
 
-            string::size_type pos = or_match.find_last_of(sep);
-            if ((pos != 0) && (pos != string::npos)) {
-                // Main expression
-                expression_str = or_match.substr(1, pos - 1);
-
-                // Expression flags
-                for (string::size_type i = pos + 1; i < or_match.size(); ++i) {
-                    switch (or_match[i]) {
-                    case 'i':
-                        flags |= REG_ICASE;
-                        break;
-                    default:
-                        cerr << _("Invalid flag for regular expression.\n");
-                        break;
-                    }
+            // Expression flags
+            for (string::size_type i = pos + 1; i < match.size(); ++i) {
+                switch (match[i]) {
+                case 'i':
+                    flags |= REG_ICASE;
+                    break;
+                case '!':
+                    _reg_inverted = true;
+                    break;
+                default:
+                    cerr << "Invalid flag \"" << match[i] << "\" for regular expression." << endl;
+                    break;
                 }
-            } else {
-                cerr << _("Invalid format of regular expression.\n");
             }
 
             expression = Util::to_mb_str(expression_str);
-
         } else {
-            expression = Util::to_mb_str(or_match);
+            if (full) {
+                cerr << "Invalid format of regular expression, missing separator " << SEPARATOR << endl;
+            }
+            expression = Util::to_mb_str(match);
         }
 
         _reg_ok = ! regcomp(&_regex, expression.c_str(), flags);
@@ -152,7 +148,7 @@ RegexString::parse_match(const std::wstring &or_match, bool full)
 //! except \. References to sub expressions are made with \num. \0 Represents
 //! the part of the string that matched.
 bool
-RegexString::parse_replace(const std::wstring &or_replace)
+RegexString::parse_replace(const std::wstring &replace)
 {
     _ref_max = 0;
 
@@ -160,23 +156,23 @@ RegexString::parse_replace(const std::wstring &or_replace)
     wstring::size_type begin = 0, end = 0, last = 0;
 
     // Go through the string and split at \num points
-    while ((end = or_replace.find_first_of('\\', begin)) != string::npos) {
+    while ((end = replace.find_first_of('\\', begin)) != string::npos) {
         // Store string between references.
         if (end > last) {
-            part = or_replace.substr(last, end - last);
-            _ref_list.push_back(RegexString::Part(part));
+            part = replace.substr(last, end - last);
+            _refs.push_back(RegexString::Part(part));
         }
 
         // Get reference number.
-        for (begin = ++end; isdigit(or_replace[end]); end++)
+        for (begin = ++end; isdigit(replace[end]); end++)
             ;
 
         if (end > begin) {
             // Convert number and add item.
-            part = or_replace.substr(begin, end - last);
-            int ref = strtol(Util::to_mb_str(part).c_str(), NULL, 10);
+            part = replace.substr(begin, end - last);
+            int ref = strtol(Util::to_mb_str(part).c_str(), 0, 10);
             if (ref >= 0) {
-                _ref_list.push_back(RegexString::Part(L"", ref));
+                _refs.push_back(RegexString::Part(L"", ref));
                 if (ref > _ref_max) {
                     _ref_max = ref;
                 }
@@ -187,9 +183,9 @@ RegexString::parse_replace(const std::wstring &or_replace)
         begin = last + 1;
     }
 
-    if (begin < or_replace.size()) {
-        part = or_replace.substr(begin, or_replace.size() - begin);
-        _ref_list.push_back(RegexString::Part(part));
+    if (begin < replace.size()) {
+        part = replace.substr(begin, replace.size() - begin);
+        _refs.push_back(RegexString::Part(part));
     }
 
     _ref_max++;
@@ -199,33 +195,32 @@ RegexString::parse_replace(const std::wstring &or_replace)
 
 //! @brief Parses ed s style command. /from/to/
 bool
-RegexString::parse_ed_s(const std::wstring &or_ed_s)
+RegexString::parse_ed_s(const std::wstring &ed_s)
 {
-    if (or_ed_s.size() < 3) {
+    if (ed_s.size() < 3) {
         return false;
     }
 
-    wchar_t c_delimeter = or_ed_s[0];
+    wchar_t c_delimeter = ed_s[0];
     string::size_type middle, end;
 
     // Middle.
-    for (middle = 1; middle < or_ed_s.size(); middle++) {
-        if ((or_ed_s[middle] == c_delimeter)
-            && (or_ed_s[middle - 1] != '\\')) {
+    for (middle = 1; middle < ed_s.size(); middle++) {
+        if ((ed_s[middle] == c_delimeter) && (ed_s[middle - 1] != '\\')) {
             break;
         }
     }
 
     // End.
-    for (end = middle + 1; end < or_ed_s.size(); end++) {
-        if ((or_ed_s[end] == c_delimeter) && (or_ed_s[end - 1] != '\\')) {
+    for (end = middle + 1; end < ed_s.size(); end++) {
+        if ((ed_s[end] == c_delimeter) && (ed_s[end - 1] != '\\')) {
             break;
         }
     }
 
     wstring match, replace;
-    match = or_ed_s.substr(1, middle - 1);
-    replace = or_ed_s.substr(middle + 1, end - middle - 1);
+    match = ed_s.substr(1, middle - 1);
+    replace = ed_s.substr(middle + 1, end - middle - 1);
 
     parse_match(match);
     parse_replace(replace);
@@ -233,17 +228,18 @@ RegexString::parse_ed_s(const std::wstring &or_ed_s)
     return true;
 }
 
-//! @brief Matches RegexString against or_rhs, needs successfull parse_match.
+//! @brief Matches RegexString against rhs, needs successfull parse_match.
 bool
-RegexString::operator==(const std::wstring &or_rhs)
+RegexString::operator==(const std::wstring &rhs)
 {
     if (! _reg_ok) {
         return false;
     }
 
-    string rhs(Util::to_mb_str(or_rhs));
+    string mb_rhs(Util::to_mb_str(rhs));
+    bool match = ! regexec(&_regex, mb_rhs.c_str(), 0, 0, 0);
 
-    return ! regexec(&_regex, rhs.c_str(), 0, 0, 0);
+    return _reg_inverted ? ! match : match;
 }
 
 //! @brief Free resources used by RegexString.
@@ -254,4 +250,5 @@ RegexString::free_regex(void)
         regfree(&_regex);
         _reg_ok = false;
     }
+    _reg_inverted = false;
 }
