@@ -16,10 +16,8 @@
 #include "PTexturePlain.hh"
 #include "Util.hh"
 
-#include <vector>
 #include <iostream>
 
-using std::list;
 using std::vector;
 using std::map;
 using std::string;
@@ -62,9 +60,9 @@ PTexture*
 TextureHandler::getTexture(const std::string &texture)
 {
     // check for already existing entry
-    list<TextureHandler::Entry*>::iterator it(_texture_list.begin());
+    vector<TextureHandler::Entry*>::const_iterator it(_textures.begin());
 
-    for (; it != _texture_list.end(); ++it) {
+    for (; it != _textures.end(); ++it) {
         if (*(*it) == texture) {
             (*it)->incRef();
             return (*it)->getTexture();
@@ -79,7 +77,7 @@ TextureHandler::getTexture(const std::string &texture)
         TextureHandler::Entry *entry = new TextureHandler::Entry(texture, ptexture);
         entry->incRef();
 
-        _texture_list.push_back(entry);
+        _textures.push_back(entry);
     }
 
     return ptexture;
@@ -91,9 +89,9 @@ PTexture*
 TextureHandler::referenceTexture(PTexture *texture)
 {
     // Check for already existing entry
-    list<TextureHandler::Entry*>::iterator it(_texture_list.begin());
+    vector<TextureHandler::Entry*>::const_iterator it(_textures.begin());
 
-    for (; it != _texture_list.end(); ++it) {
+    for (; it != _textures.end(); ++it) {
         if ((*it)->getTexture() == texture) {
             (*it)->incRef();
             return texture;
@@ -104,7 +102,7 @@ TextureHandler::referenceTexture(PTexture *texture)
     TextureHandler::Entry *entry = new TextureHandler::Entry("", texture);
     entry->incRef();
 
-    _texture_list.push_back(entry);
+    _textures.push_back(entry);
 
     return texture;
 }
@@ -115,15 +113,15 @@ TextureHandler::returnTexture(PTexture *texture)
 {
     bool found = false;
 
-    list<TextureHandler::Entry*>::iterator it(_texture_list.begin());
-    for (; it != _texture_list.end(); ++it) {
+    vector<TextureHandler::Entry*>::iterator it(_textures.begin());
+    for (; it != _textures.end(); ++it) {
         if ((*it)->getTexture() == texture) {
             found = true;
 
             (*it)->decRef();
             if ((*it)->getRef() == 0) {
                 delete *it;
-                _texture_list.erase(it);
+                _textures.erase(it);
             }
             break;
         }
@@ -159,7 +157,7 @@ TextureHandler::parse(const std::string &texture)
             ptexture = parseSolidRaised(tok);
             break;
         case PTexture::TYPE_IMAGE:
-            ptexture = new PTextureImage(tok[1]);
+            ptexture = new PTextureImage(tok[0]);
             break;
         case PTexture::TYPE_NO:
         default:
