@@ -18,7 +18,6 @@
 #include "PWinObj.hh"
 #include "PFont.hh"
 #include "PTexture.hh"
-#include "ColorHandler.hh"
 #include "FontHandler.hh"
 #include "ImageHandler.hh"
 #include "TextureHandler.hh"
@@ -30,7 +29,6 @@
 using std::cerr;
 using std::endl;
 using std::string;
-using std::list;
 using std::vector;
 using std::map;
 
@@ -38,7 +36,7 @@ using std::map;
 
 //! @brief Theme::PDecorButtonData constructor.
 Theme::PDecorButtonData::PDecorButtonData(void)
-    : ThemeData(),
+    : ThemeData(), _shape(true),
       _left(false), _width(1), _height(1)
 {
     for (uint i = 0; i < BUTTON_STATE_NO; ++i) {
@@ -71,15 +69,20 @@ Theme::PDecorButtonData::load(CfgParser::Entry *section)
     CfgParser::iterator it(section->begin());
     for (; it != section->end(); ++it) {
         if (Config::instance()->parseActionEvent(*it, ae, BUTTONCLICK_OK, true)) {
-            _ae_list.push_back (ae);
+            _aes.push_back (ae);
         }
     }
 
     // Got some actions, consider it to be a valid button.
-    if (_ae_list.size() > 0) {
+    if (_aes.size() > 0) {
         TextureHandler *th = TextureHandler::instance();
         CfgParser::Entry *value;
 
+        value = section->find_entry("SETSHAPE");
+        if (value) {
+            _shape = Util::isTrue(value->get_value());
+        }
+
         value = section->find_entry("FOCUSED");
         if (value) {
             _texture[BUTTON_STATE_FOCUSED] = th->getTexture(value->get_value());
@@ -223,23 +226,23 @@ Theme::PDecorData::load(CfgParser::Entry *section)
     TextureHandler *th = TextureHandler::instance(); // convenience
 
     vector<string> tok;
-    list<CfgParserKey*> key_list;
+    vector<CfgParserKey*> keys;
     string value_pad, value_focused, value_unfocused;
 
-    key_list.push_back(new CfgParserKeyNumeric<int>("HEIGHT", _title_height, 10, 0));
-    key_list.push_back(new CfgParserKeyNumeric<int>("WIDTHMIN", _title_width_min, 0));
-    key_list.push_back(new CfgParserKeyNumeric<int>("WIDTHMAX", _title_width_max, 100, 0, 100));
-    key_list.push_back(new CfgParserKeyBool("WIDTHSYMETRIC", _title_width_symetric));
-    key_list.push_back(new CfgParserKeyBool("HEIGHTADAPT", _title_height_adapt));
-    key_list.push_back(new CfgParserKeyString("PAD", value_pad, "0 0 0 0", 7));
-    key_list.push_back(new CfgParserKeyString("FOCUSED", value_focused, "Empty", th->getLengthMin ()));
-    key_list.push_back(new CfgParserKeyString("UNFOCUSED", value_unfocused, "Empty", th->getLengthMin ()));
+    keys.push_back(new CfgParserKeyNumeric<int>("HEIGHT", _title_height, 10, 0));
+    keys.push_back(new CfgParserKeyNumeric<int>("WIDTHMIN", _title_width_min, 0));
+    keys.push_back(new CfgParserKeyNumeric<int>("WIDTHMAX", _title_width_max, 100, 0, 100));
+    keys.push_back(new CfgParserKeyBool("WIDTHSYMETRIC", _title_width_symetric));
+    keys.push_back(new CfgParserKeyBool("HEIGHTADAPT", _title_height_adapt));
+    keys.push_back(new CfgParserKeyString("PAD", value_pad, "0 0 0 0", 7));
+    keys.push_back(new CfgParserKeyString("FOCUSED", value_focused, "Empty", th->getLengthMin ()));
+    keys.push_back(new CfgParserKeyString("UNFOCUSED", value_unfocused, "Empty", th->getLengthMin ()));
 
     // Free up resources
-    title_section->parse_key_values(key_list.begin(), key_list.end());
+    title_section->parse_key_values(keys.begin(), keys.end());
 
-    for_each (key_list.begin(), key_list.end(), Util::Free<CfgParserKey*>());
-    key_list.clear();
+    for_each (keys.begin(), keys.end(), Util::Free<CfgParserKey*>());
+    keys.clear();
 
     // Handle parsed data.
     _texture_main[FOCUSED_STATE_FOCUSED] = th->getTexture(value_focused);
@@ -261,15 +264,15 @@ Theme::PDecorData::load(CfgParser::Entry *section)
 
     CfgParser::Entry *separator_section = title_section->find_section ("SEPARATOR");
     if (separator_section) {
-        key_list.push_back(new CfgParserKeyString("FOCUSED", value_focused, "Empty", th->getLengthMin()));
-        key_list.push_back(new CfgParserKeyString("UNFOCUSED", value_unfocused, "Empty", th->getLengthMin()));
+        keys.push_back(new CfgParserKeyString("FOCUSED", value_focused, "Empty", th->getLengthMin()));
+        keys.push_back(new CfgParserKeyString("UNFOCUSED", value_unfocused, "Empty", th->getLengthMin()));
 
         // Parse data
-        separator_section->parse_key_values(key_list.begin(), key_list.end());
+        separator_section->parse_key_values(keys.begin(), keys.end());
 
         // Free up resources
-        for_each (key_list.begin(), key_list.end(), Util::Free<CfgParserKey*>());
-        key_list.clear();
+        for_each (keys.begin(), keys.end(), Util::Free<CfgParserKey*>());
+        keys.clear();
 
         // Handle parsed data.
         _texture_separator[FOCUSED_STATE_FOCUSED] = th->getTexture(value_focused);
@@ -335,11 +338,11 @@ Theme::PDecorData::unload(void)
         }
     }
 
-    list<Theme::PDecorButtonData*>::iterator it(_button_list.begin());
-    for (; it != _button_list.end(); ++it) {
+    vector<Theme::PDecorButtonData*>::const_iterator it(_buttons.begin());
+    for (; it != _buttons.end(); ++it) {
         delete *it;
     }
-    _button_list.clear();
+    _buttons.clear();
 }
 
 //! @brief Checks data properties, prints warning and tries to fix.
@@ -407,9 +410,9 @@ Theme::PDecorData::loadButtons(CfgParser::Entry *section)
             continue;
         }
 
-        Theme::PDecorButtonData *btn = new Theme::PDecorButtonData ();
+        Theme::PDecorButtonData *btn = new Theme::PDecorButtonData();
         if (btn->load((*it)->get_section())) {
-            _button_list.push_back(btn);
+            _buttons.push_back(btn);
         } else {
             delete btn;
         }
@@ -604,22 +607,22 @@ Theme::PMenuData::check(void)
 void
 Theme::PMenuData::loadState (CfgParser::Entry *section, ObjectState state)
 {
-    list<CfgParserKey*> key_list;
+    vector<CfgParserKey*> keys;
     string value_font, value_background, value_item;
     string value_text, value_arrow, value_separator;
 
-    key_list.push_back(new CfgParserKeyString ("FONT", value_font));
-    key_list.push_back(new CfgParserKeyString ("BACKGROUND", value_background, "Solid #ffffff"));
-    key_list.push_back(new CfgParserKeyString ("ITEM", value_item, "Solid #ffffff"));
-    key_list.push_back(new CfgParserKeyString ("TEXT", value_text, "Solid #000000"));
-    key_list.push_back(new CfgParserKeyString ("ARROW", value_arrow, "Solid #000000"));
+    keys.push_back(new CfgParserKeyString ("FONT", value_font));
+    keys.push_back(new CfgParserKeyString ("BACKGROUND", value_background, "Solid #ffffff"));
+    keys.push_back(new CfgParserKeyString ("ITEM", value_item, "Solid #ffffff"));
+    keys.push_back(new CfgParserKeyString ("TEXT", value_text, "Solid #000000"));
+    keys.push_back(new CfgParserKeyString ("ARROW", value_arrow, "Solid #000000"));
     if (state < OBJECT_STATE_SELECTED) {
-        key_list.push_back(new CfgParserKeyString("SEPARATOR", value_separator, "Solid #000000"));
+        keys.push_back(new CfgParserKeyString("SEPARATOR", value_separator, "Solid #000000"));
     }
 
-    section->parse_key_values(key_list.begin(), key_list.end());
+    section->parse_key_values(keys.begin(), keys.end());
 
-    for_each(key_list.begin(), key_list.end(), Util::Free<CfgParserKey*>());
+    for_each(keys.begin(), keys.end(), Util::Free<CfgParserKey*>());
 
     TextureHandler *th = TextureHandler::instance ();
 
@@ -657,17 +660,17 @@ Theme::TextDialogData::~TextDialogData(void)
 bool
 Theme::TextDialogData::load(CfgParser::Entry *section)
 {
-    list<CfgParserKey*> key_list;
+    vector<CfgParserKey*> keys;
     string value_font, value_text, value_texture, value_pad;
 
-    key_list.push_back(new CfgParserKeyString("FONT", value_font));
-    key_list.push_back(new CfgParserKeyString("TEXT", value_text, "#000000"));
-    key_list.push_back(new CfgParserKeyString("TEXTURE", value_texture, "Solid #ffffff"));
-    key_list.push_back(new CfgParserKeyString("PAD", value_pad, "0 0 0 0", 7));
+    keys.push_back(new CfgParserKeyString("FONT", value_font));
+    keys.push_back(new CfgParserKeyString("TEXT", value_text, "#000000"));
+    keys.push_back(new CfgParserKeyString("TEXTURE", value_texture, "Solid #ffffff"));
+    keys.push_back(new CfgParserKeyString("PAD", value_pad, "0 0 0 0", 7));
 
-    section->parse_key_values(key_list.begin(), key_list.end());
+    section->parse_key_values(keys.begin(), keys.end());
 
-    for_each(key_list.begin(), key_list.end(), Util::Free<CfgParserKey*>());
+    for_each(keys.begin(), keys.end(), Util::Free<CfgParserKey*>());
 
     // Handle parsed data.
     _font = FontHandler::instance ()->getFont (value_font);
@@ -722,7 +725,7 @@ Theme::TextDialogData::check(void)
  */
 Theme::WorkspaceIndicatorData::WorkspaceIndicatorData(void)
     : ThemeData(),
-      font(0), font_color(0), texture_background(0), 
+      font(0), font_color(0), texture_background(0),
       texture_workspace(0), texture_workspace_act(0),
       edge_padding(0), workspace_padding(0)
 {
@@ -742,21 +745,21 @@ Theme::WorkspaceIndicatorData::~WorkspaceIndicatorData(void)
 bool
 Theme::WorkspaceIndicatorData::load(CfgParser::Entry *section)
 {
-    list<CfgParserKey*> key_list;
+    vector<CfgParserKey*> keys;
 
     string value_font, value_color, value_tex_bg;
     string value_tex_ws, value_tex_ws_act;
 
-    key_list.push_back(new CfgParserKeyString("FONT", value_font));
-    key_list.push_back(new CfgParserKeyString("TEXT", value_color));
-    key_list.push_back(new CfgParserKeyString("BACKGROUND", value_tex_bg));
-    key_list.push_back(new CfgParserKeyString("WORKSPACE", value_tex_ws));
-    key_list.push_back(new CfgParserKeyString("WORKSPACEACTIVE", value_tex_ws_act));
-    key_list.push_back(new CfgParserKeyNumeric<int>("EDGEPADDING", edge_padding, 5, 0));
-    key_list.push_back(new CfgParserKeyNumeric<int>("WORKSPACEPADDING", workspace_padding, 2, 0));
+    keys.push_back(new CfgParserKeyString("FONT", value_font));
+    keys.push_back(new CfgParserKeyString("TEXT", value_color));
+    keys.push_back(new CfgParserKeyString("BACKGROUND", value_tex_bg));
+    keys.push_back(new CfgParserKeyString("WORKSPACE", value_tex_ws));
+    keys.push_back(new CfgParserKeyString("WORKSPACEACTIVE", value_tex_ws_act));
+    keys.push_back(new CfgParserKeyNumeric<int>("EDGEPADDING", edge_padding, 5, 0));
+    keys.push_back(new CfgParserKeyNumeric<int>("WORKSPACEPADDING", workspace_padding, 2, 0));
 
-    section->parse_key_values(key_list.begin(), key_list.end());
-    for_each(key_list.begin(), key_list.end(), Util::Free<CfgParserKey*>());  
+    section->parse_key_values(keys.begin(), keys.end());
+    for_each(keys.begin(), keys.end(), Util::Free<CfgParserKey*>());
 
     font = FontHandler::instance()->getFont(value_font);
     font_color = FontHandler::instance()->getColor(value_color);
@@ -862,7 +865,7 @@ Theme::HarbourData::unload(void)
 {
     if (_texture) {
         TextureHandler::instance()->returnTexture(_texture);
-        _texture = 0;        
+        _texture = 0;
     }
 }
 
@@ -882,11 +885,9 @@ Theme::HarbourData::check(void)
 
 //! @brief Theme constructor
 Theme::Theme(void)
-    : _image_handler(0),
-      _is_loaded(false), _invert_gc(None)
+    : _is_loaded(false), _invert_gc(None)
 {
-    // image handler
-    _image_handler = new ImageHandler();
+    new ImageHandler();
 
     // Map between theme sections and ThemeData structures.
     _section_data_map["MENU"] = &_menu_data;
@@ -917,7 +918,7 @@ Theme::~Theme(void)
 
     XFreeGC(X11::getDpy(), _invert_gc);
 
-    delete _image_handler;
+    delete ImageHandler::instance();
 }
 
 /**
@@ -932,7 +933,7 @@ Theme::load(const std::string &dir)
     }
     string theme_file(norm_dir + string("theme"));
 
-    if (! Util::requireReload(_cfg_state, theme_file)) {
+    if (! _cfg_files.requireReload(theme_file)) {
         return false;
     }
 
@@ -961,16 +962,16 @@ Theme::load(const std::string &dir)
     // Setup quirks and requirements before parsing.
     if (theme_ok) {
         if (theme.is_dynamic_content()) {
-            _cfg_state.clear();
+            _cfg_files.clear();
         } else {
-            _cfg_state = theme.get_file_list();
+            _cfg_files = theme.getCfgFiles();
         }
         loadThemeRequire(theme, theme_file);
     }
 
     // Set image basedir.
-    _image_handler->path_clear();
-    _image_handler->path_push_back(_theme_dir);
+    ImageHandler::instance()->path_clear();
+    ImageHandler::instance()->path_push_back(_theme_dir);
 
     // Load decor data.
     CfgParser::Entry *section = theme.get_entry_root()->find_section("PDECOR");
@@ -1018,15 +1019,15 @@ Theme::loadThemeRequire(CfgParser &theme_cfg, std::string &file)
 {
     CfgParser::Entry *section;
 
-    // Look for requires section, 
+    // Look for requires section,
     section = theme_cfg.get_entry_root()->find_section("REQUIRE");
     if (section) {
-        list<CfgParserKey*> key_list;
+        vector<CfgParserKey*> keys;
         bool value_templates;
 
-        key_list.push_back(new CfgParserKeyBool("TEMPLATES", value_templates, false));
-        section->parse_key_values(key_list.begin(), key_list.end());
-        for_each(key_list.begin(), key_list.end(), Util::Free<CfgParserKey*>());
+        keys.push_back(new CfgParserKeyBool("TEMPLATES", value_templates, false));
+        section->parse_key_values(keys.begin(), keys.end());
+        for_each(keys.begin(), keys.end(), Util::Free<CfgParserKey*>());
 
         // Re-load configuration with templates enabled.
         if (value_templates) {
