@@ -27,7 +27,6 @@
 #include "Theme.hh"
 #include "PFont.hh"
 #include "PTexture.hh"
-#include "ColorHandler.hh"
 #include "FontHandler.hh"
 #include "PixmapHandler.hh"
 #include "TextureHandler.hh"
@@ -40,7 +39,6 @@
 #include "KeyGrabber.hh"
 #include "MenuHandler.hh"
 #include "Harbour.hh"
-#include "HarbourMenu.hh"
 #include "DockApp.hh"
 #include "CmdDialog.hh"
 #include "SearchDialog.hh"
@@ -48,8 +46,6 @@
 #include "WorkspaceIndicator.hh"
 
 #include <iostream>
-#include <list>
-#include <algorithm>
 #include <functional>
 #include <memory>
 #include <cassert>
@@ -72,7 +68,6 @@ using std::cout;
 using std::cerr;
 using std::endl;
 using std::find;
-using std::list;
 using std::map;
 using std::mem_fun;
 using std::string;
@@ -137,9 +132,9 @@ WindowManager::start(const std::string &command_line,
         static_cast<RootWO*>(PWinObj::getRootPWinObj())->setEwmhDesktopNames();
     
         // add all frames to the MRU list
-        _instance->_mru_list.resize(Frame::frame_size());
-        copy(Frame::frame_begin(), Frame::frame_end (),
-             _instance->_mru_list.begin());
+        _instance->_mru.resize(Frame::frame_size());
+        copy(Frame::frame_begin(), Frame::frame_end(),
+             _instance->_mru.begin());
 
         _instance->execStartFile();
     } else {
@@ -166,7 +161,7 @@ WindowManager::WindowManager(const std::string &command_line,
         :
         _screen_resources(0),
         _keygrabber(0),
-        _config(0), _color_handler(0),
+        _config(0),
         _font_handler(0), _texture_handler(0),
         _theme(0), _action_handler(0),
         _autoproperties(0),
@@ -177,6 +172,11 @@ WindowManager::WindowManager(const std::string &command_line,
         _startup(false), _shutdown(false), _reload(false),
         _allow_grouping(true), _hint_wo(0), _root_wo(0)
 {
+    _screen_edges[0] = 0;
+    _screen_edges[1] = 0;
+    _screen_edges[2] = 0;
+    _screen_edges[3] = 0;
+
     struct sigaction act;
 
     // Set up the signal handlers.
@@ -205,7 +205,7 @@ WindowManager::~WindowManager(void)
     delete _search_dialog;
     delete _status_window;
     delete _workspace_indicator;
-    MenuHandler::destroy();
+    MenuHandler::deleteMenus();
     delete _harbour;
     delete _root_wo;
     delete _hint_wo;
@@ -215,7 +215,6 @@ WindowManager::~WindowManager(void)
     Workspaces::free();
     delete _config;
     delete _theme;
-    delete _color_handler;
     delete _font_handler;
     delete _texture_handler;
     delete _screen_resources;
@@ -245,7 +244,7 @@ void
 WindowManager::cleanup(void)
 {
     // update all nonactive clients properties
-    list<Frame*>::iterator it_f(Frame::frame_begin());
+    vector<Frame*>::const_iterator it_f(Frame::frame_begin());
     for (; it_f != Frame::frame_end(); ++it_f) {
         (*it_f)->updateInactiveChildInfo();
     }
@@ -257,8 +256,8 @@ WindowManager::cleanup(void)
     // To preserve stacking order when destroying the frames, we go through
     // the PWinObj list from the Workspaces and put all Frames into our own
     // list, then we delete the frames in order.
-    list<Frame*> frame_list;
-    list<PWinObj*>::iterator it_w(Workspaces::begin());
+    vector<Frame*> frame_list;
+    Workspaces::iterator it_w(Workspaces::begin());
     for (; it_w != Workspaces::end(); ++it_w) {
         if ((*it_w)->getType() == PWinObj::WO_FRAME) {
             frame_list.push_back(static_cast<Frame*>(*it_w));
@@ -271,21 +270,22 @@ WindowManager::cleanup(void)
     }
 
     // Delete all Clients.
-    list<Client*> client_list(Client::client_begin(), Client::client_end());
-    list<Client*>::iterator it_c(client_list.begin());
-    for (; it_c != client_list.end(); ++it_c) {
-        delete *it_c;
-    }
+    while (Client::client_begin() != Client::client_end())
+        delete *Client::client_begin();
 
     if (_keygrabber) {
         _keygrabber->ungrabKeys(X11::getRoot());
     }
 
     // destroy screen edge
-    screenEdgeDestroy();
+    for (int i=0; i < 4; ++i) {
+        Workspaces::remove(_screen_edges[i]);
+        delete _screen_edges[i];
+        _screen_edges[i]=0;
+    }
 
     XInstallColormap(X11::getDpy(), X11::getColormap());
-    XSetInputFocus(X11::getDpy(), PointerRoot, RevertToPointerRoot, CurrentTime);
+    X11::setInputFocus(PointerRoot);
 }
 
 /**
@@ -305,8 +305,6 @@ WindowManager::setupDisplay(bool replace)
     // Setup screen, init atoms and claim the display.
     X11::init(dpy, _config->isHonourRandr());
 
-    Atoms::init();
-
     try {
         // Create hint window _before_ root window.
         _hint_wo = new HintWO(X11::getRoot(), replace);
@@ -318,7 +316,6 @@ WindowManager::setupDisplay(bool replace)
     _root_wo = new RootWO(X11::getRoot());
     PWinObj::setRootPWinObj(_root_wo);
 
-    _color_handler = new ColorHandler(dpy);
     _font_handler = new FontHandler();
     _texture_handler = new TextureHandler();
     _action_handler = new ActionHandler();
@@ -336,9 +333,9 @@ WindowManager::setupDisplay(bool replace)
     Workspaces::setSize(_config->getWorkspaces());
     Workspaces::setPerRow(_config->getWorkspacesPerRow());
 
-    _harbour = new Harbour(_theme);
+    _harbour = new Harbour;
 
-    MenuHandler::init(_theme);
+    MenuHandler::createMenus(_theme);
 
     _cmd_dialog = new CmdDialog(_theme);
     _search_dialog = new SearchDialog(_theme);
@@ -349,7 +346,7 @@ WindowManager::setupDisplay(bool replace)
                   _screen_resources->getCursor(ScreenResources::CURSOR_ARROW));
 
 #ifdef HAVE_XRANDR
-    XRRSelectInput(dpy, X11::getRoot(), RRScreenChangeNotifyMask|RRCrtcChangeNotifyMask);
+    XRRSelectInput(dpy, X11::getRoot(), RRScreenChangeNotifyMask);
 #endif // HAVE_XRANDR
 
     _keygrabber = new KeyGrabber;
@@ -373,15 +370,14 @@ WindowManager::scanWindows(void)
     
     uint num_wins;
     Window d_win1, d_win2, *wins;
-    XWindowAttributes attr;
 
     // Lets create a list of windows on the display
     XQueryTree(X11::getDpy(), X11::getRoot(),
                &d_win1, &d_win2, &wins, &num_wins);
-    list<Window> win_list(wins, wins + num_wins);
+    vector<Window> win_list(wins, wins + num_wins);
     XFree(wins);
 
-    list<Window>::iterator it(win_list.begin());
+    vector<Window>::iterator it(win_list.begin());
 
     // We filter out all windows with the the IconWindowHint
     // set not pointing to themselves, making DockApps
@@ -395,7 +391,7 @@ WindowManager::scanWindows(void)
         if (wm_hints) {
             if ((wm_hints->flags&IconWindowHint) &&
                     (wm_hints->icon_window != *it)) {
-                list<Window>::iterator i_it(find(win_list.begin(), win_list.end(), wm_hints->icon_window));
+                vector<Window>::iterator i_it(find(win_list.begin(), win_list.end(), wm_hints->icon_window));
                 if (i_it != win_list.end())
                     *i_it = None;
             }
@@ -409,13 +405,14 @@ WindowManager::scanWindows(void)
             continue;
         }
         
+        XWindowAttributes attr;
         XGetWindowAttributes(X11::getDpy(), *it, &attr);
         if (! attr.override_redirect && attr.map_state != IsUnmapped) {
             XWMHints *wm_hints = XGetWMHints(X11::getDpy(), *it);
             if (wm_hints) {
                 if ((wm_hints->flags&StateHint) &&
                         (wm_hints->initial_state == WithdrawnState)) {
-                    _harbour->addDockApp(new DockApp(_theme, *it));
+                    _harbour->addDockApp(new DockApp(*it));
                 } else {
                     client = new Client(*it);
                     if (! client->isAlive()) {
@@ -442,9 +439,9 @@ WindowManager::scanWindows(void)
 
     // Try to find transients for all clients, on restarts ordering might
     // not be correct.
-    list<Client*>::iterator it_client = Client::client_begin();
+    vector<Client*>::const_iterator it_client = Client::client_begin();
     for (; it_client != Client::client_end(); ++it_client) {
-        if ((*it_client)->isTransient() && ! (*it_client)->getTransientClient()) {
+        if ((*it_client)->isTransient() && ! (*it_client)->getTransientForClient()) {
             (*it_client)->findAndRaiseIfTransient();
         }
     }
@@ -459,76 +456,49 @@ WindowManager::scanWindows(void)
 void
 WindowManager::screenEdgeCreate(void)
 {
-    if (_screen_edge_list.size() != 0) {
-        return;
-    }
-
     bool indent = Config::instance()->getScreenEdgeIndent();
 
-    _screen_edge_list.push_back(new EdgeWO(X11::getRoot(), SCREEN_EDGE_LEFT,
-                                           indent && (_config->getScreenEdgeSize(SCREEN_EDGE_LEFT) > 0)));
-    _screen_edge_list.push_back(new EdgeWO(X11::getRoot(), SCREEN_EDGE_RIGHT,
-                                           indent && (_config->getScreenEdgeSize(SCREEN_EDGE_RIGHT) > 0)));
-    _screen_edge_list.push_back(new EdgeWO(X11::getRoot(), SCREEN_EDGE_TOP,
-                                           indent && (_config->getScreenEdgeSize(SCREEN_EDGE_TOP) > 0)));
-    _screen_edge_list.push_back(new EdgeWO(X11::getRoot(), SCREEN_EDGE_BOTTOM,
-                                           indent && (_config->getScreenEdgeSize(SCREEN_EDGE_BOTTOM) > 0)));
+    _screen_edges[0] = new EdgeWO(X11::getRoot(), SCREEN_EDGE_LEFT,
+                                  indent && (_config->getScreenEdgeSize(SCREEN_EDGE_LEFT) > 0));
+    _screen_edges[1] = new EdgeWO(X11::getRoot(), SCREEN_EDGE_RIGHT,
+                                  indent && (_config->getScreenEdgeSize(SCREEN_EDGE_RIGHT) > 0));
+    _screen_edges[2] = new EdgeWO(X11::getRoot(), SCREEN_EDGE_TOP,
+                                  indent && (_config->getScreenEdgeSize(SCREEN_EDGE_TOP) > 0));
+    _screen_edges[3] = new EdgeWO(X11::getRoot(), SCREEN_EDGE_BOTTOM,
+                                  indent && (_config->getScreenEdgeSize(SCREEN_EDGE_BOTTOM) > 0));
 
     // make sure the edge stays ontop
-    list<EdgeWO*>::iterator it(_screen_edge_list.begin());
-    for (; it != _screen_edge_list.end(); ++it) {
-        Workspaces::insert(*it);
+    for (int i=0; i < 4; ++i) {
+        Workspaces::insert(_screen_edges[i]);
     }
 
     screenEdgeResize();
 }
 
-//! @brief
-void
-WindowManager::screenEdgeDestroy(void)
-{
-    if (_screen_edge_list.size() == 0) {
-        return;
-    }
-
-    list<EdgeWO*>::iterator it(_screen_edge_list.begin());
-    for (; it != _screen_edge_list.end(); ++it) {
-        Workspaces::remove(*it);
-        delete *it;
-    }
-}
-
 //! @brief
 void
 WindowManager::screenEdgeResize(void)
 {
-    assert(_screen_edge_list.size() == 4);
-
     uint l_size = std::max(_config->getScreenEdgeSize(SCREEN_EDGE_LEFT), 1);
     uint r_size = std::max(_config->getScreenEdgeSize(SCREEN_EDGE_RIGHT), 1);
     uint t_size = std::max(_config->getScreenEdgeSize(SCREEN_EDGE_TOP), 1);
     uint b_size = std::max(_config->getScreenEdgeSize(SCREEN_EDGE_BOTTOM), 1);
 
-    list<EdgeWO*>::iterator it(_screen_edge_list.begin());
-
     // Left edge
-    (*it)->moveResize(0, 0, l_size, X11::getHeight());
-    ++it;
+    _screen_edges[0]->moveResize(0, 0, l_size, X11::getHeight());
 
     // Right edge
-    (*it)->moveResize(X11::getWidth() - r_size, 0, r_size, X11::getHeight());
-    ++it;
+    _screen_edges[1]->moveResize(X11::getWidth() - r_size, 0, r_size, X11::getHeight());
 
     // Top edge
-    (*it)->moveResize(l_size, 0, X11::getWidth() - l_size - r_size, t_size);
-    ++it;
+    _screen_edges[2]->moveResize(l_size, 0, X11::getWidth() - l_size - r_size, t_size);
 
     // Bottom edge
-    (*it)->moveResize(l_size, X11::getHeight() - b_size, X11::getWidth() - l_size - r_size, b_size);
+    _screen_edges[3]->moveResize(l_size, X11::getHeight() - b_size, X11::getWidth() - l_size - r_size, b_size);
 
-    for (it = _screen_edge_list.begin(); it != _screen_edge_list.end(); ++it) {
-      (*it)->configureStrut(_config->getScreenEdgeIndent()
-                            && (_config->getScreenEdgeSize((*it)->getEdge()) > 0));
+    for (int i=0; i<4; ++i) {
+        _screen_edges[i]->configureStrut(_config->getScreenEdgeIndent()
+                        && _config->getScreenEdgeSize(_screen_edges[i]->getEdge()) > 0);
     }
 
     X11::updateStrut();
@@ -537,15 +507,14 @@ WindowManager::screenEdgeResize(void)
 void
 WindowManager::screenEdgeMapUnmap(void)
 {
-    assert (_screen_edge_list.size() == 4);
-
-    list<EdgeWO*>::iterator it(_screen_edge_list.begin());
-    for (; it != _screen_edge_list.end(); ++it) {
-        if ((_config->getScreenEdgeSize((*it)->getEdge()) > 0) &&
-            (_config->getEdgeListFromPosition((*it)->getEdge())->size() > 0)) {
-            (*it)->mapWindow();
+    EdgeWO* edge;
+    for (int i=0; i<4; ++i) {
+        edge = _screen_edges[i];
+        if (_config->getScreenEdgeSize(edge->getEdge()) > 0
+            && _config->getEdgeListFromPosition(edge->getEdge())->size() > 0) {
+            edge->mapWindow();
         } else {
-            (*it)->unmapWindow();
+            edge->unmapWindow();
         }
     }
 }
@@ -560,9 +529,7 @@ WindowManager::doReload(void)
     doReloadKeygrabber();
     doReloadAutoproperties();
 
-    MenuHandler::instance()->reloadMenus();
-    // Special case for HARBOUR menu which is not included in the menu map
-    _harbour->getHarbourMenu()->reload(static_cast<CfgParser::Entry*>(0));
+    MenuHandler::reloadMenus();
     doReloadHarbour();
 
     _root_wo->setEwmhDesktopNames();
@@ -654,7 +621,7 @@ WindowManager::doReloadKeygrabber(bool force)
     _keygrabber->grabKeys(X11::getRoot());
 
     // Regrab keys and buttons
-    list<Client*>::iterator c_it(Client::client_begin());
+    vector<Client*>::const_iterator c_it(Client::client_begin());
     for (; c_it != Client::client_end(); ++c_it) {
         (*c_it)->grabButtons();
         _keygrabber->ungrabKeys((*c_it)->getWindow());
@@ -674,12 +641,12 @@ WindowManager::doReloadAutoproperties(void)
 
     // NOTE: we need to load autoproperties after decor have been updated
     // as otherwise old theme data pointer will be used and sig 11 pekwm.
-    list<Client*>::iterator it_c(Client::client_begin());
+    vector<Client*>::const_iterator it_c(Client::client_begin());
     for (; it_c != Client::client_end(); ++it_c) {
         (*it_c)->readAutoprops(APPLY_ON_RELOAD);
     }
 
-    list<Frame*>::iterator it_f(Frame::frame_begin());
+    vector<Frame*>::const_iterator it_f(Frame::frame_begin());
     for (; it_f != Frame::frame_end(); ++it_f) {
         (*it_f)->readAutoprops(APPLY_ON_RELOAD);
     }
@@ -801,7 +768,6 @@ WindowManager::doEventLoop(void)
                 handleFocusInEvent(&ev.xfocus);
                 break;
             case FocusOut:
-                handleFocusOutEvent(&ev.xfocus);
                 break;
 
             case SelectionClear:
@@ -812,13 +778,31 @@ WindowManager::doEventLoop(void)
 
             default:
 #ifdef HAVE_SHAPE
-                if (X11::hasExtensionShape() && (ev.type == X11::getEventShape())) {
-                    handleShapeEvent(&ev.xany);
+                if (X11::hasExtensionShape() && ev.type == X11::getEventShape()) {
+                    XShapeEvent *sev = reinterpret_cast<XShapeEvent*>(&ev);
+                    X11::setLastEventTime(sev->time);
+                    Client *client = Client::findClient(sev->window);
+                    if (client) {
+                        client->handleShapeEvent(sev);
+                    }
                 }
 #endif // HAVE_SHAPE
 #ifdef HAVE_XRANDR
-                if (X11::hasExtensionXRandr() && (ev.type == X11::getEventXRandr())) {
-                    handleXRandrEvent(reinterpret_cast<XRRNotifyEvent*>(&ev));
+                if (X11::hasExtensionXRandr()) {
+                    if (ev.type == X11::getEventXRandr() + RRScreenChangeNotify) {
+                        XRRScreenChangeNotifyEvent *scr_ev =
+                                reinterpret_cast<XRRScreenChangeNotifyEvent*>(&ev);
+
+                        X11::updateGeometry(scr_ev->width, scr_ev->height);
+                        _harbour->updateGeometry();
+                        screenEdgeResize();
+
+                        // Make sure windows are visible after resize
+                        vector<PDecor*>::const_iterator it(PDecor::pdecor_begin());
+                        for (; it != PDecor::pdecor_end(); ++it) {
+                            Workspaces::placeWoInsideScreen(*it);
+                        }
+                    }
                 }
 #endif // HAVE_XRANDR
                 break;
@@ -1083,7 +1067,7 @@ WindowManager::handleMapRequestEvent(XMapRequestEvent *ev)
             if (wm_hints) {
                 if ((wm_hints->flags&StateHint) &&
                         (wm_hints->initial_state == WithdrawnState)) {
-                    _harbour->addDockApp(new DockApp(_theme, ev->window));
+                    _harbour->addDockApp(new DockApp(ev->window));
                 } else {
                     Client *client = new Client(ev->window, true);
                     if (! client->isAlive()) {
@@ -1226,7 +1210,10 @@ WindowManager::handleLeaveNotify(XCrossingEvent *ev)
 void
 WindowManager::handleFocusInEvent(XFocusChangeEvent *ev)
 {
-    if ((ev->mode == NotifyGrab) || (ev->mode == NotifyUngrab)) {
+    if (ev->mode == NotifyGrab
+        || ev->mode == NotifyUngrab
+        || ev->detail == NotifyVirtual
+        || ev->detail == NotifyNonlinearVirtual) {
         return;
     }
 
@@ -1277,8 +1264,7 @@ WindowManager::handleFocusInEvent(XFocusChangeEvent *ev)
 
                 // update the MRU list (except for skip focus windows, see #297)
                 if (! static_cast<Client*>(wo)->isSkip(SKIP_FOCUS_TOGGLE)) {
-                    _mru_list.remove(static_cast<Frame*>(wo->getParent()));
-                    _mru_list.push_front(static_cast<Frame*>(wo->getParent()));
+                    addToMRUFront(static_cast<Frame*>(wo->getParent()));
                 }
             } else {
                 wo->setFocused(true);
@@ -1288,15 +1274,6 @@ WindowManager::handleFocusInEvent(XFocusChangeEvent *ev)
     }
 }
 
-//! @brief Handles FocusOut Events.
-void
-WindowManager::handleFocusOutEvent(XFocusChangeEvent *ev)
-{
-    // Get the last focus in event, no need to go through them all.
-    while (X11::checkTypedEvent(FocusOut, (XEvent *) ev))
-        ;
-}
-
 /**
  * Handle XClientMessageEvent events.
  *
@@ -1310,11 +1287,11 @@ WindowManager::handleClientMessageEvent(XClientMessageEvent *ev)
 
         if (ev->format == 32) {
 
-            if (ev->message_type == Atoms::getAtom(NET_CURRENT_DESKTOP)) {
+            if (ev->message_type == X11::getAtom(NET_CURRENT_DESKTOP)) {
                 Workspaces::setWorkspace(ev->data.l[0], true);
 
             } else if (ev->message_type ==
-                       Atoms::getAtom(NET_NUMBER_OF_DESKTOPS)) {
+                       X11::getAtom(NET_NUMBER_OF_DESKTOPS)) {
                 if (ev->data.l[0] > 0) {
                     Workspaces::setSize(ev->data.l[0]);
                 }
@@ -1347,7 +1324,7 @@ void
 WindowManager::handlePropertyEvent(XPropertyEvent *ev)
 {
     if (ev->window == X11::getRoot()) {
-        if (ev->atom == Atoms::getAtom(NET_DESKTOP_NAMES)) {
+        if (ev->atom == X11::getAtom(NET_DESKTOP_NAMES)) {
             _root_wo->readEwmhDesktopNames();
             Workspaces::setNames();
         }
@@ -1393,70 +1370,6 @@ WindowManager::handleExposeEvent(XExposeEvent *ev)
     }
 }
 
-
-//! @brief Handle shape events applying shape to clients
-void
-WindowManager::handleShapeEvent(XAnyEvent *ev)
-{
-#ifdef HAVE_SHAPE
-    Client *client = Client::findClient(ev->window);
-    if (client && client->getParent()) {
-        static_cast<Frame*>(client->getParent())->handleShapeEvent(ev);
-    }
-#endif // HAVE_SHAPE
-}
-
-#ifdef HAVE_XRANDR
-//! @brief Handles XRandr events
-//! @param ev XRRNotifyEvent to handle.
-void
-WindowManager::handleXRandrEvent(XRRNotifyEvent *ev)
-{
-    if (ev->subtype == RRNotify_CrtcChange) {
-        handleXRandrCrtcChangeEvent(reinterpret_cast<XRRCrtcChangeNotifyEvent*>(ev));
-    } else {
-        handleXRandrScreenChangeEvent(reinterpret_cast<XRRScreenChangeNotifyEvent*>(ev));
-    }
-}
-
-/**
- * Handle screen change event.
- *
- * Reads the screen geometry and head information all over, updates
- * the screen edge and harbour.
- *
- * @param ev XRRScreenChangeNotifyEvent event to handle.
- */
-void
-WindowManager::handleXRandrScreenChangeEvent(XRRScreenChangeNotifyEvent *ev)
-{
-#ifdef DEBUG
-    cerr << __FILE__ << "@" << __LINE__ << ": WindowManager::handleXRandrScreenChangeEvent()" << endl;
-#endif // DEBUG
-
-    X11::updateGeometry(ev->width, ev->height);
-    _harbour->updateGeometry();
-    screenEdgeResize();
-
-    // Make sure windows are visible after resize
-    list<PDecor*>::iterator it(PDecor::pdecor_begin());
-    for (; it != PDecor::pdecor_end(); ++it) {
-        Workspaces::placeWoInsideScreen(*it);
-    }
-}
-
-//! @brief Handle crtc change event, does nothing.
-//! @param ev XRRCrtcChangeNotifyEvent event to handle.
-void
-WindowManager::handleXRandrCrtcChangeEvent(XRRCrtcChangeNotifyEvent *ev)
-{
-#ifdef DEBUG
-    cerr << __FILE__ << "@" << __LINE__ << ": WindowManager::handleXRandrCrtcChangeEvent()" << endl;
-#endif // DEBUG
-}
-
-#endif // HAVE_XRANDR
-
 // Event handling routines stop ============================================
 
 //! @brief Searches for a PWinObj to focus, and then gives it input focus
@@ -1472,8 +1385,8 @@ WindowManager::findWOAndFocus(PWinObj *search)
 
     // search window object didn't exist, go through the MRU list
     if (! focus) {
-        list<Frame *>::iterator f_it = _mru_list.begin();
-        for (; ! focus && f_it != _mru_list.end(); ++f_it) {
+        vector<Frame *>::const_iterator f_it = _mru.begin();
+        for (; ! focus && f_it != _mru.end(); ++f_it) {
             if ((*f_it)->isMapped() && (*f_it)->isFocusable()) {
                 focus = *f_it;
             }
@@ -1496,27 +1409,27 @@ WindowManager::familyRaiseLower(Client *client, bool raise)
 {
     Client *parent;
     Window win_search;
-    if (! client->getTransientClient()) {
+    if (! client->getTransientForClient()) {
         parent = client;
         win_search = client->getWindow();
     } else {
-        parent = client->getTransientClient();
-        win_search = client->getTransientClientWindow();
+        parent = client->getTransientForClient();
+        win_search = client->getTransientForClientWindow();
     }
 
-    list<Client*> client_list;
+    vector<Client*> client_list;
     Client::findFamilyFromWindow(client_list, win_search);
 
     if (parent) { // make sure parent gets underneath the children
         if (raise) {
-            client_list.push_front(parent);
+            client_list.insert(client_list.begin(), parent);
         } else {
             client_list.push_back(parent);
         }
     }
 
     Frame *frame;
-    list<Client*>::iterator it(client_list.begin());
+    vector<Client*>::const_iterator it(client_list.begin());
     for (; it != client_list.end(); ++it) {
         frame = dynamic_cast<Frame*>((*it)->getParent());
         if (frame) {
@@ -1529,13 +1442,6 @@ WindowManager::familyRaiseLower(Client *client, bool raise)
     }
 }
 
-//! @brief Remove from MRU list.
-void
-WindowManager::removeFromFrameList(Frame *frame)
-{
-    _mru_list.remove(frame);
-}
-
 /**
  * Match Frame against autoproperty.
  */
@@ -1606,7 +1512,7 @@ WindowManager::findGroupMatch(AutoProperty *property)
 
     // Moving on to the rest of the frames.
     if (! frame) {
-        list<Frame*>::iterator it(Frame::frame_begin());
+        vector<Frame*>::const_iterator it(Frame::frame_begin());
         for (; it != Frame::frame_end(); ++it) {
             if (findGroupMatchProperty(*it, property)) {
                 frame = *it;
@@ -1622,7 +1528,7 @@ WindowManager::findGroupMatch(AutoProperty *property)
 void
 WindowManager::attachMarked(Frame *frame)
 {
-    list<Client*>::iterator it(Client::client_begin());
+    vector<Client*>::const_iterator it(Client::client_begin());
     for (; it != Client::client_end(); ++it) {
         if ((*it)->isMarked()) {
             if ((*it)->getParent() != frame) {
@@ -1678,10 +1584,10 @@ WindowManager::getNextFrame(Frame* frame, bool mapped, uint mask)
     }
 
     Frame *next_frame = 0;
-    list<Frame*>::iterator f_it(find(Frame::frame_begin(), Frame::frame_end(), frame));
+    vector<Frame*>::const_iterator f_it(find(Frame::frame_begin(), Frame::frame_end(), frame));
 
     if (f_it != Frame::frame_end()) {
-        list<Frame*>::iterator n_it(f_it);
+        vector<Frame*>::const_iterator n_it(f_it);
 
         if (++n_it == Frame::frame_end()) {
             n_it = Frame::frame_begin();
@@ -1713,10 +1619,10 @@ WindowManager::getPrevFrame(Frame* frame, bool mapped, uint mask)
     }
 
     Frame *next_frame = 0;
-    list<Frame*>::iterator f_it(find(Frame::frame_begin(), Frame::frame_end(), frame));
+    vector<Frame*>::const_iterator f_it(find(Frame::frame_begin(), Frame::frame_end(), frame));
 
     if (f_it != Frame::frame_end()) {
-        list<Frame*>::iterator n_it(f_it);
+        vector<Frame*>::const_iterator n_it(f_it);
 
         if (n_it == Frame::frame_begin()) {
             n_it = --Frame::frame_end();
