@@ -10,8 +10,8 @@
 #include "config.h"
 #endif // HAVE_CONFIG_H
 
+#include "Debug.hh"
 #include "Workspaces.hh"
-#include "Atoms.hh"
 #include "ParseUtil.hh"
 #include "Config.hh"
 #include "PWinObj.hh"
@@ -33,8 +33,6 @@ extern "C" {
 #include <X11/Xatom.h> // for XA_WINDOW
 }
 
-using std::list;
-using std::vector;
 using std::string;
 using std::find;
 using std::wostringstream;
@@ -58,8 +56,8 @@ Workspaces::Workspace::~Workspace(void)
 uint Workspaces::_active;
 uint Workspaces::_previous;
 uint Workspaces::_per_row;
-std::list<PWinObj*> Workspaces::_wo_list;
-std::vector<Workspaces::Workspace *> Workspaces::_workspace_list;
+vector<PWinObj*> Workspaces::_wobjs;
+vector<Workspaces::Workspace *> Workspaces::_workspace_list;
 
 //! @brief Workspaces destructor
 void
@@ -93,8 +91,8 @@ Workspaces::setSize(uint number)
 
     // We have more workspaces than we want, lets remove the last ones
     if (before > number) {
-        list<PWinObj*>::iterator it(_wo_list.begin());
-        for (; it != _wo_list.end(); ++it) {
+        const_iterator it(_wobjs.begin());
+        for (; it != _wobjs.end(); ++it) {
             if ((*it)->getWorkspace() > (number - 1))
                 (*it)->setWorkspace(number - 1);
         }
@@ -114,7 +112,7 @@ Workspaces::setSize(uint number)
     // Tell the rest of the world how many workspaces we have.
     XChangeProperty(X11::getDpy(),
                     X11::getRoot(),
-                    Atoms::getAtom(NET_NUMBER_OF_DESKTOPS),
+                    X11::getAtom(NET_NUMBER_OF_DESKTOPS),
                     XA_CARDINAL, 32, PropModeReplace,
                     (uchar *) &number, 1);
 
@@ -165,9 +163,7 @@ Workspaces::setWorkspace(uint num, bool focus)
 
     // switch workspace
     hideAll(_active);
-    AtomUtil::setLong(X11::getRoot(),
-                      Atoms::getAtom(NET_CURRENT_DESKTOP),
-                      num);
+    X11::setLong(X11::getRoot(), NET_CURRENT_DESKTOP, num);
 
     _previous = _active;
     _active = num;
@@ -312,43 +308,93 @@ Workspaces::warpToWorkspace(uint num, int dir)
 }
 
 /**
- * Adds a PWinObj to the stacking list.
+ * Adds a PWinObj to the stacking list. Assumes that wo is not in _wobjs already.
  * @param wo PWinObj to insert
  * @param raise Whether to insert at the bottom or top of the layer (defaults to true).
  */
 void
 Workspaces::insert(PWinObj *wo, bool raise)
 {
-    list<PWinObj*>::iterator it(_wo_list.begin()), position(_wo_list.end());
-    for (; it != _wo_list.end() && position == _wo_list.end(); ++it) {
-        if (raise) {
-            // If raising, make sure the inserted wo gets below the first
-            // window in the next layer.
-            if ((*it)->getLayer() > wo->getLayer()) {
-                position = it;
-            }
-        } else {
-            // If lowering, put the window below the first window with the same level.
-            if (wo->getLayer() <= (*it)->getLayer()) {
-                position = it;
+    PWinObj *top_obj = 0;
+    Frame *frame, *wo_frame = dynamic_cast<Frame*>(wo);
+    iterator it;
+
+    if (! raise && wo_frame && wo_frame->getTransFor()
+                && wo_frame->getTransFor()->getLayer() == wo_frame->getLayer()) {
+        // Lower only to the top of the transient_for window.
+        it = ++find(_wobjs.begin(), _wobjs.end(), wo_frame->getTransFor()->getParent());
+        top_obj = it!=_wobjs.end()?*it:0; // I think it==_wobjs.end() can't happen
+    } else {
+        it = _wobjs.begin();
+        for (; it != _wobjs.end(); ++it) {
+            if (raise) {
+                // If raising, make sure the inserted wo gets below the first
+                // window in the next layer.
+                if ((*it)->getLayer() > wo->getLayer()) {
+                    top_obj = *it;
+                    break;
+                }
+            } else if (wo->getLayer() <= (*it)->getLayer()) {
+                // If lowering, put the window below the first window with the same level.
+                top_obj = *it;
+                break;
             }
         }
     }
 
-    _wo_list.insert(position, wo);
+    _wobjs.insert(it, wo);
 
-    if (position == _wo_list.end()) {
-         XRaiseWindow(X11::getDpy(), wo->getWindow());
-    } else {
-         stackWinUnderWin((*position)->getWindow(), wo->getWindow());
+    vector<PWinObj*> winstack;
+    winstack.reserve(3);
+    winstack.push_back(wo);
+
+    if (wo_frame && wo_frame->hasTrans()) {
+        vector<Client*>::const_iterator t_it;
+        for (it = _wobjs.begin(); *it != wo;) {
+            if ((frame = dynamic_cast<Frame*>(*it))) {
+                t_it = find(wo_frame->getTransBegin(), wo_frame->getTransEnd(),
+                            frame->getActiveClient());
+                if (t_it != wo_frame->getTransEnd()) {
+                    winstack.push_back(frame);
+                    it = _wobjs.erase(it);
+                    continue;
+                }
+            }
+            ++it;
+        }
+
+        it = ++find(_wobjs.begin(), _wobjs.end(), wo);
+        _wobjs.insert(it, winstack.begin()+1, winstack.end());
     }
+
+    if (top_obj) {
+        winstack.push_back(top_obj);
+    } else {
+        XRaiseWindow(X11::getDpy(), winstack.back()->getWindow());
+    }
+
+    const unsigned size = winstack.size();
+    Window *wins = new Window[size];
+    vector<PWinObj*>::const_iterator wit(winstack.end());
+    for (unsigned i=0; i<size; ++i) {
+        wins[i] = (*--wit)->getWindow();
+    }
+    X11::stackWindows(wins, size);
+    delete [] wins;
 }
 
 //! @brief Removes a PWinObj from the stacking list.
 void
 Workspaces::remove(PWinObj* wo)
 {
-    _wo_list.remove(wo);
+    iterator it_wo(_wobjs.begin());
+    for (;it_wo != _wobjs.end();) {
+        if (wo == *it_wo) {
+            it_wo = _wobjs.erase(it_wo);
+        } else {
+            ++it_wo;
+        }
+    }
 
     // remove from last focused
     vector<Workspace*>::iterator it(_workspace_list.begin());
@@ -363,8 +409,8 @@ Workspaces::remove(PWinObj* wo)
 void
 Workspaces::hideAll(uint workspace)
 {
-    list<PWinObj*>::iterator it(_wo_list.begin());
-    for (; it != _wo_list.end(); ++it) {
+    const_iterator it(_wobjs.begin());
+    for (; it != _wobjs.end(); ++it) {
         if (! ((*it)->isSticky()) && ! ((*it)->isHidden()) &&
                 ((*it)->getWorkspace() == workspace)) {
             (*it)->unmapWindow();
@@ -376,8 +422,8 @@ Workspaces::hideAll(uint workspace)
 void
 Workspaces::unhideAll(uint workspace, bool focus)
 {
-    list<PWinObj*>::iterator it(_wo_list.begin());
-    for (; it != _wo_list.end(); ++it) {
+    const_iterator it(_wobjs.begin());
+    for (; it != _wobjs.end(); ++it) {
         if (! (*it)->isMapped() && ! (*it)->isIconified() && ! (*it)->isHidden()
                 && ((*it)->getWorkspace() == workspace)) {
             (*it)->mapWindow(); // don't restack ontop windows
@@ -428,12 +474,12 @@ Workspaces::unhideAll(uint workspace, bool focus)
 void
 Workspaces::raise(PWinObj* wo)
 {
-    list<PWinObj*>::iterator it(find(_wo_list.begin(), _wo_list.end(), wo));
+    iterator it(find(_wobjs.begin(), _wobjs.end(), wo));
 
-    if (it == _wo_list.end()) { // no Frame to raise.
+    if (it == _wobjs.end()) { // no Frame to raise.
         return;
     }
-    _wo_list.erase(it);
+    _wobjs.erase(it);
 
     insert(wo, true); // reposition and restack
 }
@@ -442,68 +488,15 @@ Workspaces::raise(PWinObj* wo)
 void
 Workspaces::lower(PWinObj* wo)
 {
-    list<PWinObj*>::iterator it(find(_wo_list.begin(), _wo_list.end(), wo));
+    iterator it(find(_wobjs.begin(), _wobjs.end(), wo));
 
-    if (it == _wo_list.end()) // no Frame to raise.
+    if (it == _wobjs.end()) // no Frame to raise.
         return;
-    _wo_list.erase(it);
+    _wobjs.erase(it);
 
     insert(wo, false); // reposition and restack
 }
 
-//! @brief Places the PWinObj above the window win
-//! @param wo PWinObj to place.
-//! @param win Window to place Frame above.
-//! @param restack Restack the X windows, defaults to true.
-void
-Workspaces::stackAbove(PWinObj *wo, Window win, bool restack)
-{
-    list<PWinObj*>::iterator old_pos(find(_wo_list.begin(), _wo_list.end(), wo));
-
-    if (old_pos != _wo_list.end()) {
-        list<PWinObj*>::iterator it(_wo_list.begin());
-        for (; it != _wo_list.end(); ++it) {
-            if (win == (*it)->getWindow()) {
-                _wo_list.erase(old_pos);
-                _wo_list.insert(++it, wo);
-
-                // Before restacking make sure we are the active frame
-                // also that there are two different frames
-                if (restack) {
-                    stackWinUnderWin(win, wo->getWindow());
-                }
-                break;
-            }
-        }
-    }
-}
-
-//! @brief Places the PWinObj below the window win
-//! @param wo PWinObj to place.
-//! @param win Window to place Frame under
-//! @param restack Restack the X windows, defaults to true
-void
-Workspaces::stackBelow(PWinObj* wo, Window win, bool restack)
-{
-    list<PWinObj*>::iterator old_pos(find(_wo_list.begin(), _wo_list.end(), wo));
-
-    if (old_pos != _wo_list.end()) {
-        list<PWinObj*>::iterator it(_wo_list.begin());
-        for (; it != _wo_list.end(); ++it) {
-            if (win == (*it)->getWindow()) {
-                _wo_list.erase(old_pos);
-                _wo_list.insert(it, wo);
-
-                if (restack) {
-                    stackWinUnderWin(wo->getWindow(), win);
-                }
-                break;
-            }
-        }
-    }
-}
-
-
 //! @brief
 PWinObj*
 Workspaces::getLastFocused(uint workspace)
@@ -525,30 +518,16 @@ Workspaces::setLastFocused(uint workspace, PWinObj* wo)
     _workspace_list[workspace]->setLastFocused(wo);
 }
 
-//! @brief Helper function to stack a window below another
-//! @param win_over Window to place win_under under
-//! @param win_under Window to place under win_over
-void
-Workspaces::stackWinUnderWin(Window over, Window under)
-{
-    if (over == under) {
-        return;
-    }
-
-    Window windows[2] = { over, under };
-    XRestackWindows(X11::getDpy(), windows, 2);
-}
-
 //! @brief Create name for workspace num
 wstring
 Workspaces::getWorkspaceName(uint num)
 {
-  wostringstream buf;
-  buf << num + 1;
-  buf << L": ";
-  buf << Config::instance()->getWorkspaceName(num);
-
-  return buf.str();
+    wostringstream buf;
+    buf << num + 1;
+    buf << L": ";
+    buf << Config::instance()->getWorkspaceName(num);
+    
+    return buf.str();
 }
 
 // MISC METHODS
@@ -557,8 +536,8 @@ Workspaces::getWorkspaceName(uint num)
 PWinObj*
 Workspaces::getTopWO(uint type_mask)
 {
-    list<PWinObj*>::reverse_iterator r_it = _wo_list.rbegin();
-    for (; r_it != _wo_list.rend(); ++r_it) {
+    const_reverse_iterator r_it = _wobjs.rbegin();
+    for (; r_it != _wobjs.rend(); ++r_it) {
         if ((*r_it)->isMapped()
                 && (*r_it)->isFocusable()
                 && ((*r_it)->getType()&type_mask)) {
@@ -578,39 +557,39 @@ Workspaces::buildClientList(unsigned int &num_windows)
     Frame *frame;
     Client *client, *client_active;
 
-    list<Window> windows_list;
-    list<PWinObj*>::iterator it_f, it_c;
-    for (it_f = _wo_list.begin(); it_f != _wo_list.end(); ++it_f) {
+    vector<Window> windows;
+    iterator it_f;
+    const_iterator it_c;
+    for (it_f = _wobjs.begin(); it_f != _wobjs.end(); ++it_f) {
         if ((*it_f)->getType() != PWinObj::WO_FRAME) {
-	  continue;
-	}
+            continue;
+        }
 
-	frame = static_cast<Frame*>(*it_f);
-	client_active = frame->getActiveClient();
+        frame = static_cast<Frame*>(*it_f);
+        client_active = frame->getActiveClient();
 
-	if (Config::instance()->isReportAllClients()) {
+        if (Config::instance()->isReportAllClients()) {
             for (it_c = frame->begin(); it_c != frame->end(); ++it_c) {
                 client = dynamic_cast<Client*>(*it_c);
-                if (client
-		    && ! client->isSkip(SKIP_TASKBAR)
-		    && client != client_active) {
-                    windows_list.push_back(client->getWindow());
-	        }
-	    }
-	}
+                if (client && ! client->isSkip(SKIP_TASKBAR)
+                    && client != client_active) {
+                    windows.push_back(client->getWindow());
+                }
+            }
+        }
 
-	if (client_active && ! client_active->isSkip(SKIP_TASKBAR)) {
-	    windows_list.push_back(client_active->getWindow());
-	}
+        if (client_active && ! client_active->isSkip(SKIP_TASKBAR)) {
+            windows.push_back(client_active->getWindow());
+        }
     }
 
-    num_windows = windows_list.size();
-    Window *windows = new Window[num_windows ? num_windows : 1];
+    num_windows = windows.size();
+    Window *wins = new Window[num_windows ? num_windows : 1];
     if (num_windows > 0) {
-      copy(windows_list.begin(), windows_list.end(), windows);
+        copy(windows.begin(), windows.end(), wins);
     }
 
-    return windows;
+    return wins;
 }
 
 /**
@@ -622,10 +601,8 @@ Workspaces::updateClientList(void)
     unsigned int num_windows;
     Window *windows = buildClientList(num_windows);
 
-    AtomUtil::setWindows(X11::getRoot(),
-			 Atoms::getAtom(NET_CLIENT_LIST),
-			 windows, num_windows);
-    
+    X11::setWindows(X11::getRoot(), NET_CLIENT_LIST, windows, num_windows);
+
     delete [] windows;
 }
 
@@ -638,9 +615,7 @@ Workspaces::updateClientStackingList(void)
     unsigned int num_windows;
     Window *windows = buildClientList(num_windows);
 
-    AtomUtil::setWindows(X11::getRoot(),
-			 Atoms::getAtom(NET_CLIENT_LIST_STACKING),
-			 windows, num_windows);
+    X11::setWindows(X11::getRoot(), NET_CLIENT_LIST_STACKING, windows, num_windows);
 
     delete [] windows;
 }
@@ -651,32 +626,46 @@ Workspaces::updateClientStackingList(void)
 void
 Workspaces::placeWo(PWinObj *wo, Window parent)
 {
-    bool placed = false;
+    Geometry head;
+    int i, head_nr = X11::getCurrHead();
+    vector<bool> fsHead(X11::getNumHeads(), false);
 
-    list<uint>::iterator it(Config::instance()->getPlacementModelBegin());
-    for (; (placed != true) &&
-            (it != Config::instance()->getPlacementModelEnd()); ++it) {
-        switch (*it) {
-        case PLACE_SMART:
-            placed = placeSmart(wo);
-            break;
-        case PLACE_MOUSE_NOT_UNDER:
-            placed = placeMouseNotUnder(wo);
-            break;
-        case PLACE_MOUSE_CENTERED:
-            placed = placeMouseCentered(wo);
-            break;
-        case PLACE_MOUSE_TOP_LEFT:
-            placed = placeMouseTopLeft(wo);
-            break;
-        case PLACE_CENTERED_ON_PARENT:
-            placed = placeCenteredOnParent(wo, parent);
-            break;
-        default:
-            // do nothing
-            break;
+    // Collect the information which head has a fullscreen window.
+    // To be conservative for now we ignore fullscreen windows on
+    // the desktop or normal layer, because it might be a file
+    // manager in desktop mode, for example.
+    vector<PWinObj*>::const_iterator it(_wobjs.begin()), end(_wobjs.end());
+    for (; it != end; ++it) {
+        if ((*it)->isMapped() && (*it)->getType() == PWinObj::WO_FRAME) {
+            Client *client = static_cast<Frame*>((*it))->getActiveClient();
+            if (client && client->isFullscreen()
+                       && client->getLayer()>LAYER_NORMAL) {
+                fsHead[client->getHead()] = true;
+            }
         }
     }
+
+    // Try to place the window
+    i = head_nr;
+    do {
+        if (! fsHead[i] && placeWo(wo, i, parent)) {
+            return;
+        }
+        i = (i+1)%X11::getNumHeads();
+    } while (i != head_nr);
+
+    // We failed to place the window, so put it in the top-left
+    // corner but still try to avoid a head with fullscreen window on.
+    i = head_nr;
+    do {
+        if (! fsHead[i]) {
+            break;
+        }
+        i = (i+1)%X11::getNumHeads();
+    } while (i != head_nr);
+
+    X11::getHeadInfoWithEdge(i, head);
+    wo->move(head.x, head.y);
 }
 
 /**
@@ -696,17 +685,51 @@ Workspaces::placeWoInsideScreen(PWinObj *wo)
         }
     }
 
-    placeInsideScreen(gm_after, strut);
+    X11::placeInsideScreen(gm_after, strut);
     if (gm_before != gm_after) {
         wo->move(gm_after.x, gm_after.y);
     }
 }
 
+bool
+Workspaces::placeWo(PWinObj *wo, uint head_num, Window parent)
+{
+    bool placed = false;
+    Geometry head;
+    X11::getHeadInfoWithEdge(head_num, head);
+
+    vector<uint>::const_iterator it(Config::instance()->getPlacementModelBegin());
+    for (; !placed && it != Config::instance()->getPlacementModelEnd(); ++it) {
+        switch (static_cast<enum PlacementModel>(*it)) {
+        case PLACE_SMART:
+            placed = placeSmart(wo, head);
+            break;
+        case PLACE_MOUSE_NOT_UNDER:
+            placed = placeMouseNotUnder(wo, head);
+            break;
+        case PLACE_MOUSE_CENTERED:
+            placed = placeMouseCentered(wo);
+            break;
+        case PLACE_MOUSE_TOP_LEFT:
+            placed = placeMouseTopLeft(wo);
+            break;
+        case PLACE_CENTERED_ON_PARENT:
+            placed = placeCenteredOnParent(wo, parent);
+            break;
+        default:
+            placed = false;
+            break;
+        }
+    }
+
+    return placed;
+}
+
 //! @brief Tries to find empty space to place the client in
 //! @return true if client got placed, else false
 //! @todo What should we do about Xinerama as when we don't have it enabled we care about the struts.
 bool
-Workspaces::placeSmart(PWinObj* wo)
+Workspaces::placeSmart(PWinObj* wo, const Geometry &head)
 {
     PWinObj *wo_e;
     bool placed = false;
@@ -725,9 +748,6 @@ Workspaces::placeSmart(PWinObj* wo)
     uint wo_width = wo->getWidth() + Config::instance()->getPlacementOffsetX();
     uint wo_height = wo->getHeight() + Config::instance()->getPlacementOffsetY();
 
-    Geometry head;
-    X11::getHeadInfoWithEdge(X11::getCurrHead(), head);
-
     start_x = (Config::instance()->getPlacementLtR())
               ? (head.x)
               : (head.x + head.width - wo_width);
@@ -785,11 +805,8 @@ Workspaces::placeSmart(PWinObj* wo)
 
 //! @brief Places the wo in a corner of the screen not under the pointer
 bool
-Workspaces::placeMouseNotUnder(PWinObj *wo)
+Workspaces::placeMouseNotUnder(PWinObj *wo, const Geometry &head)
 {
-    Geometry head;
-    X11::getHeadInfoWithEdge(X11::getCurrHead(), head);
-
     int mouse_x, mouse_y;
     X11::getMousePosition(mouse_x, mouse_y);
 
@@ -831,8 +848,8 @@ Workspaces::placeMouseCentered(PWinObj *wo)
     Geometry gm(mouse_x - (wo->getWidth() / 2), mouse_y - (wo->getHeight() / 2),
                 wo->getWidth(), wo->getHeight());
 
-    // make sure it's within the screens border
-    placeInsideScreen(gm);
+    // make sure it's within the screen's border
+    X11::placeInsideScreen(gm);
 
     wo->move(gm.x, gm.y);
 
@@ -847,7 +864,7 @@ Workspaces::placeMouseTopLeft(PWinObj *wo)
     X11::getMousePosition(mouse_x, mouse_y);
 
     Geometry gm(mouse_x, mouse_y, wo->getWidth(), wo->getHeight());
-    placeInsideScreen(gm); // make sure it's within the screens border
+    X11::placeInsideScreen(gm); // make sure it's within the screen's border
 
     wo->move(gm.x, gm.y);
 
@@ -872,34 +889,6 @@ Workspaces::placeCenteredOnParent(PWinObj *wo, Window parent)
     return false;
 }
 
-//! @brief Makes sure the window is inside the screen.
-void
-Workspaces::placeInsideScreen(Geometry &gm, Strut *strut)
-{
-    // Do not include screen edges when calculating the position if the window
-    // has a strut as it then is likely to be a panel or the like placed
-    // along the edge of the screen.
-    Geometry head;
-    if (strut) {
-        X11::getHeadInfo(X11::getCurrHead(), head);
-    } else {
-        X11::getHeadInfoWithEdge(X11::getCurrHead(), head);
-    }
-
-    if (gm.x < head.x) {
-        gm.x = head.x;
-    } else if ((gm.x + gm.width) > (head.x + head.width)) {
-        gm.x = head.x + head.width - gm.width;
-    }
-
-    if (gm.y < head.y) {
-        gm.y = head.y;
-    } else if ((gm.y + gm.height) > (head.y + head.height)) {
-        gm.y = head.y + head.height - gm.height;
-    }
-}
-
-//! @brief
 PWinObj*
 Workspaces::isEmptySpace(int x, int y, const PWinObj* wo)
 {
@@ -908,8 +897,8 @@ Workspaces::isEmptySpace(int x, int y, const PWinObj* wo)
     }
 
     // say that it's placed, now check if we are wrong!
-    list<PWinObj*>::iterator it(_wo_list.begin());
-    for (; it != _wo_list.end(); ++it) {
+    const_iterator it(_wobjs.begin());
+    for (; it != _wobjs.end(); ++it) {
         // Skip ourselves, non-mapped and desktop objects. Iconified means
         // skip placement.
         if (wo == (*it) || ! (*it)->isMapped() || (*it)->isIconified()
@@ -921,9 +910,7 @@ Workspaces::isEmptySpace(int x, int y, const PWinObj* wo)
         // automatically fail.
         if ((*it)->getType() == PWinObj::WO_FRAME) {
             Client *client = static_cast<Frame*>((*it))->getActiveClient();
-            if (client &&
-                (client->isFullscreen()
-                 || (client->isMaximizedVert() && client->isMaximizedHorz()))) {
+            if (client && client->isMaximizedVert() && client->isMaximizedHorz()) {
                 continue;
             }
         }
@@ -947,90 +934,75 @@ Workspaces::isEmptySpace(int x, int y, const PWinObj* wo)
 PWinObj*
 Workspaces::findDirectional(PWinObj *wo, DirectionType dir, uint skip)
 {
+    struct Triplet {
+        Triplet() : x(0), y(0), wo(0) { }
+        int x, y;
+        PWinObj *wo;
+        bool lessX(const Triplet &p) {
+            return x<p.x || (x==p.x && y<p.y) || (x==p.x && y==p.y && wo < p.wo);
+        }
+        bool lessY(const Triplet &p) {
+            return y<p.y || (y==p.y && x<p.x) || (x==p.x && y==p.y && wo < p.wo);
+        }
+    } S, T, N; // Start, Temp, Next
+
     // search from the frame not client, if client
     if (wo->getType() == PWinObj::WO_CLIENT) {
         wo = static_cast<Client*>(wo)->getParent();
     }
 
-    PWinObj *found_wo = 0;
+    S.x = wo->getX()+wo->getWidth()/2;
+    S.y = wo->getY()+wo->getHeight()/2;
+    S.wo = N.wo = wo;
 
-    uint score = 0, score_min;
-    int wo_main, wo_sec;
-    int diff_main = 0;
-
-#ifdef HAVE_LIMITS
-    score_min = numeric_limits<uint>::max();
-#else // !HAVE_LIMITS
-    score_min = ~0;
-#endif // HAVE_LIMITS
-
-    // init wo variables
-    if ((dir == DIRECTION_UP) || (dir == DIRECTION_DOWN)) {
-        wo_main = wo->getY() + wo->getHeight() / 2;
-        wo_sec = wo->getX() + wo->getWidth() / 2;
-    } else {
-        wo_main = wo->getX() + wo->getWidth() / 2;
-        wo_sec = wo->getY() + wo->getHeight() / 2;
-    }
-
-    list<PWinObj*>::iterator it(_wo_list.begin());
-    for (; it != _wo_list.end(); ++it) {
-        if ((wo == (*it)) || ! ((*it)->isMapped())) {
-            continue; // skip ourselves and unmapped wo's
+    const_iterator it(_wobjs.begin());
+    for (; it != _wobjs.end(); ++it) {
+        T.wo = *it;
+        if (wo == T.wo) {
+            continue;
         }
-        if (((*it)->getType() != PWinObj::WO_FRAME) ||
-                static_cast<Frame*>(*it)->isSkip(skip)) {
+        if (T.wo->getType() != PWinObj::WO_FRAME ||
+                static_cast<Frame*>(T.wo)->isSkip(skip) ||
+                ! T.wo->isMapped()) {
             continue; // only include frames and not having skip set
         }
+        T.x = T.wo->getX()+T.wo->getWidth()/2;
+        T.y = T.wo->getY()+T.wo->getHeight()/2;
+
+        if (N.wo == S.wo) {
+            N = T;
+            continue;
+        }
 
-        // check main direction, making sure it's at the right side
-        // we check against the middle of the window as it gives a saner feeling
-        // than the edges IMHO
         switch (dir) {
         case DIRECTION_UP:
-            diff_main = wo_main - ((*it)->getY() + (*it)->getHeight() / 2);
+            if ( (N.lessY(S) && T.lessY(S) && N.lessY(T)) ||
+                 (!N.lessY(S) && (T.lessY(S) || N.lessY(T))) ) {
+                N = T;
+            }
             break;
         case DIRECTION_DOWN:
-            diff_main = ((*it)->getY() + (*it)->getHeight() / 2) - wo_main;
+            if ( (S.lessY(N) && S.lessY(T) && T.lessY(N)) ||
+                 (!S.lessY(N) && (S.lessY(T) || T.lessY(N))) ) {
+                N = T;
+            }
             break;
         case DIRECTION_LEFT:
-            diff_main = wo_main - ((*it)->getX() + (*it)->getWidth() / 2);
+            if ( (N.lessX(S) && T.lessX(S) && N.lessX(T)) ||
+                 (!N.lessX(S) && (T.lessX(S) || N.lessX(T))) ) {
+                N = T;
+            }
             break;
         case DIRECTION_RIGHT:
-            diff_main = ((*it)->getX() + (*it)->getWidth() / 2) - wo_main;
+            if ( (S.lessX(N) && S.lessX(T) && T.lessX(N)) ||
+                 (!S.lessX(N) && (S.lessX(T) || T.lessX(N))) ) {
+                N = T;
+            }
             break;
         default:
-            return 0; // no direction to search
-        }
-
-        if (diff_main < 0) {
-            continue; // wrong direction
-        }
-
-        score = diff_main;
-
-        if ((dir == DIRECTION_UP) || (dir == DIRECTION_DOWN)) {
-            if ((wo_sec < (*it)->getX()) || (wo_sec > (*it)->getRX())) {
-                score += X11::getHeight() / 2;
-            }
-            score += abs (static_cast<long> (wo_sec - ((*it)->getX ()
-                                             + (*it)->getWidth () / 2)));
-
-        } else {
-            if ((wo_sec < (*it)->getY()) || (wo_sec > (*it)->getBY())) {
-                score += X11::getWidth() / 2;
-            }
-
-            score += abs (static_cast<long> (wo_sec - ((*it)->getY ()
-                                             + (*it)->getHeight () / 2)));
-        }
-
-        if (score < score_min) {
-            found_wo = *it;
-            score_min = score;
+            break;
         }
     }
-
-    return found_wo;
+    return N.wo;
 }
 
