@@ -22,8 +22,8 @@ using std::numeric_limits;
 
 extern "C" {
 #include <X11/Xlib.h>
-#ifdef HAVE_SHAPE
 #include <X11/Xutil.h>
+#ifdef HAVE_SHAPE
 #include <X11/extensions/shape.h>
 #endif // HAVE_SHAPE
 #ifdef HAVE_XRANDR
@@ -49,8 +49,6 @@ unsigned int xerrors_count = 0;
 using std::cerr;
 using std::endl;
 using std::vector;
-using std::list;
-using std::map;
 using std::string;
 using std::memset; // required for FD_ZERO
 
@@ -59,6 +57,7 @@ const uint X11::MODIFIER_TO_MASK[] = {
     Mod1Mask, Mod2Mask, Mod3Mask, Mod4Mask, Mod5Mask
 };
 const uint X11::MODIFIER_TO_MASK_NUM = sizeof(X11::MODIFIER_TO_MASK) / sizeof(X11::MODIFIER_TO_MASK[0]);
+Atom X11::_atoms[MAX_NR_ATOMS];
 
 extern "C" {
     /**
@@ -83,38 +82,98 @@ extern "C" {
     }
 }
 
-//! @brief X11::Visual constructor.
-//! @param x_visual X Visual to wrap.
-X11::PVisual::PVisual(Visual *x_visual) : _x_visual(x_visual),
-        _r_shift(0), _r_prec(0),
-        _g_shift(0), _g_prec(0),
-        _b_shift(0), _b_prec(0)
-{
-    getShiftPrecFromMask(_x_visual->red_mask, _r_shift, _r_prec);
-    getShiftPrecFromMask(_x_visual->green_mask, _g_shift, _g_prec);
-    getShiftPrecFromMask(_x_visual->blue_mask, _b_shift, _b_prec);
-}
+static const char *atomnames[] = {
+    // EWMH atoms
+    "_NET_SUPPORTED",
+    "_NET_CLIENT_LIST", "_NET_CLIENT_LIST_STACKING",
+    "_NET_NUMBER_OF_DESKTOPS",
+    "_NET_DESKTOP_GEOMETRY", "_NET_DESKTOP_VIEWPORT",
+    "_NET_CURRENT_DESKTOP", "_NET_DESKTOP_NAMES",
+    "_NET_ACTIVE_WINDOW", "_NET_WORKAREA",
+    "_NET_DESKTOP_LAYOUT", "_NET_SUPPORTING_WM_CHECK",
+    "_NET_CLOSE_WINDOW",
+    "_NET_WM_NAME", "_NET_WM_VISIBLE_NAME",
+    "_NET_WM_ICON_NAME", "_NET_WM_VISIBLE_ICON_NAME",
+    "_NET_WM_ICON", "_NET_WM_DESKTOP",
+    "_NET_WM_STRUT", "_NET_WM_PID",
+    "_NET_WM_WINDOW_OPACITY",
 
-//! @brief X11::Visual destructor.
-X11::PVisual::~PVisual(void)
-{
-}
+    "_NET_WM_WINDOW_TYPE",
+    "_NET_WM_WINDOW_TYPE_DESKTOP", "_NET_WM_WINDOW_TYPE_DOCK",
+    "_NET_WM_WINDOW_TYPE_TOOLBAR", "_NET_WM_WINDOW_TYPE_MENU",
+    "_NET_WM_WINDOW_TYPE_UTILITY", "_NET_WM_WINDOW_TYPE_SPLASH",
+    "_NET_WM_WINDOW_TYPE_DIALOG", "_NET_WM_WINDOW_TYPE_NORMAL",
 
-//! @brief Gets shift and prec from mask.
-//! @param mask red,green,blue mask of Visual.
-//! @param shift Set to the shift of mask.
-//! @param prec Set to the prec of mask.
-void
-X11::PVisual::getShiftPrecFromMask(ulong mask, int &shift, int &prec)
-{
-    for (shift = 0; ! (mask&0x1); ++shift) {
-        mask >>= 1;
+    "_NET_WM_STATE",
+    "_NET_WM_STATE_MODAL", "_NET_WM_STATE_STICKY",
+    "_NET_WM_STATE_MAXIMIZED_VERT", "_NET_WM_STATE_MAXIMIZED_HORZ",
+    "_NET_WM_STATE_SHADED",
+    "_NET_WM_STATE_SKIP_TASKBAR", "_NET_WM_STATE_SKIP_PAGER",
+    "_NET_WM_STATE_HIDDEN", "_NET_WM_STATE_FULLSCREEN",
+    "_NET_WM_STATE_ABOVE", "_NET_WM_STATE_BELOW",
+    "_NET_WM_STATE_DEMANDS_ATTENTION",
+
+    "_NET_WM_ALLOWED_ACTIONS",
+    "_NET_WM_ACTION_MOVE", "_NET_WM_ACTION_RESIZE",
+    "_NET_WM_ACTION_MINIMIZE", "_NET_WM_ACTION_SHADE",
+    "_NET_WM_ACTION_STICK",
+    "_NET_WM_ACTION_MAXIMIZE_VERT", "_NET_WM_ACTION_MAXIMIZE_HORZ",
+    "_NET_WM_ACTION_FULLSCREEN", "_NET_WM_ACTION_CHANGE_DESKTOP",
+    "_NET_WM_ACTION_CLOSE",
+    "UTF8_STRING", // When adding an ewmh atom after this,
+                   // fix setEwmhAtomsSupport(Window)
+    "STRING", "MANAGER",
+
+    // pekwm atoms
+    "_PEKWM_FRAME_ID",
+    "_PEKWM_FRAME_ORDER",
+    "_PEKWM_FRAME_ACTIVE",
+    "_PEKWM_FRAME_DECOR",
+    "_PEKWM_FRAME_SKIP",
+    "_PEKWM_TITLE",
+
+    // ICCCM atoms
+    "WM_NAME",
+    "WM_ICON_NAME",
+    "WM_HINTS",
+    "WM_CLASS",
+    "WM_STATE",
+    "WM_CHANGE_STATE",
+    "WM_PROTOCOLS",
+    "WM_DELETE_WINDOW",
+    "WM_COLORMAP_WINDOWS",
+    "WM_TAKE_FOCUS",
+    "WM_WINDOW_ROLE",
+    "WM_CLIENT_MACHINE",
+
+    // miscellaneous atoms
+    "_MOTIF_WM_HINTS"
+};
+
+/**
+ * Helper class for XColor.
+ */
+
+class X11::ColorEntry {
+public:
+    ColorEntry(const std::string &name) : _name(name), _ref(0) { }
+    ~ColorEntry(void) { }
+
+    inline XColor *getColor(void) { return &_xc; }
+
+    inline uint getRef(void) const { return _ref; }
+    inline void incRef(void) { _ref++; }
+    inline void decRef(void) { if (_ref > 0) { _ref--; } }
+
+    inline bool operator==(const std::string &name) {
+        return (::strcasecmp(_name.c_str(), name.c_str()) == 0);
     }
-    
-    for (prec = 0; (mask&0x1); ++prec) {
-        mask >>= 1;
-    }
-}
+
+private:
+    std::string _name;
+    XColor _xc;
+    uint _ref;
+};
 
 //! @brief X11 constructor
 void
@@ -137,7 +196,7 @@ X11::init(Display *dpy, bool honour_randr)
     _root = RootWindow(_dpy, _screen);
 
     _depth = DefaultDepth(_dpy, _screen);
-    _visual = new X11::PVisual(DefaultVisual(_dpy, _screen));
+    _visual = DefaultVisual(_dpy, _screen);
     _colormap = DefaultColormap(_dpy, _screen);
     _modifier_map = XGetModifierMapping(_dpy);
 
@@ -181,12 +240,29 @@ X11::init(Display *dpy, bool honour_randr)
 
     XSync(_dpy, false);
     XUngrabServer(_dpy);
+
+    _xc_default.pixel = BlackPixel(_dpy, _screen);
+    _xc_default.red = _xc_default.green = _xc_default.blue = 0;
+
+    assert(sizeof(atomnames)/sizeof(char*) == MAX_NR_ATOMS);
+    if (! XInternAtoms(_dpy, const_cast<char**>(atomnames), MAX_NR_ATOMS, 0, _atoms)) {
+        std::cerr << "Error: XInternAtoms didn't return all requested atoms." << std::endl;
+    }
 }
 
 //! @brief X11 destructor
 void
 X11::destruct(void) {
-    delete _visual;
+    if (_colours.size() > 0) {
+        ulong *pixels = new ulong[_colours.size()];
+        for (uint i=0; i < _colours.size(); ++i) {
+            pixels[i] = _colours[i]->getColor()->pixel;
+            delete _colours[i];
+        }
+        XFreeColors(_dpy, X11::getColormap(),
+                    pixels, _colours.size(), 0);
+        delete [] pixels;
+    }
 
     if (_modifier_map) {
         XFreeModifiermap(_modifier_map);
@@ -201,6 +277,71 @@ X11::destruct(void) {
     _dpy = 0;
 }
 
+XColor *
+X11::getColor(const std::string &color)
+{
+    // check for already existing entry
+    vector<ColorEntry*>::const_iterator it(_colours.begin());
+
+    if (strcasecmp(color.c_str(), "EMPTY") == 0) {
+        return &_xc_default;
+    }
+
+    for (; it != _colours.end(); ++it) {
+        if (*(*it) == color) {
+            (*it)->incRef();
+            return (*it)->getColor();
+        }
+    }
+
+    // create new entry
+    ColorEntry *entry = new ColorEntry(color);
+
+    // X alloc
+    XColor dummy;
+    if (XAllocNamedColor(_dpy, X11::getColormap(),
+                         color.c_str(), entry->getColor(), &dummy) == 0) {
+#ifdef DEBUG
+        cerr << __FILE__ << "@" << __LINE__ << ": "
+             << "X11::getColor(" << color << ")" << endl
+             << " *** failed to alloc color: " << color << endl;
+#endif // DEBUG
+        delete entry;
+        entry = 0;
+    } else {
+        _colours.push_back(entry);
+        entry->incRef();
+    }
+
+    return entry ? entry->getColor() : &_xc_default;
+}
+
+void
+X11::returnColor(XColor *xc)
+{
+    bool _free_on_return = false;
+
+    if (&_xc_default == xc) { // no need to return default color
+        return;
+    }
+
+    vector<ColorEntry*>::iterator it(_colours.begin());
+
+    for (; it != _colours.end(); ++it) {
+        if ((*it)->getColor() == xc) {
+            (*it)->decRef();
+            if (_free_on_return && ((*it)->getRef() == 0)) {
+                ulong pixels[1] = { (*it)->getColor()->pixel };
+                XFreeColors(X11::getDpy(), X11::getColormap(), pixels, 1, 0);
+
+                delete *it;
+                _colours.erase(it);
+            }
+            break;
+        }
+    }
+}
+
 /**
  * Figure out what keys the Num and Scroll Locks are
  */
@@ -459,9 +600,9 @@ X11::getHeadInfo(uint head, Geometry &head_info)
 Geometry
 X11::getHeadGeometry(uint head)
 {
-  Geometry gm(_screen_gm);
-  getHeadInfo(head, gm);
-  return gm;
+    Geometry gm(_screen_gm);
+    getHeadInfo(head, gm);
+    return gm;
 }
 
 //! @brief Fill information about head and the strut.
@@ -491,6 +632,87 @@ X11::getHeadInfoWithEdge(uint num, Geometry &head)
     head.height -= strut_val;
 }
 
+bool
+X11::getProperty(Window win, AtomName aname, Atom type,
+            ulong expected, uchar **data, ulong *actual)
+{
+    Atom r_type;
+    int r_format, status;
+    ulong read = 0, left = 0;
+
+    *data = 0;
+    do {
+        if (*data) {
+            XFree(*data);
+            *data = 0;
+        }
+        expected += left;
+
+        status =
+            XGetWindowProperty(_dpy, win, _atoms[aname],
+                               0L, expected, False, type,
+                               &r_type, &r_format, &read, &left, data);
+
+        if (status != Success || type != r_type || read == 0) {
+            if (*data) {
+                XFree(*data);
+                *data = 0;
+            }
+            left = 0;
+        }
+    } while (left);
+
+    if (actual) {
+        *actual = read;
+    }
+
+    return (*data != 0);
+}
+
+bool
+X11::getTextProperty(Window win, Atom atom, std::string &value)
+{
+    // Read text property, return if it fails.
+    XTextProperty text_property;
+    if (! XGetTextProperty(_dpy, win, &text_property, atom)
+        || ! text_property.value || ! text_property.nitems) {
+        return false;
+    }
+
+    if (text_property.encoding == XA_STRING) {
+        value = reinterpret_cast<const char*>(text_property.value);
+    } else {
+        char **mb_list;
+        int num;
+
+        XmbTextPropertyToTextList(_dpy, &text_property, &mb_list, &num);
+        if (mb_list && num > 0) {
+            value = *mb_list;
+            XFreeStringList(mb_list);
+        }
+    }
+
+    XFree(text_property.value);
+
+    return true;
+}
+
+//! @brief
+void*
+X11::getEwmhPropData(Window win, AtomName prop, Atom type, int &num)
+{
+    Atom type_ret;
+    int format_ret;
+    ulong items_ret, after_ret;
+    uchar *prop_data = 0;
+
+    XGetWindowProperty(_dpy, win, _atoms[prop], 0, 0x7fffffff,
+                       False, type, &type_ret, &format_ret, &items_ret,
+                       &after_ret, &prop_data);
+    num = items_ret;
+    return prop_data;
+}
+
 void
 X11::getMousePosition(int &x, int &y)
 {
@@ -525,7 +747,7 @@ void
 X11::addStrut(Strut *strut)
 {
     assert(strut);
-    _strut_list.push_back(strut);
+    _struts.push_back(strut);
 
     updateStrut();
 }
@@ -535,7 +757,9 @@ void
 X11::removeStrut(Strut *strut)
 {
     assert(strut);
-    _strut_list.remove(strut);
+    vector<Strut *>::iterator it(find(_struts.begin(), _struts.end(), strut));
+    if (it != _struts.end())
+        _struts.erase(it);
 
     updateStrut();
 }
@@ -551,14 +775,14 @@ X11::updateStrut(void)
     _strut.bottom = 0;
 
     for (vector<Head>::iterator it(_heads.begin()); it != _heads.end(); ++it) {
-      it->strut.left = 0;
-      it->strut.right = 0;
-      it->strut.top = 0;
-      it->strut.bottom = 0;
+        it->strut.left = 0;
+        it->strut.right = 0;
+        it->strut.top = 0;
+        it->strut.bottom = 0;
     }
 
     Strut *strut;
-    for(list<Strut*>::iterator it(_strut_list.begin()); it != _strut_list.end(); ++it) {
+    for(vector<Strut*>::iterator it(_struts.begin()); it != _struts.end(); ++it) {
         if ((*it)->head < 0) {
             strut = &_strut;
         } else if (static_cast<uint>((*it)->head) < _heads.size()) {
@@ -734,6 +958,8 @@ X11::getKeycodeFromMask(uint mask)
 /**
  * Wrapper for XKeycodeToKeysym and XkbKeycodeToKeysym depending on which one is available.
  */
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wdeprecated-declarations"
 KeySym
 X11::getKeysymFromKeycode(KeyCode keycode)
 {
@@ -744,6 +970,7 @@ X11::getKeysymFromKeycode(KeyCode keycode)
 #endif
         return XKeycodeToKeysym(_dpy, keycode, 0);
 }
+#pragma GCC diagnostic pop
 
 Display *X11::_dpy;
 bool X11::_honour_randr = false;
@@ -752,7 +979,7 @@ int X11::_screen = -1;
 int X11::_depth = -1;
 Geometry X11::_screen_gm;
 Window X11::_root = None;
-X11::PVisual *X11::_visual;
+Visual *X11::_visual = 0;
 Colormap X11::_colormap = None;
 XModifierKeymap *X11::_modifier_map;
 bool X11::_has_extension_shape = false;
@@ -769,4 +996,6 @@ Time X11::_last_event_time;
 Window X11::_last_click_id = None;
 Time X11::_last_click_time[BUTTON_NO - 1];
 Strut X11::_strut;
-std::list<Strut*> X11::_strut_list;
+vector<Strut*> X11::_struts;
+vector<X11::ColorEntry*> X11::_colours;
+XColor X11::_xc_default;
